#ifndef CENTROIDSDLG_H
#define CENTROIDSDLG_H

#include <QDialog>
#include <QVector>
#include <QStandardItemModel>
#include <QStandardItem>
#include <QItemSelection>
#include <QOpenGLFunctions>

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif

class jav_centroid_type {
    public:
                 float xyz[3];
                 float xyzN[3];
                 int   nat;
                 int   *atoms;
                 bool  visible;
                 int   transparency;
                 QColor colore;
                 bool  showlab;
                 QString label;
                 bool  selected;
                 static float radN;
                 void DrawCentroid(QOpenGLFunctions *f, QColor c);
                 void Normalizza();
#if defined(JAV_WIDGET)
                 static void setJav(CrystalViewWidget *w) {m_jav=w;}
                 static CrystalViewWidget *m_jav;
#else
                 static void setJav(JavWindow *w) {m_jav=w;}
                 static JavWindow *m_jav;
#endif
private:
                 void DrawSelected(float add);
                 void Draw(QOpenGLFunctions *f);
                 void DrawLabel(QColor c);
};

namespace Ui {
class CentroidsDlg;
}

#include <QStyledItemDelegate>

class CenterDelegate: public QStyledItemDelegate{
public:
    using QStyledItemDelegate::QStyledItemDelegate;
protected:
    void initStyleOption(QStyleOptionViewItem *option, const QModelIndex &index) const override
    {
        QStyledItemDelegate::initStyleOption(option, index);
        option->displayAlignment = Qt::AlignHCenter | Qt::AlignVCenter;
    }
};

class CentroidsDlg : public QDialog
{
    Q_OBJECT

public:
    explicit CentroidsDlg(QWidget *parent = 0);
    ~CentroidsDlg();
    void clearCentroids(int whichModel);
    void clearAll();
    void addCentroid(int whichModel, jav_centroid_type c);
    void FillList();
    void DrawCentroids(int kmodel, QOpenGLFunctions *f, QColor c);
    void NormalizzaCentroids(int kmodel);

private:
    Ui::CentroidsDlg *ui;
    QStandardItemModel *centModel;
    QVector <jav_centroid_type> *AllCentr;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif

private slots:
    void on_findButton_clicked();
    void on_closeButton_clicked();
    void colorChanged(QColor newColor, int k);
    void alphaChanged(int value, int k);
    void tableCentroids_dataChanged(const QModelIndex & current, const QModelIndex &);
    void rowSelectionChanged(const QItemSelection &, const QItemSelection &);

signals:
    void centroidChanged();
};

#endif // CENTROIDSDLG_H
