#include "crystaldata.h"

#include <QDebug>

extern "C" void crystal_phase_name(int idph,char *phase_name);
extern "C" int crystal_phase_name_len(int idph);
extern "C" int scatters_number(int idph);
extern "C" void get_scatter_info(int idph, int kscatt, char *lab);
extern "C" int build_restraints_list(int idph, int kaction);
extern "C" int nat_restraint(int idph, int idres);
extern "C" void c_restraint_info(int idph, int idres, int *code, int vat[], int vtype[], float *val, float *targ, float *sigma, int *active);
extern "C" void c_restraint_settings_info(int idph, float wei[], int *targetType);
extern "C" void c_put_restraint_info(int idph, int code, int *vat, int nat, float val, float targ, float sigma, int active);
extern "C" void c_put_restraint_settings(int idph, float wei[]);
extern "C" int constraints_number(int idph);
extern "C" int nat_constraint(int idph, int idc);
extern "C" void c_constraint_info(int idph, int idc, int *code, int vat[], int vpar[]);

namespace crydata {

QString crystalPhaseName(int idph) {
    int len = crystal_phase_name_len(idph)+1;
    auto name = new char[len];
    crystal_phase_name(idph, name);
    QString sname = name;
    delete [] name;
    return sname;
}

QVector<scatter_type> make_scatters(int phId)
{
    int nscat = scatters_number(phId);
    QVector<scatter_type> scatters(nscat);
    auto lab = new char[10];
    for (int i = 0; i < nscat; i++) {
        get_scatter_info(phId, i+1, lab);
        scatters[i].lab = QString::fromUtf8(lab);
    }
    delete [] lab;
    return scatters;
}

void get_restraints_settings(int phId, QVector<double> &wei, int &targetType) {
    float weires[3];
    c_restraint_settings_info(phId, weires, &targetType);
    for (int i = 0; i < 3; i++) wei[i] = weires[i];
}

void make_restraints(int phId, QVector<crydata::RestraintProperties> &restr, int kaction)
{
    auto scatters = crydata::make_scatters(phId);

    int nres = build_restraints_list(phId, kaction);
    restr.resize(nres);    
    for (int i = 0; i < nres; i++) {
        int natr = nat_restraint(phId, i+1);
        auto vat = new int[natr];
        auto vtype = new int[natr];
        int active;
        c_restraint_info(phId, i+1, &restr[i].code, vat, vtype,
                         &restr[i].val, &restr[i].targ, &restr[i].sigma, &active);
        restr[i].active = active;
        restr[i].atm.reserve(natr);
        restr[i].type.reserve(natr);
        for (int j = 0; j < natr; j++) {
            restr[i].atm.push_back(vat[j]);
            restr[i].type.push_back(scatters[vtype[j]].lab);
        }
        delete [] vat;
        delete [] vtype;
    }
}

void put_restraints(int phId, const QVector<crydata::RestraintProperties> &restr, const QVector<double> &wei)
{
    for (int i = 0; i < restr.size(); i++) {
        int natr = restr.at(i).atm.size();
        auto vat = new int[natr];
        for (int j = 0; j < natr; j++) vat[j] = restr.at(i).atm[j] + 1;
        c_put_restraint_info(phId, restr.at(i).code, vat, natr, restr.at(i).val,
                             restr.at(i).targ, restr.at(i).sigma, restr.at(i).active);

        delete [] vat;
    }

    auto wres = new float[3];
    for (int i = 0; i < 3; i++) wres[i] = static_cast<float>(wei[i]);
    c_put_restraint_settings(phId, wres);
}

void make_constraints(int phId, QVector<ConstraintProperties> &constr)
{
    int nconstr = constraints_number(phId);
    //qInfo() << "NCONSTR: " << nconstr;
    constr.resize(nconstr);
    for (int i = 0; i < nconstr; i++) {
        int nat = nat_constraint(phId, i+1);
        auto vat = new int[nat];
        auto vpar = new int[nat];
        //qInfo() << "  N: " << i << " NAT: " << nat;
        c_constraint_info(phId, i+1, &constr[i].code, vat, vpar);
        constr[i].atm.reserve(nat);
        constr[i].par.reserve(nat);
        for (int j = 0; j < nat; j++) {
            constr[i].atm.push_back(vat[j]);
            constr[i].par.push_back(vpar[j]);
        }
        //qInfo() << "PAR: " << constr[i].par;
        //qInfo() << "ATM: " << constr[i].atm;
        delete [] vat;
        delete [] vpar;
    }
}

int isParInConstraint(QVector<ConstraintProperties> &constr, int atom, int par) {

    for (int i = 0; i < constr.size(); i++) {
        for (int j = 0; j < constr[i].par.size(); j++) {
            if (constr[i].par[j] == par && constr[i].atm[j] == atom) {
                return i;
            }
        }
    }
    return -1;
}

int getAtomCode(const AtomProperties &atom)
{
    int fixed = 0;
    for (int i = 0; i < 3; i++) {
        if (atom.rcod[i] > 0)
            return 1;
        else if (atom.rcod[i] < 0)
            fixed++;
    }

    if (fixed == 3) return -1;

    return 0;
}

}

