#include "cubeobject.h"
#include "vertex.h"

#include <QOpenGLShaderProgram>

CubeObject::CubeObject() {}

void CubeObject::create(ShaderProgram &shader)
{
    // 3 vertices x 2 triangles x 6 faces = 36 vertices in a cube
    QVector<Vertex> vert{
                    // positions          // texture coords
        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D( 0.5f, -0.5f, -0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D( 0.5f,  0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D( 0.5f,  0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D(-0.5f,  0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 0.0f)},

        {QVector3D(-0.5f, -0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D( 0.5f, -0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D(-0.5f,  0.5f,  0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D(-0.5f, -0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},

        {QVector3D(-0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D(-0.5f,  0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D(-0.5f, -0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D(-0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},

        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D( 0.5f,  0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D( 0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D( 0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D( 0.5f, -0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},

        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D( 0.5f, -0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D( 0.5f, -0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D( 0.5f, -0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D(-0.5f, -0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D(-0.5f, -0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},

        {QVector3D(-0.5f,  0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)},
        {QVector3D( 0.5f,  0.5f, -0.5f),  QVector2D( 1.0f, 1.0f)},
        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D( 0.5f,  0.5f,  0.5f),  QVector2D( 1.0f, 0.0f)},
        {QVector3D(-0.5f,  0.5f,  0.5f),  QVector2D( 0.0f, 0.0f)},
        {QVector3D(-0.5f,  0.5f, -0.5f),  QVector2D( 0.0f, 1.0f)}
    };

    m_vao.create();
    m_vao.bind();

    m_vbo = QOpenGLBuffer(QOpenGLBuffer::VertexBuffer);
    m_vbo.create();
    m_vbo.setUsagePattern(QOpenGLBuffer::StaticDraw);
    m_vbo.bind();

    int vertMemSize = vert.size() * sizeof(Vertex);
    m_vbo.allocate(vert.data(), vertMemSize);

    // Position attribute
    shader.program()->enableAttributeArray(0);
    shader.program()->setAttributeBuffer(0, GL_FLOAT, 0, 3, 5*sizeof(float));

    // Texture coord attribute
    shader.program()->enableAttributeArray(1);
    shader.program()->setAttributeBuffer(1, GL_FLOAT, 3*sizeof(float), 2, 5*sizeof(float));

    m_vbo.release();
    m_vao.release();

    for (int j = 0; j < 6; ++j) {
        textures[j] = new QOpenGLTexture(QImage(QString("://textures/splash_tex%1.png").arg(j + 1)).mirrored());
        textures[j]->setWrapMode(QOpenGLTexture::Repeat);
        textures[j]->setMinificationFilter(QOpenGLTexture::LinearMipMapLinear);
        textures[j]->setMagnificationFilter(QOpenGLTexture::Linear);
    }

    // tell opengl for each sampler to which texture unit it belongs to (only has to be done once)
    shader.use();
    shader.setInt("texture1", 0);
}

void CubeObject::destroy()
{
    m_vao.destroy();
    m_vbo.destroy();
    delete texture1;
    for (int i = 0; i < 6; ++i)
        delete textures[i];
}

void CubeObject::render(ShaderProgram &shader)
{
    //glActiveTexture(GL_TEXTURE0);

    m_vao.bind();
    int i = 0;
    for (QVector3D &pos : cubePositions) {        
        textures[i%6]->bind();
        QMatrix4x4 model;
        model.setToIdentity();
        model.translate(pos);
        float angle = 20.0f * i;
        i++;
        model.rotate(angle, 1.0f, 0.3f, 0.5f);
        shader.setMat4("model", model);
        glDrawArrays(GL_TRIANGLES, 0, 36);
    }
    //m_vao.release(); // Actually, not necessary!
}
