#include "formulaeditor.h"

extern "C" int ValidateFormula(const char *formula, int lenformula);
extern "C" int ValidateFormulaInfo(const char *formula, int lenformula, double volume, double info[]);

FormulaEditor::FormulaEditor(QWidget *parent) :
    QLineEdit(parent),
    infoEnabled(false),
    cellVolume(-1.0),
    valid(false)
{
    connect(this, &QLineEdit::textChanged, [=]() {
        validateFormula();

        QPalette palette = this->palette();
        if (valid) {
            palette.setColor(QPalette::Text, Qt::black);
            this->setPalette(palette);
        } else {
            palette.setColor(QPalette::Text, Qt::red);
            this->setPalette(palette);
        }

        emit formulaChanged();
    });
}

void FormulaEditor::validateFormula()
{
    if (infoEnabled) {
        double info[4];
        valid = ValidateFormulaInfo(this->text().toStdString().c_str(), this->text().length(), cellVolume, info);
        vol = info[0];
        volPerAtom = info[1];
        dens = info[2];
        mw = info[3];
    } else {
        valid = ValidateFormula(this->text().toStdString().c_str(), this->text().length());
    }
}

double FormulaEditor::volume() const
{
    return vol;
}

double FormulaEditor::volumePerAtom() const
{
    return volPerAtom;
}

double FormulaEditor::density() const
{
    return dens;
}

double FormulaEditor::MW() const
{
    return mw;
}

bool FormulaEditor::isValid() const
{
    return valid;
}

void FormulaEditor::setCellVolume(double value)
{
    cellVolume = value;
}

void FormulaEditor::setInfoEnabled(bool enabled, double cellVol)
{
    infoEnabled = enabled;
    cellVolume = cellVol;
}
