module gopt_interop
   use iso_c_binding
   use simulated_anneal, only: SimAnn_Conditions_type
   use bb_bc, only: bb_bc_condition

   implicit none

   type, bind(C) :: c_gopt_info_type
     real(c_float)                :: res
     real(c_float)                :: resmin
     real(c_float)                :: resmax
     integer(c_int)               :: cftype
     integer(c_int)               :: nrun
     integer(c_int)               :: seed
     integer(c_int)               :: algo
     integer(c_int), dimension(8) :: cfvet
     integer(c_int)               :: ncfvet
     integer(c_int)               :: nexport
   end type c_gopt_info_type

   type, bind(C) :: c_sa_conditions
     real(c_float)  :: t_ini
     real(c_float)  :: temp_fact
     integer(c_int) :: nt
     integer(c_int) :: autoti
     integer(c_int) :: autont
   end type c_sa_conditions

   type, bind(C) :: c_bb_conditions
     integer(c_int) :: niter
     integer(c_int) :: autont
   end type c_bb_conditions

   type, bind(C) :: solution_info
     integer(c_int) :: run
     real(c_float)  :: cf
   end type solution_info

   type(SimAnn_Conditions_type), private  :: sa_settings
   type(bb_bc_condition), private         :: bcond_settings

   contains

   elemental function cast_gopt_to_c(f_type) result(c_type)
   use progtype, only: opti_info_type
   type(opti_info_type), intent(in) :: f_type
   type(c_gopt_info_type)           :: c_type
!
   c_type%res = real(f_type%res,c_float)
   c_type%cftype = int(f_type%cftype,c_int)
   c_type%nrun = int(f_type%nrun,c_int)
   c_type%seed = int(f_type%seed,c_int)
   c_type%algo = int(f_type%algo,c_int)
   c_type%nexport = int(f_type%nexport,c_int)
!
   end function cast_gopt_to_c

! -----------------------------------------------------------------------------------------------------

   subroutine cast_gopt_to_f(c_type, f_type)
   use progtype, only: opti_info_type
   type(c_gopt_info_type), intent(in)  :: c_type
   type(opti_info_type), intent(inout) :: f_type
!
   f_type%res = real(c_type%res)
   f_type%cftype = int(c_type%cftype)
   f_type%nrun = int(c_type%nrun)
   f_type%seed = int(c_type%seed)
   f_type%algo = int(c_type%algo)
!
   end subroutine cast_gopt_to_f

! -----------------------------------------------------------------------------------------------------

   elemental function cast_sacond_to_c(f_type) result(c_type)
   use sa_type, only: SimAnn_Conditions_type
   use arrayutil, only: logical_to_int
   type(SimAnn_Conditions_type), intent(in) :: f_type
   type(c_sa_conditions)                    :: c_type
!
   c_type%t_ini     = real(f_type%t_ini,c_float)
   c_type%temp_fact = real(f_type%temp_fact,c_float)
   c_type%nt        = int(f_type%nt,c_int)
   c_type%autoti    = logical_to_int(f_type%autoti)
   c_type%autont    = logical_to_int(f_type%autont)
!
   end function cast_sacond_to_c

! -----------------------------------------------------------------------------------------------------

   subroutine cast_sacond_to_f(c_type,f_type)
   use sa_type, only: SimAnn_Conditions_type
   use arrayutil, only: int_to_logical
   type(c_sa_conditions), intent(in)           :: c_type
   type(SimAnn_Conditions_type), intent(inout) :: f_type
!
   f_type%t_ini     = real(c_type%t_ini)
   f_type%temp_fact = real(c_type%temp_fact)
   f_type%nt        = int(c_type%nt)
   f_type%autoti    = int_to_logical(c_type%autoti)
   f_type%autont    = int_to_logical(c_type%autont)
!
   end subroutine cast_sacond_to_f

! -----------------------------------------------------------------------------------------------------

   elemental function cast_bbcond_to_c(f_type) result(c_type)
   use bb_bc, only: bb_bc_condition
   use arrayutil, only: logical_to_int
   type(bb_bc_condition), intent(in) :: f_type
   type(c_bb_conditions)             :: c_type
!
   c_type%niter  = int(f_type%niter,c_int)
   c_type%autont = logical_to_int(f_type%auto)
!
   end function cast_bbcond_to_c

! -----------------------------------------------------------------------------------------------------

   subroutine cast_bbcond_to_f(c_type,f_type)
   use bb_bc, only: bb_bc_condition
   use arrayutil, only: int_to_logical
   type(c_bb_conditions), intent(in)    :: c_type
   type(bb_bc_condition), intent(inout) :: f_type
!
   f_type%niter  = int(c_type%niter)
   f_type%auto = int_to_logical(c_type%autont)
!
   end subroutine cast_bbcond_to_f

! -----------------------------------------------------------------------------------------------------

   subroutine get_gopt_info(gopt_info,sa_info,bb_info,ifButton) bind(C,name="get_gopt_info")
   use iso_c_binding
   use variables, only: dataset, cryst
   use counts, only: dvalue
   use sannel, only: goptim, sa_prelim, set_allowed_cfunction
   use errormod
   use bb_bc
!
   type(c_gopt_info_type), intent(out) :: gopt_info
   type(c_sa_conditions), intent(out)  :: sa_info
   type(c_bb_conditions), intent(out)  :: bb_info
   integer(c_int), intent(in), value   :: ifButton
   type(error_type)                    :: saerr
   integer, dimension(:), allocatable  :: cfvet
!
   if (ifButton == 0) then
!
!      Sto chiamando SA da comando %sannel nell'input: mi leggo le direttive
       call sa_prelim(goptim,sa_settings,bcond_settings,.true.,saerr=saerr)
   else
!
!      Sto chiamando SA da menubar: non mi leggo le direttive
       call sa_prelim(goptim,sa_settings,bcond_settings,.false.,saerr=saerr)
   endif
!
   if (saerr%signal) then
       call saerr%print()
   else
       gopt_info = cast_gopt_to_c(goptim)
       gopt_info%resmin = dataset(1)%resolution()
       gopt_info%resmax = cryst(1)%refc(2)%dval()
       call set_allowed_cfunction(1,cfvet)
       gopt_info%ncfvet = size(cfvet)
       gopt_info%cfvet(:size(cfvet)) = cfvet

       sa_info = cast_sacond_to_c(sa_settings)
       bb_info = cast_bbcond_to_c(bcond_settings)
   endif
!
   end subroutine get_gopt_info

! -----------------------------------------------------------------------------------------------------

   subroutine finalize_gopt(gopt_info,sa_info,bb_info) bind(C,name="finalize_gopt")
   use sannel, only: goptim, sa_run
!corr   use ilmviewmod, only: jav_set_modify
   type(c_gopt_info_type), intent(in) :: gopt_info
   type(c_sa_conditions), intent(in)  :: sa_info
   type(c_bb_conditions), intent(in)  :: bb_info
!
!corr   call jav_set_modify(0)

   call cast_gopt_to_f(gopt_info,goptim)
   call cast_sacond_to_f(sa_info,sa_settings)
   call cast_bbcond_to_f(bb_info,bcond_settings)
!
   call sa_run(goptim,sa_settings,bcond_settings)

!corr   call jav_set_modify(3)
!
   end subroutine finalize_gopt

! -----------------------------------------------------------------------------------------------------

   subroutine c_evaluate_res(res,numref,tthmaxr) bind(C,name="c_evaluate_res")
   use sannel, only: evaluate_res
!
   implicit none
   real(c_double), intent(in), value  :: res      ! risoluzione
   integer(c_int), intent(out)        :: numref   ! numero di riflessi
   real(c_double), intent(out)        :: tthmaxr  ! 2theta massimo
   integer(c_int)                     :: ier      ! >0 se res e' inferiore al valore minimo
   real                               :: tmax
!
   call evaluate_res(real(res),numref,tmax,ier)
   tthmaxr = real(tmax,c_double)
!
   end subroutine c_evaluate_res

! -----------------------------------------------------------------------------------------------------

   subroutine c_exit_sanneal() bind(C,name="c_exit_sanneal")
   use sannel, only: exitsanneal
   call exitsanneal()
   end subroutine c_exit_sanneal

! -----------------------------------------------------------------------------------------------------

   subroutine get_solution_size(nsol,size_name) bind(C,name="get_solution_size")
   use variables, only: cryst
   integer(c_int), intent(out) :: nsol
   integer(c_int), intent(out) :: size_name
!
   nsol = cryst(1)%nmodels() 
   size_name = len_trim(cryst(1)%cr_name)+1
!
   end subroutine get_solution_size 

! -----------------------------------------------------------------------------------------------------

   subroutine get_solution_info(info,strcol) bind(C,name="get_solution_info")
   use model_util
   use variables, only: cryst
   use strutil
   type(solution_info), dimension(*), intent(out) :: info
   character(c_char), dimension(*), intent(out)   :: strcol 
   integer                                        :: i
   real, dimension(5)                             :: infolist
   integer                                        :: nsol
!   
   strcol(:len_trim(cryst(1)%cr_name)+1) = toCString(cryst(1)%cr_name)
   nsol = cryst(1)%nmodels() 
   do i=1,nsol  
      call model_get(cryst(1)%model(i),fom=infolist)
      info(i)%run = nint(infolist(5))
      info(i)%cf = infolist(1)
   enddo
!
   end subroutine get_solution_info

! -----------------------------------------------------------------------------------------------------

   function fragment_number(idph) bind(C,name="fragment_number")  
   use variables, only: cryst
   use crystal_phase
   integer(c_int), value :: idph
   integer(c_int)        :: fragment_number
!
   fragment_number = cryst(idph)%numfrag()
!
   end function fragment_number
 
! -----------------------------------------------------------------------------------------------------

   subroutine get_fragment_atom_number(idph, natfrag) bind(C,name="get_fragment_atom_number")
   use variables, only: cryst
   use crystal_phase
   integer(c_int), value :: idph
   integer(c_int), intent(out) :: natfrag(*)
!
   natfrag(:cryst(idph)%numfrag()) = cryst(idph)%frag(:)%nat
!
   end subroutine get_fragment_atom_number

! -----------------------------------------------------------------------------------------------------

   subroutine get_fragment_atoms(idph, idfrag, atomvet, rcod, tlow, thigh, rbound, rat) bind(C,name="get_fragment_atoms")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph, idfrag
   integer(c_int), intent(out)       :: atomvet(*)
   integer(c_int), intent(out)       :: rcod(6)
   real(c_float), intent(out)        :: tlow(3), thigh(3), rbound(2)
   integer(c_int), intent(out)       :: rat
   integer                           :: nat
!
   nat = cryst(idph)%frag(idfrag)%nat
   atomvet(:nat) = cryst(idph)%frag(idfrag)%pos(:) - 1
   rcod(:) = cryst(idph)%frag(idfrag)%rcod(:)
   tlow(:) = cryst(idph)%frag(idfrag)%tlow(:)
   tlow(:) = cryst(idph)%frag(idfrag)%tlow(:)
   thigh(:) = cryst(idph)%frag(idfrag)%thigh(:)
   rbound(:) = cryst(idph)%frag(idfrag)%rbound(:)
!  Fortran: rat = 0 => centre of mass; rat > 0 => around atom rat
!  C++    : rat < 0 => centre of mass; rat >= 0 => around atom rat; for rat < 0: -rat+1 is the id to label on button
   rat = cryst(idph)%frag(idfrag)%rat - 1
!
   end subroutine get_fragment_atoms

! -----------------------------------------------------------------------------------------------------

   subroutine put_fragment_atoms(idph, idfrag, rcod, tlow, thigh, rbound, rat) bind(C,name="put_fragment_atoms")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph, idfrag
   integer(c_int), intent(in)        :: rcod(6)
   real(c_float), intent(in)         :: tlow(3), thigh(3), rbound(2)
   integer(c_int), intent(in), value :: rat
!
   cryst(idph)%frag(idfrag)%rcod(:) = rcod(:)
   cryst(idph)%frag(idfrag)%tlow(:) = tlow(:) 
   cryst(idph)%frag(idfrag)%thigh(:) = thigh(:)
   cryst(idph)%frag(idfrag)%rbound(:) = rbound(:) 
   if (rat < 0) then
       cryst(idph)%frag(idfrag)%rat = 0
   else 
       cryst(idph)%frag(idfrag)%rat = rat + 1
   endif
!
   end subroutine put_fragment_atoms

! -----------------------------------------------------------------------------------------------------

   function rotation_number(idph) bind(C,name="rotation_number")  
   use variables, only: cryst
   use crystal_phase
   integer(c_int), value :: idph
   integer(c_int)        :: rotation_number
!
   rotation_number = cryst(idph)%numrot()
!
   end function rotation_number

! -----------------------------------------------------------------------------------------------------

   subroutine get_rotation_atoms(idph,idrot,atomvet,rcod,bounds,torsion) bind(C,name="get_rotation_atoms")
   use variables, only: cryst
   integer(c_int), intent(in), value         :: idph,idrot
   integer(c_int), intent(out), dimension(4) :: atomvet
   integer(c_int), intent(out)               :: rcod
   real(c_float),  intent(out), dimension(2) :: bounds
   real(c_float), intent(out)                :: torsion
!
   atomvet(:) = cryst(idph)%rot(idrot)%ators(:)
   rcod = cryst(idph)%rot(idrot)%rcod
   bounds(:) = cryst(idph)%rot(idrot)%trbound(:2)
   torsion = cryst(idph)%rot(idrot)%torsval
!
   end subroutine get_rotation_atoms

! -----------------------------------------------------------------------------------------------------

   subroutine put_rotation_atoms(idph,idrot,rcod,bounds,torsion) bind(C,name="put_rotation_atoms")
   use variables, only: cryst
   integer(c_int), intent(in), value        :: idph, idrot
   integer(c_int), intent(in), value        :: rcod
   real(c_float),  intent(in), dimension(2) :: bounds
   real(c_float), intent(in), value         :: torsion
!
   cryst(idph)%rot(idrot)%rcod = rcod
   cryst(idph)%rot(idrot)%trbound(:2) = bounds(:)
   cryst(idph)%rot(idrot)%torsval = torsion
!
   end subroutine put_rotation_atoms

end module gopt_interop
