#include "indexingdialog.h"
#include "ui_indexingdialog.h"
#include "mainwindow.h"
#include "qt_utils.h"
#include "processdialog.h"

#include <QDir>
#include <QProcess>
#include <QMessageBox>

extern "C" void open_indexation_win(index_settings settings0[], index_settings settings1[]);
extern "C" void finish_indexC(index_settings settings[], bool sel[]);
extern "C" void export_dicvolC();
extern "C" void export_mcmailleC();
extern "C" void end_dicvol();

IndexingDialog::IndexingDialog(MainWindow *mw) :
    QDialog(mw),
    ui(new Ui::IndexingDialog),
    mWindow(mw)
{
    ui->setupUi(this);
    //adjustSize(); //force to use the minimum required space - doesn't work on linux
    ui->buttonBox->button(QDialogButtonBox::Apply)->setText("Run");
    ui->buttonBox->button(QDialogButtonBox::Apply)->setIcon(QIcon(":/pixmaps/pixmaps/go.png"));
    ui->buttonBox->button(QDialogButtonBox::Apply)->setDefault(true);

    QDoubleValidator *doubleValidator = new QDoubleValidator(this);
    ui->treorMaxVolume->setValidator(doubleValidator);
    ui->treorMaxAbc->setValidator(doubleValidator);
    ui->treorMinFom->setValidator(doubleValidator);
    ui->treorMw->setValidator(doubleValidator);
    ui->treorDensity->setValidator(doubleValidator);
    ui->treorErrDensity->setValidator(doubleValidator);

    ui->dicvolLineError->setValidator(doubleValidator);
    ui->dicvolMinFom->setValidator(doubleValidator);
    ui->dicvolMaxAbc->setValidator(doubleValidator);
    ui->dicvolMinVolume->setValidator(doubleValidator);
    ui->dicvolMaxVolume->setValidator(doubleValidator);
    ui->dicvolMw->setValidator(doubleValidator);
    ui->dicvolDensity->setValidator(doubleValidator);
    ui->dicvolErrDensity->setValidator(doubleValidator);    
    ui->dicvolZeroSLineEdit->setValidator(doubleValidator);

    ui->mcmailleZeroSLineEdit->setValidator(doubleValidator);
    ui->wLineEdit->setValidator(doubleValidator);
    ui->mcmailleMinVolume->setValidator(doubleValidator);
    ui->mcmailleMaxVolume->setValidator(doubleValidator);
    ui->mcmailleMaxAbc->setValidator(doubleValidator);

    connect(ui->ngrid0RadioButton, &QRadioButton::clicked, this, &IndexingDialog::disableBlackBoxMode);
    connect(ui->ngrid1RadioButton, &QRadioButton::clicked, this, &IndexingDialog::disableBlackBoxMode);
    connect(ui->ngrid2RadioButton, &QRadioButton::clicked, this, &IndexingDialog::disableBlackBoxMode);
    connect(ui->ngrid3RadioButton, &QRadioButton::clicked, this, &IndexingDialog::enableBlackBoxMode);
    connect(ui->ngrid_3RadioButton, &QRadioButton::clicked, this, &IndexingDialog::enableBlackBoxMode);
    connect(ui->ngrid4RadioButton, &QRadioButton::clicked, this, &IndexingDialog::enableBlackBoxMode);

    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &IndexingDialog::onButtonBoxClicked);
    connect(ui->checkBoxNtreor09, &QCheckBox::toggled, this, &IndexingDialog::onCheckBoxNtreor09Toggled);
    connect(ui->checkBoxDicvol, &QCheckBox::toggled, this, &IndexingDialog::onCheckBoxDicvolToggled);
    connect(ui->checkBoxMcmaille, &QCheckBox::toggled, this, &IndexingDialog::onCheckBoxMcmailleToggled);
}

IndexingDialog::~IndexingDialog()
{
    delete ui;
}

void IndexingDialog::setIndexing(ProgramType prog)
{
    uncheckPrograms();

    switch (prog) {
    case NTREOR09:
        ui->checkBoxNtreor09->setChecked(true);
        break;
    case DICVOL:
        ui->checkBoxDicvol->setChecked(true);
        break;
    case MCMAILLE:
        ui->checkBoxMcmaille->setChecked(true);
        break;
    }    

    open_indexation_win(defaultSettings,currentSettings);
    for (int i = 0; i < 3; i++) {
        savedSettings[i] = currentSettings[i];
    }

    setWidgetState(currentSettings);

    mWindow->saveEnabledActions();
    mWindow->enableActions(MainWindow::DialogOpenAction);
}

void IndexingDialog::setOutFileName(const QString &fileName, IndexingDialog::ProgramType prog)
{
    outFileName[prog] = fileName;
}

QString IndexingDialog::getOutFileName(IndexingDialog::ProgramType prog)
{
    return outFileName[prog];
}

void IndexingDialog::onCheckBoxNtreor09Toggled(bool checked)
{
    if (checked) ui->tabPrograms->setCurrentIndex(NTREOR09);
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(isProgramSelected());
}

void IndexingDialog::onCheckBoxDicvolToggled(bool checked)
{
    if (checked) ui->tabPrograms->setCurrentIndex(DICVOL);
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(isProgramSelected());
}

void IndexingDialog::onCheckBoxMcmailleToggled(bool checked)
{
    if (checked) ui->tabPrograms->setCurrentIndex(MCMAILLE);
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(isProgramSelected());
}

void IndexingDialog::onButtonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Apply) {
        getWidgetState(currentSettings);
        QDialog::accept();
        mWindow->restoreEnabledActions();

        bool selectedPrograms[3];
        selectedPrograms[NTREOR09] = ui->checkBoxNtreor09->isChecked() ? true : false;
        selectedPrograms[DICVOL] = ui->checkBoxDicvol->isChecked() ? true : false;
        selectedPrograms[MCMAILLE] = ui->checkBoxMcmaille->isChecked() ? true : false;
        if (selectedPrograms[NTREOR09]) setOutFileName("condens.out",NTREOR09);
        if (selectedPrograms[DICVOL]) setOutFileName("dicvol_out.ord",DICVOL);
        if (selectedPrograms[MCMAILLE]) setOutFileName("mcmaille.imp",MCMAILLE);

        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        finish_indexC(currentSettings,selectedPrograms);
        QApplication::restoreOverrideCursor();

    } else if (stdButton == QDialogButtonBox::Cancel) {
        // setWidgetState(savedSettings);
        // mWindow->restoreEnabledActions();

    } else if (stdButton == QDialogButtonBox::RestoreDefaults) {        
        setWidgetState(defaultSettings);

    } else if (stdButton == QDialogButtonBox::Help) {
        mWindow->openHelp("indexing-by-expo2014");
    }
}

void IndexingDialog::enableBlackBoxMode(bool enabled)
{
    disableBlackBoxMode(!enabled);
}

void IndexingDialog::disableBlackBoxMode(bool enabled)
{
    ui->mcmailleCrystalSystem->setEnabled(enabled);
    ui->mcmailleImpurityLines->setEnabled(enabled);
    ui->volMinLabel->setEnabled(enabled);
    setEnabledWidgetsInLayout(ui->volumeHorizontalLayout,enabled);
    ui->wLabel->setEnabled(enabled);
    ui->wLineEdit->setEnabled(enabled);
    ui->abcMaxLabel->setEnabled(enabled);
    ui->mcmailleMaxAbc->setEnabled(enabled);
}

void IndexingDialog::reject()
{
    setWidgetState(savedSettings);
    mWindow->restoreEnabledActions();
    QDialog::reject();
}

void IndexingDialog::setWidgetState(const index_settings indSettings[])
{
    //NTREOR09
    ui->treorMaxVolume->setText(QString::number(indSettings[NTREOR09].volmax));
    ui->treorMaxAbc->setText(QString::number(indSettings[NTREOR09].abcmax));
    ui->treorImpurityLines->setValue(indSettings[NTREOR09].n_imp);
    ui->treorMinFom->setText(QString::number(indSettings[NTREOR09].fom));
    ui->treorMw->setText(QString::number(indSettings[NTREOR09].mol_weight));
    ui->treorDensity->setText(QString::number(indSettings[NTREOR09].dens));
    ui->treorErrDensity->setText(QString::number(indSettings[NTREOR09].delden));

    //DICVOL14
    ui->dicvolLines->setValue(indSettings[DICVOL].nlines);
    ui->dicvolImpurityLines->setValue(indSettings[DICVOL].n_imp);
    ui->dicvolLineError->setText(QString::number(indSettings[DICVOL].eps));
    ui->dicvolMinFom->setText(QString::number(indSettings[DICVOL].fom));
    ui->dicvolMaxAbc->setText(QString::number(indSettings[DICVOL].abcmax));
    ui->dicvolMinVolume->setText(QString::number(indSettings[DICVOL].volmin));
    ui->dicvolMaxVolume->setText(QString::number(indSettings[DICVOL].volmax));
    ui->dicvolMw->setText(QString::number(indSettings[DICVOL].mol_weight));
    ui->dicvolDensity->setText(QString::number(indSettings[DICVOL].dens));
    ui->dicvolErrDensity->setText(QString::number(indSettings[DICVOL].delden));
    ui->dicvolCrystalSystem->setCrystalSystemState(indSettings[DICVOL].syst);
    if (indSettings[DICVOL].search_type == 0) {
        ui->optimizedRadioButton->setChecked(true);
    } else {
        ui->extendedRadioButton->setChecked(true);
    }
    ui->dicvolZeroSLineEdit->setText(QString::number(indSettings[DICVOL].zero));
    ui->dicvolZeroRefCheckBox->setChecked(indSettings[DICVOL].zero_ref);
    if (indSettings[DICVOL].zero_s == 0) {
        ui->zeroS0radio->setChecked(true);
    } else if (indSettings[DICVOL].zero_s == 1) {
        ui->zeroS1radio->setChecked(true);
    } else {
        ui->zeroS2radio->setChecked(true);
    }

    //MCMAILLE
    switch (indSettings[MCMAILLE].search_type) {
    case 0:
        ui->ngrid0RadioButton->setChecked(true);
        disableBlackBoxMode(true);
        break;
    case 1:
        ui->ngrid1RadioButton->setChecked(true);
        disableBlackBoxMode(true);
        break;
    case 2:
        ui->ngrid2RadioButton->setChecked(true);
        disableBlackBoxMode(true);
        break;
    case 3:
        ui->ngrid3RadioButton->setChecked(true);
        enableBlackBoxMode(true);
        break;
    case -3:
        ui->ngrid_3RadioButton->setChecked(true);
        enableBlackBoxMode(true);
        break;
    case 4:
        ui->ngrid4RadioButton->setChecked(true);
        enableBlackBoxMode(true);
        break;
    }
    ui->mcmailleLines->setValue(indSettings[MCMAILLE].nlines);
    ui->mcmailleImpurityLines->setValue(indSettings[MCMAILLE].n_imp);
    ui->mcmailleZeroSLineEdit->setText(QString::number(indSettings[MCMAILLE].zero));
    ui->wLineEdit->setText(QString::number(indSettings[MCMAILLE].width));
    ui->mcmailleMinVolume->setText(QString::number(indSettings[MCMAILLE].volmin));
    ui->mcmailleMaxVolume->setText(QString::number(indSettings[MCMAILLE].volmax));
    ui->mcmailleMaxAbc->setText(QString::number(indSettings[MCMAILLE].abcmax));
    ui->mcmailleCrystalSystem->setCrystalSystemState(indSettings[MCMAILLE].syst);
}

void IndexingDialog::getWidgetState(index_settings indSettings[])
{
    //NTREOR09
    indSettings[NTREOR09].volmax = ui->treorMaxVolume->text().toFloat();
    indSettings[NTREOR09].abcmax = ui->treorMaxAbc->text().toFloat();
    indSettings[NTREOR09].n_imp = ui->treorImpurityLines->value();
    indSettings[NTREOR09].fom = ui->treorMinFom->text().toFloat();
    indSettings[NTREOR09].mol_weight = ui->treorMw->text().toFloat();
    indSettings[NTREOR09].dens = ui->treorDensity->text().toFloat();
    indSettings[NTREOR09].delden = ui->treorErrDensity->text().toFloat();

    //DICVOL14
    indSettings[DICVOL].nlines = ui->dicvolLines->value();
    indSettings[DICVOL].n_imp = ui->dicvolImpurityLines->value();
    indSettings[DICVOL].eps = ui->dicvolLineError->text().toFloat();
    indSettings[DICVOL].fom = ui->dicvolMinFom->text().toFloat();
    indSettings[DICVOL].abcmax = ui->dicvolMaxAbc->text().toFloat();
    indSettings[DICVOL].volmin = ui->dicvolMinVolume->text().toFloat();
    indSettings[DICVOL].volmax = ui->dicvolMaxVolume->text().toFloat();
    indSettings[DICVOL].mol_weight = ui->dicvolMw->text().toFloat();
    indSettings[DICVOL].dens = ui->dicvolDensity->text().toFloat();
    indSettings[DICVOL].delden = ui->dicvolErrDensity->text().toFloat();
    ui->dicvolCrystalSystem->getCrystalSystemState(indSettings[DICVOL].syst);
    indSettings[DICVOL].search_type = ui->extendedRadioButton->isChecked() ? 1 : 0;
    indSettings[DICVOL].zero = ui->dicvolZeroSLineEdit->text().toFloat();
    indSettings[DICVOL].zero_ref = ui->dicvolZeroRefCheckBox->isChecked() ? 1 : 0;
    if (ui->zeroS0radio->isChecked()) {
        indSettings[DICVOL].zero_s = 0;
    } else if (ui->zeroS1radio->isChecked()) {
        indSettings[DICVOL].zero_s = 1;
    } else if (ui->zeroS2radio->isChecked()) {
        indSettings[DICVOL].zero_s = 2;
    }

    //MCMAILLE
    if (ui->ngrid0RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = 0;
    } else if (ui->ngrid1RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = 1;
    } else if (ui->ngrid2RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = 2;
    } else if (ui->ngrid3RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = 3;
    } else if (ui->ngrid_3RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = -3;
    } else if (ui->ngrid4RadioButton->isChecked()) {
        indSettings[MCMAILLE].search_type = 4;
    }
    indSettings[MCMAILLE].nlines = ui->mcmailleLines->value();
    indSettings[MCMAILLE].n_imp = ui->mcmailleImpurityLines->value();
    indSettings[MCMAILLE].zero = ui->mcmailleZeroSLineEdit->text().toFloat();
    indSettings[MCMAILLE].width = ui->wLineEdit->text().toFloat();
    indSettings[MCMAILLE].volmin = ui->mcmailleMinVolume->text().toFloat();
    indSettings[MCMAILLE].volmax = ui->mcmailleMaxVolume->text().toFloat();
    indSettings[MCMAILLE].abcmax = ui->mcmailleMaxAbc->text().toFloat();
    ui->mcmailleCrystalSystem->getCrystalSystemState(indSettings[MCMAILLE].syst);
}

void IndexingDialog::uncheckPrograms()
{
    ui->checkBoxNtreor09->setChecked(false);
    ui->checkBoxDicvol->setChecked(false);
    ui->checkBoxMcmaille->setChecked(false);
}

bool IndexingDialog::isProgramSelected()
{
    if (ui->checkBoxNtreor09->isChecked()) return true;
    if (ui->checkBoxDicvol->isChecked()) return true;
    if (ui->checkBoxMcmaille->isChecked()) return true;

    return false;
}

extern "C" void dicvol_process(int screen, int *ier)
{
    *ier = 0;
    //QString progname = "dicvol06.exe";
    QString progname = "DICVOL14.exe";
    MainWindow *mw = getMainWindow();
    if (QDir(QCoreApplication::applicationDirPath()).exists(progname)) {
        //Create input file for dicvol
        export_dicvolC();

        //Remove existent .ord output file
        QFile::remove("dicvol_out.ord");

        if (screen) {            
            ProcessDialog *dicvol = new ProcessDialog(mw);

            dicvol->setProgramName(progname);
#ifdef Q_OS_WINDOWS
            dicvol->setProgram(QCoreApplication::applicationDirPath()+QDir::separator()+progname);
            dicvol->show();
#else
            dicvol->setProgram("wine");
            QStringList args = {QCoreApplication::applicationDirPath()+QDir::separator()+progname};
            dicvol->setArguments(args);
#endif
            dicvol->start();
            dicvol->process()->waitForStarted();
            dicvol->process()->write("dicvol_inp\n");
            dicvol->process()->write("dicvol_out\n");

            // Terminate process using Stop button on the toolbar
            QObject::connect(mw, &MainWindow::actionStopTriggered, dicvol, &ProcessDialog::terminateProcess);

            dicvol->waitAndProcessEvents();
            dicvol->close();
            QObject::disconnect(mw, nullptr, dicvol, nullptr);
        } else {
            QProcess *dicvol = new QProcess(mw);
#ifdef Q_OS_WINDOWS
            dicvol->start(QCoreApplication::applicationDirPath()+QDir::separator()+progname);
#else
            QStringList args = {QCoreApplication::applicationDirPath()+QDir::separator()+progname};
            dicvol->start("wine",args);
#endif
            dicvol->waitForStarted();
            dicvol->write("dicvol_inp\n");
            dicvol->write("dicvol_out\n");
            dicvol->waitForFinished(-1);
        }

    } else {
        if (screen) {
            QMessageBox::warning(mw, QObject::tr("%1 error").arg(progname),
                                 QObject::tr("Can't find %1 program in %2").arg(progname,QCoreApplication::applicationDirPath()));
        } else {
            qCritical() << QString("Can't find %1 program in %2").arg(progname,QCoreApplication::applicationDirPath());
        }
        *ier = 2;
    }
}

extern "C" void mcmaille_process(int screen, int *ier)
{
    *ier = 0;
#ifdef Q_OS_WINDOWS
    QString progname = "McMaille.exe";
#else
    QString progname = "McMaille";
#endif
    MainWindow *mw = getMainWindow();
    if (QDir(QCoreApplication::applicationDirPath()).exists(progname)) {

        //Create input file for McMaille
        export_mcmailleC();

        //Copy hkl file from data files directory to the working directory
        QStringList hklFileList = {"cub.hkl", "hex.hkl", "rho.hkl", "tet.hkl", "ort.hkl", "mon.hkl",  "tri.hkl"};
        foreach(QString hklFile, hklFileList) {
            QFile::copy(MainWindow::getPathDataFiles()+hklFile,QDir::currentPath()+QDir::separator()+hklFile);
        }

        //Remove existent .ord output file
        QFile::remove("mcmaille.imp");
        if (screen) {
            ProcessDialog *mcmaille = new ProcessDialog(mw);
            mcmaille->show();

            mcmaille->setProgramName(progname);
            mcmaille->setProgram(QCoreApplication::applicationDirPath()+QDir::separator()+progname);
            mcmaille->setArguments({"mcmaille"});

            mcmaille->start();
            mcmaille->process()->waitForStarted();
            mcmaille->process()->write("q\n");

            // Terminate process using Stop button on the toolbar
            QObject::connect(mw, &MainWindow::actionStopTriggered, mcmaille, &ProcessDialog::terminateProcess);

            mcmaille->waitAndProcessEvents();
            mcmaille->close();
            QObject::disconnect(mw, nullptr, mcmaille, nullptr);
        } else {
            QProcess *mcmaille = new QProcess(mw);
            QStringList args = {"mcmaille"};
            mcmaille->start(QCoreApplication::applicationDirPath()+QDir::separator()+progname,args);
            mcmaille->write("q\n"); //"Type any character and RETURN to continue:"
            mcmaille->waitForFinished(-1);
        }

    } else {
        if (screen) {
            QMessageBox::warning(mw, QObject::tr("%1 error").arg(progname),
                                 QObject::tr("Can't find %1 program in %2").arg(progname,QCoreApplication::applicationDirPath()));
        } else {
            qCritical() << QString("Can't find %1 program in %2").arg(progname,QCoreApplication::applicationDirPath());
        }
        *ier = 2;
    }
}
