#include "mainwindow.h"
//#include "javwindow.h"
#include "ui_mainwindow.h"
#include "expokeysettings.h"
#include "qt_utils.h"
#include "texteditor.h"
#include "directives.h"
#include "nr.h"
#include "xpdutils.h"
#include "filtri.h"
#include "savedialog.h"
#include "imgsettingsdialog.h"
#include "fileutils.h"
#if defined(Q_OS_WIN)
#include "maintenancetool.h"
#else
#include "updater.h"
#endif

#include <iostream>
#include <algorithm>
#include <QSettings>
#include <QFileDialog>
#include <QMessageBox>   
#include <float.h>
#include <QtGlobal>
#include <QDebug>

#include <vector>

MainWindow *mMainWindow;
QString MainWindow::pathDataFiles = "";

extern "C" void lanciajavs_();
extern "C" void putlab95_(const char *filein, const char *fileout, int *ier, size_t idum1, size_t idum2);
extern "C" void get_reflection_info(int phaseIndex, int refIndex, int hkl[], float *tth, float *dval, int *err);
extern "C" int  importa_file(const char *filename, int len, int ftype);
extern "C" void esportanew_(int *iCod, const char *filename, size_t len);
extern "C" void open_diffraction_patt(const char *fileIn, int lenIn, const char *fileOut, int lenOut, int addData, int *err);
extern "C" int  readprojectwin(const char *filename, int len);
extern "C" void writeprojectwin(const char *filename, size_t len);
extern "C" void exitsir2002();
extern "C" void open_summary(const char *filename, size_t len);
extern "C" void run_peaksearchwin();
extern "C" void LoadPeaksC(const char *filename, int length, int tipo, int *ier);
extern "C" void SavePeaksC(const char *filename, int length, int tipo);
extern "C" void spg_determine();
extern "C" void open_cell_list();
extern "C" void spg_list();
extern "C" void save_cell_list(const char *filename, int length);
extern "C" void delete_peaksC(int pkvet[], int npeak);
extern "C" int  get_plot_size(int ptype);
extern "C" void get_plot_xy(float x[], float y[], float *wave, int plot_type);
extern "C" void process_action_points(int kaction, double xp, double yp,int *ier);
extern "C" void restart_dm();
extern "C" void recycle();
extern "C" void randomsolve();
extern "C" void open_hkl_list();
extern "C" void fsrbmw(int coderbm);
extern "C" void fslsq();
extern "C" void send_event(int event);
extern "C" void rietveld();
extern "C" void set_profile_enabled(int enabled);
extern "C" int is_extraction_active();
extern "C" void jav_export_structure_(const char *filename, int *tipo, int *wm, int *ifD, size_t sz);

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
    ui(new Ui::MainWindow),
    mAction(NoAction),
    savedZoomAction(mAction),
    yLowerRange(DBL_MAX),
    yUpperRange(-DBL_MAX),
    mLastYOffset(0),
    rescalePlot(true),
    mFileDialog(nullptr),
    projectFileSaved(false)
{
    ui->setupUi(this);

    setWindowTitle(qApp->applicationDisplayName()+"-"+qApp->applicationVersion());

    // set objects for the statusbar
    statusLabel1 = new QLabel(this);
    statusLabel1->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    statusProgressBar = new QProgressBar(this);
    statusProgressBar->setTextVisible(true);
    statusProgressBar->hide();
    ui->statusBar->addPermanentWidget(statusLabel1,3);
    ui->statusBar->addPermanentWidget(statusProgressBar,1);
    // send statustip on statusLabel1
    connect(ui->statusBar, &QStatusBar::messageChanged, this, [=](){
        statusLabel1->setText(ui->statusBar->currentMessage());
    });

    ui->splitter1->setStretchFactor(0,1);

    textViewer = new TextEditor(this);

    createActionGroup();

//   actionsSetup();

    crystalViewMenu = new CrystalViewMenu(this);
    createCrystalViewMenu();
    actionsSetup();
    //createActionConnect();

    //Set style of button 'Next'
    ui->toolBarNext->widgetForAction(ui->actionNext)->setObjectName(ui->actionNext->objectName());
    ui->toolBarNext->setStyleSheet("QToolButton#actionNext { background-color:red; border-radius:5px; }");

    readAction();
    ui->horizontalScrollBar->setPageStep(scrollBarLen);
    ui->horizontalScrollBar->hide();
    // create connection between axes and scroll bars:
    connect(ui->horizontalScrollBar, SIGNAL(valueChanged(int)), this, SLOT(horzScrollBarChanged(int)));
    connect(ui->plot->xAxis, SIGNAL(rangeChanged(QCPRange)), this, SLOT(xAxisChanged(QCPRange)));
    connect(ui->plot, SIGNAL(mouseMove(QMouseEvent*)), this, SLOT(myMoveEvent(QMouseEvent*)));
    connect(ui->plot, SIGNAL(selectionChangedByUser()), this, SLOT(selectionChanged()));

    connect(ui->plot, &CustomPlotZoom::addDeletePointAction, this, &MainWindow::addDeletePoint);

    //hide empty plot
    ui->plot->axisRect(0)->setVisible(false);
    ui->plot2->axisRect(0)->setVisible(false);

    calc.setGtype(graphItem::Calculated);
    calc.setDefaultIDColor(2);
    calc.setColorLine();
    calc.setName();
// TODO
// remove parameter idcolor from setColorLine
// fix default idcolor in setColor (ex. 3 for back and bpoints)
// setDefaultLine and scatter with no par, default idcolor shoud be written in graphItem.cpp
    back.setGtype(graphItem::Background);
    back.setDefaultIDColor(3);
    back.setColorLine();
    back.setName();
    bpoints.setGtype(graphItem::Background_Points);
    bpoints.setDefaultIDColor(3);
    bpoints.setColorLine();
    bpoints.setName();
    bpoints.setLineStyle(Qt::NoPen);
    bpoints.setLineConnectionType(QCPGraph::lsNone);
    diff.setGtype(graphItem::Difference);
    diff.setDefaultIDColor(4);
    diff.setColorLine();
    diff.setName();
    cdiff.setGtype(graphItem::Cumulative);
    cdiff.setDefaultIDColor(5);
    cdiff.setColorLine();
    cdiff.setName();
    peaks.setGtype(graphItem::Peaks);
    peaks.setDefaultIDColor(6);
    peaks.setColorLine();
    peaks.setName();
    peaks.setLineConnectionType(QCPGraph::lsImpulse);
    smooth.setGtype(graphItem::Smoothing);
    smooth.setName();
    smooth.setPen(QPen(Qt::green,1,Qt::DashLine));
    undpeaks.setGtype(graphItem::Unindexed_Peaks);
    undpeaks.setName();
    QColor colorUndPeaks = peaks.getPen().color();
    QCPScatterStyle scatter(QCPScatterStyle::ssTriangleInverted,colorUndPeaks,colorUndPeaks,10);
    undpeaks.setScatter(scatter);
    undpeaks.setLineStyle(Qt::NoPen);
    undpeaks.setLineConnectionType(QCPGraph::lsNone);
    sysAbs.setGtype(graphItem::Systematic_Absences);
    sysAbs.setName();        
    sysAbs.setLineStyle(Qt::NoPen);
    sysAbs.setLineConnectionType(QCPGraph::lsNone);
    selectedRef.setGtype(graphItem::Selected_Reflections);
    selectedRef.setName();
    selectedRef.setLineStyle(Qt::NoPen);
    selectedRef.setLineConnectionType(QCPGraph::lsNone);
    selectedPeak.setGtype(graphItem::Selected_Peaks);
    selectedPeak.setName();
    selectedPeak.setLineStyle(Qt::NoPen);
    selectedPeak.setLineConnectionType(QCPGraph::lsNone);
    intervalLimit.setGtype(graphItem::Intervals);
    intervalLimit.setName();

    createDialogs();

    ui->actionLegend->setChecked(plotSettings.legendVisible);
    connect(ui->actionLegend, &QAction::triggered, this, [=](bool state){
       plotSettings.legendVisible = state;
       ui->plot->legend->setVisible(state);
       ui->plot->replot();
    });

    ui->peakDockWidget->close();
    connect(ui->peakDockWidget, &QDockWidget::visibilityChanged, this, [this](bool visible){
       if (!visible) clearSelectedPeaks();
    });

    defaultState = saveState();
    resizeDocks({ui->dockWidget,ui->dockWidget_2},{50,50},Qt::Horizontal);

    readSettings();
    createRecentActions();
    enableActions(InitAction);

    //test for modern opengl
    //createOpenGLDock();

    mMainWindow = this;
}

GLint getMaxSamples()
{
   GLint maxSamples = 0;
   QOffscreenSurface sfc;
   sfc.create();
   QOpenGLContext ctx;
   if(ctx.create()) {
      ctx.makeCurrent(&sfc);
      glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
      ctx.doneCurrent();
   }
   return maxSamples;
}

void MainWindow::createDialogs()
{
#if !defined(JAV_WIDGET)
    int nMaxSamples = getMaxSamples();
    //jav = new JavWindow(nMaxSamples,this); //always on top on windows OS
    jav = new JavWindow(nMaxSamples,nullptr);
#endif
    // mFileDialog = new MyFileDialog(this);
    // connect(mFileDialog, SIGNAL(editSelected(QString,MyFileDialog*)), this, SLOT(on_editSelected(QString,MyFileDialog*)));
    // mFileDialog->setNameFilter("Expo Input Files (*.exp);;All Files (*.*)");
    // mFileDialog->setWindowTitle(tr("Load Expo Input File"));
    rangeDialog = new RangeDialog(this);
    indexingDialog = new IndexingDialog(this);
    peakSearchDialog = new PeakSearchDialog(this);
    backgroundDialog = new BackgroundDialog(this);
    extraBackgroundDialog = new ExtraBackgroundDialog;
    smoothingDialog = new SmoothingDialog(this);
    spgDialog = new SpacegDialog(this);
    cellListDialog = new CellListDialog(this);
    spaceListDialog = new SpaceGroupListDialog(this);
    exploreTrialsDialog = new ExploreTrialsDialog(this);
    globOptDialog = new GlobalOptDialog(this);
    dmDialog = new DirectMethodsDialog(this);
    fourDialog = new FourierDialog(this);
#if defined(JAV_WIDGET)
    fourPeakDialog = new FourierPeakListDialog(this);
#else
    fourPeakDialog = new FourierPeakListDialog(jav);
#endif
    hklDialog = new HklListDialog(this);
    rietDialog = new RietveldDialog(this);
    aboutDialog = new ExpoAboutDialog(this);
    intervalsDialog = new IntervalsDialog(this);
    plotStyleDialog = new PlotStyleDialog(this);
    connect(plotStyleDialog, &PlotStyleDialog::dialogClosed, this, &MainWindow::plotStyleClosed);
    plotSettings.read();
}

void MainWindow::actionsSetup()
{
    //Add the contextual menu with toolbar and dockwidget to menuWindow
    QList<QAction *> actions = createPopupMenu()->actions();
    foreach (QAction *action, actions) {
      ui->menuWindow->addAction(action);
    }
    QAction *peakListAction = actions.at(2);
    ui->menuPeaks->addAction(peakListAction); //Add 'Peak List' action
    connect(peakListAction, &QAction::triggered, this, [this](bool checked){
        if (checked) ui->peakListWidget->updatePeakListTable();
    });

    //File menu
    connect(ui->actionStructure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExportStructureTriggered);
    connect(ui->actionImage_Powder_Pattern, &QAction::triggered, this, &MainWindow::onActionImagePowderPatternTriggered);
    connect(ui->actionImage_Structure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExportStructureImageTriggered);
    connect(ui->action3D_Print_Structure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExport3DTriggered);
    connect(ui->actionExit, &QAction::triggered, qApp, &QApplication::quit);

    //Pattern menu
    connect(ui->actionIntervals, &QAction::triggered, this, &MainWindow::actionIntervalsTriggered);
    connect(ui->actionExtraBackground, &QAction::triggered, this, &MainWindow::actionExtraBackgroundTriggered);
    connect(ui->actionCycle, &QAction::triggered, this, &MainWindow::actionCycleTriggered);

    //Structure menu
    createCrystalViewConnect();

    //Info menu
    connect(ui->actionProfile, &QAction::triggered, this, &MainWindow::actionProfileTriggered);

    //Help menu
    connect(ui->actionDocumentation_HTML, &QAction::triggered, this, &MainWindow::actionDocumentationHtmlTriggered);
    connect(ui->actionDocumentation_PDF, &QAction::triggered, this, &MainWindow::actionDocumentationPdfTriggered);
    connect(ui->actionCheck_for_Updates, &QAction::triggered, this, &MainWindow::actionCheckForUpdatesTriggered);
    connect(ui->actionAbout, &QAction::triggered, this, &MainWindow::actionAboutTriggered);

    //Clear plotLabel to allow the user to reduce size of the plot area
    connect(ui->plot, &CustomPlotZoom::customPlotResized, this,[this](){
        ui->plotLabel1->clear();
        ui->plotLabel2->clear();
        ui->plotLabel3->clear();
    });
}

MainWindow::~MainWindow()
{
#if !defined(JAV_WIDGET)
    //jav has no parent, pointer must be deleted to avoid crash on quit
    jav->~JavWindow();
#endif
    delete ui;
}

void MainWindow::createActionGroup()
{
    QActionGroup *zoomGroup = new QActionGroup(this);
    zoomGroup->addAction(ui->actionSelection_Mode);
    zoomGroup->addAction(ui->actionHorizontal_Zoom);
    zoomGroup->addAction(ui->actionRectangle_Zoom);
    zoomGroup->addAction(ui->actionPan);
    zoomGroup->addAction(ui->actionAdd_Background_Point);
    zoomGroup->addAction(ui->actionDelete_Background_Point);
    zoomGroup->addAction(ui->actionAdd_Peak);
    zoomGroup->addAction(ui->actionDelete_Peak);
    connect(zoomGroup,&QActionGroup::triggered, this, &MainWindow::zoomGroupTriggered);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    QString titolo = "Exit " + qApp->applicationName();
    QString testo = QString("Do you really want \nstop the program %1?").arg(qApp->applicationName());
    if(QMessageBox::No == QMessageBox::question(this, titolo, testo,
                     QMessageBox::Yes | QMessageBox::No))
    {
       event->ignore();
       return;
    }
    send_event(STOP_EVENT);
    emit mainWindowClosed();
    writeSettings();
#if defined(JAV_WIDGET)
    ui->crystalWidget->saveSettings();
#else
    if (jav->isVisible()) jav->saveSettings();
#endif
    exitsir2002();

    std::cout << "\nThanks for using " << qPrintable(qApp->applicationName()) << "\nBye!\n\n";

    QApplication::quit();

    //Force close of dialog windows in a loop, kill the fortran computation
#ifndef Q_OS_WIN
    exit(EXIT_SUCCESS);
#endif
}

void MainWindow::writeSettings()
{
    QSettings settings;
    settings.setValue(EXPO_GEOMETRY_KEY, saveGeometry());
    settings.setValue(EXPO_STATE_KEY, saveState());
    settings.setValue(EXPO_SPLITSIZE_KEY1, ui->splitter1->saveState());
    //settings.setValue(EXPO_SPLITSIZE_KEY2, ui->splitter2->saveState());
}

void MainWindow::resizeMyDocks()
{
    //resizeDocks({ui->crystalViewDockWidget},{40},Qt::Horizontal);
    //qInfo() << "Width: " << width() << " Height: " << height();
    //ui->crystalViewDockWidget->resize(width()/3,ui->crystalViewDockWidget->height());
    //ui->crystalWidget->resize(width()/3,ui->crystalWidget->height());
    //ui->plot->resize(width()/2,ui->plot->height());
    //ui->crystalWidget->setMinimumWidth(400);
    //ui->plot->resize(width()/2,ui->plot->height());
    qInfo() << "Resize my docks: " << ui->crystalViewDockWidget->width() << " " << width();
    qInfo() << "Resize my docks: " << ui->crystalWidget->width() << " " << width();
}

void MainWindow::readSettings()
{
    QSettings settings;
    const QByteArray geometry = settings.value(EXPO_GEOMETRY_KEY, QByteArray()).toByteArray();
    if (geometry.isEmpty()) {
        const QRect availableGeometry = QGuiApplication::primaryScreen()->availableGeometry();
        resize(availableGeometry.width() * 0.80, availableGeometry.height() * 0.80);
        move((availableGeometry.width() - width()) / 2,
             (availableGeometry.height() - height()) / 2);
        //resizeMyDocks();

    } else {
        restoreGeometry(geometry);
    }

    restoreState(settings.value(EXPO_STATE_KEY).toByteArray());
    ui->splitter1->restoreState(settings.value(EXPO_SPLITSIZE_KEY1).toByteArray());
    //ui->splitter2->restoreState(settings.value(EXPO_SPLITSIZE_KEY2).toByteArray());
}

void MainWindow::checkAction(MouseAction action)
{
    if (action == mAction) return;

    switch (action) {
    case NoZoom:
        ui->actionSelection_Mode->setChecked(true);
        break;
    case RectangleZoom:
        ui->actionRectangle_Zoom->setChecked(true);
        break;
    case HorizontalZoom:
        ui->actionHorizontal_Zoom->setChecked(true);
        break;
    case Pan:
        ui->actionPan->setChecked(true);
        break;
    }
    setAction(action, false);
}

void MainWindow::saveAction()
{
    savedAction = mAction;
}

void MainWindow::restoreAction()
{
    checkAction(savedAction);
}

void MainWindow::setZoomAction()
{
    if (mAction != savedZoomAction)
        checkAction(savedZoomAction);
}

void MainWindow::readAction()
{
    QSettings settings;
    if (settings.contains(EXPO_ACTION_KEY)) {        
        QString sAction = settings.value(EXPO_ACTION_KEY).toString();
//        static int enumIdx = MainWindow::staticMetaObject.indexOfEnumerator("MouseAction");
//        MouseAction action = static_cast<MouseAction> (MainWindow::staticMetaObject.enumerator(enumIdx).keyToValue(sAction.toStdString().c_str()));
        QMetaEnum metaEnum = QMetaEnum::fromType<MouseAction>();
        int action = metaEnum.keyToValue(sAction.toStdString().c_str());
        checkAction(static_cast<MouseAction>(action));
    } else {
//        ui->actionHorizontal_Zoom->setChecked(true);
//        setAction(HorizontalZoom, false);
        checkAction(HorizontalZoom);
    }
}

// void MainWindow::createActionConnect()
// {
//     //File menu
//     connect(ui->actionStructure, &QAction::triggered, this, &MainWindow::onActionExportStructureTriggered);
//     connect(ui->actionImage_Powder_Pattern, &QAction::triggered, this, &MainWindow::onActionImagePowderPatternTriggered);
//     connect(ui->actionImage_Structure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExportStructureImageTriggered);
//     connect(ui->action3D_Print_Structure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExport3DTriggered);

//     //Clear plotLabel to allow the user to reduce size of the plot area
//     connect(ui->plot, &CustomPlotZoom::customPlotResized, this,[this](){
//         ui->plotLabel1->clear();
//         ui->plotLabel2->clear();
//         ui->plotLabel3->clear();
//     });

//     createCrystalViewConnect();
// }

void MainWindow::setAction(const MouseAction &action, bool writeConfig)
{
    mAction = action;
    switch (action) {
    case NoZoom:
        ui->plot->setZoomMode(CustomPlotZoom::NoZoom);
        ui->plot->setInteractions(QCP::iRangeZoom | QCP::iSelectLegend | QCP::iSelectPlottables);
        ui->plot->axisRects().at(0)->setRangeZoom(Qt::Horizontal);
        ui->plot->unsetCursor();
        ui->plot->disableTracer();
        savedZoomAction = action;
        break;
    case RectangleZoom:
        ui->plot->setZoomMode(CustomPlotZoom::RectangleZoom);
        ui->plot->setInteractions(QCP::iRangeZoom);
        ui->plot->unsetCursor();
        ui->plot->disableTracer();
        savedZoomAction = action;
        break;
    case HorizontalZoom:
        ui->plot->setZoomMode(CustomPlotZoom::HorizontalZoom);
        ui->plot->setInteractions(QCP::iRangeZoom);
        ui->plot->axisRects().at(0)->setRangeZoom(Qt::Horizontal);
        ui->plot->unsetCursor();
        ui->plot->disableTracer();
        savedZoomAction = action;
        break;
    case Pan:
        ui->plot->setZoomMode(CustomPlotZoom::PanZoom);
        ui->plot->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom);
        ui->plot->axisRects().at(0)->setRangeZoom(Qt::Horizontal);
        ui->plot->axisRects().at(0)->setRangeDrag(Qt::Horizontal);
        ui->plot->unsetCursor();
        ui->plot->disableTracer();
        savedZoomAction = action;
        break;
    case AddBackgroundPoint:
    case DeleteBackgroundPoint:
    case AddPeak:
    case DeletePeak:
        ui->plot->setZoomMode(CustomPlotZoom::AddDeletePoint);
        ui->plot->setInteractions(QCP::iRangeZoom);
        ui->plot->axisRects().at(0)->setRangeZoom(Qt::Horizontal);
        ui->plot->setCursor(Qt::CrossCursor);
        if (action == AddPeak || action == DeletePeak) {
            ui->plot->createTracer(peaks.getPen().color());
            ui->plot->attachTracerTo(ui->plot->graph(0));
        } else {
            ui->plot->disableTracer();
        }
        break;
    }

    if (writeConfig) {
        QSettings settings;
        QMetaEnum metaEnum = QMetaEnum::fromType<MouseAction>();
        const char *key = metaEnum.valueToKey(action);
        settings.setValue(EXPO_ACTION_KEY,key);
    }
}

void MainWindow::selectionChanged()
{
    // synchronize selection of graphs with selection of corresponding legend items:
    for (int i=0; i<ui->plot->graphCount(); ++i)
    {
      QCPGraph *graph = ui->plot->graph(i);
      if (graph->visible()) {
          QCPPlottableLegendItem *item = ui->plot->legend->itemWithPlottable(graph);
          if (item->selected() || graph->selected())
          {
              item->setSelected(true);
              graph->setSelection(QCPDataSelection(graph->data()->dataRange()));
          }
      }
    }

}

void MainWindow::addDeletePoint(QPoint mousePos)
{
    int ier;

    if (mAction == AddBackgroundPoint || mAction == AddPeak) {
        double xval = ui->plot->xAxis->pixelToCoord(mousePos.x());
        double yval = ui->plot->yAxis->pixelToCoord(mousePos.y());
        process_action_points(mAction, xval, yval, &ier);
        if (mAction == AddPeak) {
            ui->plot->createTracer(peaks.getPen().color()); //reattach tracer previously deleted in vedi
            ui->plot->attachTracerTo(ui->plot->graph(0));
        }

    } else if (mAction == DeleteBackgroundPoint) {
        int id = bpoints.getGraphIndex();
        double radius = qMax(5.0,bpoints.getScatter().size());
        QPointF closePoint = ui->plot->closestDataPoint(mousePos,id);
        if (closePoint.isNull()) return;

        int xp = ui->plot->xAxis->coordToPixel(closePoint.x());
        int yp = ui->plot->yAxis->coordToPixel(closePoint.y());
        QPointF dist = mousePos - QPointF(xp,yp);
        //double trueLength = std::sqrt(std::pow(dist.x(), 2) + std::pow(dist.y(), 2));

        if (dist.manhattanLength() < radius) {
            process_action_points(mAction, closePoint.x(), closePoint.y(), &ier);            
        }

    } else if (mAction == DeletePeak) {
        int id = peaks.getGraphIndex();
        if (id >= 0) {
            double xMouseCoord = ui->plot->xAxis->pixelToCoord(mousePos.x());
            double closePoint = ui->plot->closestDataPointX(xMouseCoord,id);

            int xp = ui->plot->xAxis->coordToPixel(closePoint);
            if (abs(mousePos.x() - xp) < 5) {
                process_action_points(mAction, closePoint, 0, &ier);
                ui->plot->createTracer(peaks.getPen().color()); //reattach tracer previously deleted in vedi
                ui->plot->attachTracerTo(ui->plot->graph(0));
            }
        }

    }
}

QProgressBar *MainWindow::getStatusProgressBar() const
{
    return statusProgressBar;
}

void MainWindow::setGraphicArea()
{
    ui->plot->axisRect(0)->setVisible(true);

    ui->plot->disableTracer();
    ui->plot->clearGraphs();
    ui->plot->clearPlottables();
    ui->plot->clearItems();

    plotSettings.applyTo(ui->plot);

    //ui->plot->yAxis->setLabel("Counts");

    ui->plot->xAxis2->setVisible(true);
    ui->plot->xAxis2->setTickLabels(false);
    ui->plot->xAxis2->setTicks(false);

    ui->plot->yAxis2->setVisible(true);
    ui->plot->yAxis2->setTickLabels(false);
    ui->plot->yAxis2->setTicks(false);

    //ui->plot->legend->setVisible(true);
    ui->plot->yAxis->grid()->setZeroLinePen(Qt::NoPen); // hide zero-line    

    nObserved = 0;
    nReflections = 0;
    nProfCurves = 0;
    yLowerRange = DBL_MAX;
    yUpperRange = -DBL_MIN;
    //xLowerRange = DBL_MAX;
    //xUpperRange = -DBL_MAX;

    for (int i = 0; i < obs.count(); i++) obs[i].setGraphIndex(-1);
    for (int i = 0; i < profCurves.count(); i++) profCurves[i].setGraphIndex(-1);
    back.setGraphIndex(-1);
    bpoints.setGraphIndex(-1);
    calc.setGraphIndex(-1);
    diff.setGraphIndex(-1);
    cdiff.setGraphIndex(-1);
    for (int i = 0; i < refl.count(); i++) refl[i].setGraphIndex(-1);
    peaks.setGraphIndex(-1);
    smooth.setGraphIndex(-1);
    undpeaks.setGraphIndex(-1);
    sysAbs.setGraphIndex(-1);
    sysAbs.setVisible(false);
    selectedRef.setGraphIndex(-1);
    selectedPeak.setGraphIndex(-1);
    intervalLimit.setGraphIndex(-1);
    intervalLimit.setVisible(false);
    plotWave.clear();
}

void MainWindow::addPlot(float x[], float y[], int num, int type, int visible, float wave, const QString name)
{    
    QVector<double> xvet(num);
    QVector<double> yvet(num);
    for (int i = 0; i < num; i++) {
         xvet[i] = x[i];
         yvet[i] = y[i];
    }

    if (plotSettings.getAbscissa() == xpdutils::DVALUE) {
        xpdutils::tthetaToD(xvet,yvet,wave);
        ui->plot->xAxis->setRangeReversed(true);
    } else {
        ui->plot->xAxis->setRangeReversed(false);
    }

    graphItem::ItemType gtype = static_cast<graphItem::ItemType>(type);
    switch (gtype) {
    case graphItem::Observed:
        if (obs.size() <= nObserved) { //new graphic element required
            graphItem item;
            item.setColorLine(nObserved);
            item.wave = wave;
            obs.push_back(item);
        }
        obs[nObserved].setData(xvet,yvet);
        obs[nObserved].setVisible(visible);
        obs[nObserved].setName(name);
        makePlot(xvet,yvet,obs[nObserved]);
        ui->plot->graph()->setScatterStyle(obs[nObserved].getScatter());
        nObserved++;
        break;

    case graphItem::Calculated:
        calc.setVisible(visible);
        calc.wave = wave;
        makePlot(xvet,yvet,calc);
        break;

    case graphItem::Background:
        back.setVisible(visible);
        back.wave = wave;
        makePlot(xvet,yvet,back);
        break;

    case graphItem::Background_Points:
        bpoints.setVisible(visible);
        bpoints.wave = wave;
        makePlot(xvet,yvet,bpoints);
        //ui->plot->graph()->setScatterStyle(bpoints.getScatter());
        break;

    case graphItem::Difference:
        diff.setVisible(visible);
        diff.wave = wave;
        makePlot(xvet,yvet,diff);
        break;

    case graphItem::Cumulative:
        cdiff.setVisible(visible);
        cdiff.wave = wave;
        makePlot(xvet,yvet,cdiff);
        break;

    case graphItem::Peaks:
        peaks.setVisible(visible);
        peaks.wave = wave;
        makePlot(xvet,yvet,peaks);
        //ui->plot->graph()->setLineStyle(QCPGraph::lsImpulse);
        break;

    case graphItem::Smoothing:
        smooth.setVisible(visible);
        smooth.wave = wave;
        makePlot(xvet,yvet,smooth);
        break;

    case graphItem::Unindexed_Peaks:
        undpeaks.setVisible(visible);
        undpeaks.wave = wave;
        makePlot(xvet,yvet,undpeaks);
        ui->plot->graph()->setScatterStyle(undpeaks.getScatter());
        enableRescalePlot(false);
        break;

    case graphItem::Profile_Curves:
        if (profCurves.size() <= nProfCurves) { //new graphic element required
            graphItem item;
            item.setColorLine(nProfCurves);
            item.setName(name);
            item.wave = wave;
            profCurves.push_back(item);
        }
        profCurves[nProfCurves].setData(xvet,yvet);
        profCurves[nProfCurves].setVisible(visible);
        makePlot(xvet,yvet,profCurves[nProfCurves]);
        ui->plot->graph()->setScatterStyle(profCurves[nProfCurves].getScatter());
        ui->plot->graph()->removeFromLegend();
        nProfCurves++;
        break;

    case graphItem::Reflections:
        break;
    }
}

void MainWindow::getYLimits(double &yMin, double &yMax)
{
    yMin = DBL_MAX;
    yMax = -DBL_MAX;

    QVector<graphItem>::const_iterator i;
    for (i = obs.constBegin(); i != obs.constEnd(); ++i) {
        if (i->getGraphIndex() >= 0 && i->isVisible()) {            
            if (i->getMin() < yMin) yMin = i->getMin();
            if (i->getMax() > yMax) yMax = i->getMax();
        }
    }

    if (back.getGraphIndex() >= 0 && back.isVisible()) {
        if (back.getMin() < yMin) yMin = back.getMin();
        if (back.getMax() > yMax) yMax = back.getMax();
    }

    if (peaks.getGraphIndex() >= 0 && peaks.isVisible()) {
        if (peaks.getMin() < yMin) yMin = peaks.getMin();
        if (peaks.getMax() > yMax) yMax = peaks.getMax();
    }

    if (calc.getGraphIndex() >= 0 && calc.isVisible()) {
        if (calc.getMin() < yMin) yMin = calc.getMin();
        if (calc.getMax() > yMax) yMax = calc.getMax();
    }

    if (diff.getGraphIndex() >= 0 && diff.isVisible()) {
        if (diff.getMin() < yMin) yMin = diff.getMin();
        if (diff.getMax() > yMax) yMax = diff.getMax();
    }

    if (cdiff.getGraphIndex() >= 0 && cdiff.isVisible()) {
        if (cdiff.getMin() < yMin) yMin = cdiff.getMin();
        if (cdiff.getMax() > yMax) yMax = cdiff.getMax();
    }

    if (smooth.getGraphIndex() >= 0 && smooth.isVisible()) {
        if (smooth.getMin() < yMin) yMin = smooth.getMin();
        if (smooth.getMax() > yMax) yMax = smooth.getMax();
    }
}

void MainWindow::drawReflections(const QVector<double>& x, double y, double length, QPen pen, int &itemStart, int &itemEnd)
{
    itemStart = ui->plot->itemCount();
    for (int i = 0; i < x.size(); i++) {
        QCPItemLine *line = new QCPItemLine(ui->plot);
        line->setPen(pen);
        line->setHead(QCPLineEnding::esNone);
        line->start->setCoords(x[i], y);
        line->end->setCoords(x[i], y+length);
    }
    itemEnd = x.size() + itemStart - 1;
}

void MainWindow::drawIntervals()
{
    //SONO QUI: move in draw and use maxval; pass only x
    if (!intervalLimit.isVisible()) return;

    QPen pen1(Qt::gray);
    QPen pen2(Qt::black);
    int ip=0;
    double diff = yUpperRange - yLowerRangeNoRef;
    double offset;
    QVector<double> xval = intervalLimit.getX();
    for (int i = 0; i < xval.size(); i+=2) {
        QCPItemLine *line = new QCPItemLine(ui->plot);
        if (ip % 2 == 0) {
            line->setPen(pen1);
            offset = diff * 0.01;
        } else {
            line->setPen(pen2);
            offset = 0.0;
        }
        ip++;
        line->setHead(QCPLineEnding::esSpikeArrow);
        line->setTail(QCPLineEnding::esSpikeArrow);
        line->start->setCoords(xval[i],yLowerRangeNoRef + offset);
        line->end->setCoords(xval[i+1],yLowerRangeNoRef + offset);
//        line->start->setCoords(x[i],y[i]);
//        line->end->setCoords(x[i+1],y[i+1]);
    }
}

void MainWindow::reDrawReflections(graphItem &ref, double y, double length)
{
    for (int ind = ref.itemIndexStart; ind <= ref.itemIndexEnd; ind++) {
        QCPItemLine *line = dynamic_cast<QCPItemLine *> (ui->plot->item(ind));
        int posRef = ref.itemIndexEnd - ind;
        //qInfo() << "Line: " << posRef << ref.getX().at(posRef) << y << y + length;
        line->start->setCoords(ref.getX().at(posRef), y);
        line->end->setCoords(ref.getX().at(posRef), y+length);
    }
}

void MainWindow::clearGraphSelection(graphItem &item)
{
    int id = item.getGraphIndex();
    if (id < 0) return;

    ui->plot->removeGraph(id);
    ui->plot->legend->setMaximumSize(ui->plot->legend->minimumOuterSizeHint());
    ui->plot->replot();
    item.setGraphIndex(-1);
}

void MainWindow::setSelectedRef(const QVector<int> &selected)
{
    if (refl.size() == 0) return;

    QVector<double> x(selected.size());
    for (int i = 0; i < selected.size(); i++) {
        x[i] = refl[0].getX(selected.at(i));
    }
    selectedRef.setX(x);
}

void MainWindow::drawGraphicItem(graphItem &item)
{
    int npoints = get_plot_size(item.getGtype());

    if (npoints > 0) {
        float *xv = new float[npoints];
        float *yv = new float[npoints];
        float wave;
        get_plot_xy(xv, yv, &wave, item.getGtype());

        QVector<double> xvet(npoints), yvet(npoints);
        for (int i = 0; i < npoints; i++) {
             xvet[i] = xv[i];
             yvet[i] = yv[i];
        }
        delete [] xv;
        delete [] yv;
        int id = item.getGraphIndex();
        if (id < 0) {
            item.wave = wave;
            makePlot(xvet,yvet,item);
        } else {
            ui->plot->graph(id)->setData(xvet,yvet,true);
        }

        ui->plot->legend->setMaximumSize(ui->plot->legend->minimumOuterSizeHint());
        ui->plot->replot();
    } else {
        clearGraphSelection(item);
    }
}

void MainWindow::drawPeaks()
{
    drawGraphicItem(peaks);
}

void MainWindow::drawSelectedPeaks(const QVector<int> &selected)
{
    if (selected.size() == 0) return;
    int idPeak = peaks.getGraphIndex();
    if (idPeak < 0) return;

    if (selectedPeak.getGraphIndex() < 0) {
        selectedPeak.setGraphIndex(ui->plot->graphCount());
        plotWave.append(-1);
        ui->plot->addGraph();
        ui->plot->graph()->setName(selectedPeak.getName());
        ui->plot->graph()->setPen(selectedPeak.getPen());
        QColor peakColor = peaks.getPen().color();
        QCPScatterStyle scatter = QCPScatterStyle(QCPScatterStyle::ssTriangleInverted,peakColor,peakColor,8);
        selectedPeak.setScatter(scatter);
        ui->plot->graph()->setScatterStyle(selectedPeak.getScatter());
    }

    selectedPeak.setX(selected);
    QVector<double> x(selected.size()), y(selected.size());
    for (int i = 0; i < selected.size(); i++) {
        x[i] = ui->plot->graph(idPeak)->data()->at(selected.at(i))->key;
        y[i] = ui->plot->graph(idPeak)->data()->at(selected.at(i))->value;
    }

    int id = selectedPeak.getGraphIndex();
    ui->plot->graph(id)->setData(x,y);

    //force resize of legend
    ui->plot->legend->setMaximumSize(ui->plot->legend->minimumOuterSizeHint());

    ui->plot->replot();
}

void MainWindow::deleteSelectedPeaks()
{
    if (selectedPeak.ixSize() == 0) return;

    int *pkdel = new int[selectedPeak.ixSize()];

    for (int i = 0; i < selectedPeak.ixSize(); i++) {
        pkdel[i] = selectedPeak.getIx().at(i);
    }
    delete_peaksC(pkdel, selectedPeak.ixSize());

    delete [] pkdel;
    clearSelectedPeaks();
}

void MainWindow::clearSelectedPeaks()
{
    clearGraphSelection(selectedPeak);
}

void MainWindow::drawSelectedRef()
{
    //TOFIX: combine with setSelectedRef in one function
    if (selectedRef.xSize() > 0) {
        if (selectedRef.getGraphIndex() < 0) {
            selectedRef.setGraphIndex(ui->plot->graphCount());
            plotWave.append(-1);
            ui->plot->addGraph();
            ui->plot->graph()->setName(selectedRef.getName());
            ui->plot->graph()->setPen(selectedRef.getPen());
            QColor refColor = refl.at(0).getPen().color();
            QCPScatterStyle scatter = QCPScatterStyle(QCPScatterStyle::ssTriangleInverted,refColor,refColor,8);
            selectedRef.setScatter(scatter);
            ui->plot->graph()->setScatterStyle(selectedRef.getScatter());
        }
        int id = selectedRef.getGraphIndex();
        double ypos = refl.at(0).getYPos() + refl.at(0).getLengthRef();
        ui->plot->graph(id)->setData(selectedRef.getX(),QVector<double>(selectedRef.xSize(),ypos));        

        ui->plot->replot();
    }
}

void MainWindow::clearSelectedRef()
{
    clearGraphSelection(selectedRef);
}

void MainWindow::setSystematicAbsences(const QVector<int> &refIndex)
{
    if (refl.size() == 0) return;

    QVector<double> x(refIndex.size());
    for (int i = 0; i < refIndex.size(); i++) {
        x[i] = refl[0].getX(refIndex.at(i));        
    }
    sysAbs.setX(x);
}

void MainWindow::drawSystematicAbsences()
{
    if (refl.size() == 0) return;

    if (sysAbs.getGraphIndex() < 0) {
        sysAbs.setGraphIndex(ui->plot->graphCount());
        plotWave.append(-1);
        ui->plot->addGraph();
        ui->plot->graph()->setName("Systematic Absences");
        ui->plot->graph()->setPen(sysAbs.getPen());
        QColor refColor = refl.at(0).getPen().color();
        QCPScatterStyle scatter = QCPScatterStyle(QCPScatterStyle::ssDiamond,refColor,refColor,8);
        sysAbs.setScatter(scatter);
        ui->plot->graph()->setScatterStyle(sysAbs.getScatter());
        ui->plot->legend->setMaximumSize(ui->plot->legend->minimumOuterSizeHint()); //force resize of legend
    }
    int id = sysAbs.getGraphIndex();
    double ypos = refl.at(0).getYPos() + refl.at(0).getLengthRef()/2.0;
    ui->plot->graph(id)->setData(sysAbs.getX(),QVector<double>(sysAbs.xSize(),ypos));

    ui->plot->replot();
}

void MainWindow::clearSystematicAbsences()
{
    int id = sysAbs.getGraphIndex();
    if (id < 0) return;

    ui->plot->removeGraph(id);
    ui->plot->replot();
    sysAbs.setGraphIndex(-1);
}

void MainWindow::updatePeakListTable()
{
    if (ui->peakListWidget->isVisible())
        ui->peakListWidget->updatePeakListTable();    
}

void MainWindow::enableActions(EnabledActions action, bool state)
{
    switch (action) {
    case InitAction:  //Init
        enableMenu(ui->menuExportData, false);
        ui->actionSave_Project->setEnabled(false);
        ui->actionSave_Project_As->setEnabled(false);
        enableMenu(ui->menuPattern, false);
        ui->actionExtraBackground->setVisible(false);
        ui->actionAdd_Background_Point->setEnabled(false);
        ui->actionDelete_Background_Point->setEnabled(false);
        enableMenu(ui->menuView, false);
        enableMenu(ui->menuSolve, false);
        enableMenu(ui->menuRefine, false);
        enableMenu(ui->menuInfo, false);
        ui->actionProfile->setEnabled(false);
        ui->toolBarRun->setEnabled(false);
        setNextActionVisible(false);
        crystalViewMenuEnabled(false);
        break;
    case PatternAction:   //Only Pattern
        enableMenu(ui->menuExportData, true);
        ui->actionSave_Project->setEnabled(true);
        ui->actionSave_Project_As->setEnabled(true);
        enableMenu(ui->menuPattern, true);        
        ui->actionIntervals->setEnabled(false);
        ui->actionAdd_Background_Point->setEnabled(true);
        ui->actionDelete_Background_Point->setEnabled(true);
        enableMenu(ui->menuView, true);
        ui->actionDirect_Methods->setEnabled(true);
        ui->actionExplore_Trials->setEnabled(false);
        ui->actionSimulated_Annealing->setEnabled(true);
        ui->actionRAMM_Procedure->setEnabled(false);
        ui->actionRecycle_in_Extra->setEnabled(is_extraction_active());
        enableMenu(ui->menuRefine, false);
        ui->actionRietveld->setEnabled(true);
        enableMenu(ui->menuInfo, true);
        ui->actionProfile->setEnabled(false);
        break;
    case StructureAction:  //Only Structure
        enableMenu(ui->menuExportData, true);
        ui->actionSave_Project->setEnabled(true);
        ui->actionSave_Project_As->setEnabled(true);
        enableMenu(ui->menuPattern, false);
        enableMenu(ui->menuView, false);
        enableMenu(ui->menuSolve, false);
        enableMenu(ui->menuRefine, false);
        enableMenu(ui->menuInfo, true);
        ui->actionProfile->setEnabled(false);
        break;
    case PatternAndStructureAction:  //Pattern + structure
        enableMenu(ui->menuExportData, true);
        ui->actionSave_Project->setEnabled(true);
        ui->actionSave_Project_As->setEnabled(true);
        enableMenu(ui->menuPattern, true);
        ui->actionIntervals->setEnabled(false);
        ui->actionAdd_Background_Point->setEnabled(true);
        ui->actionDelete_Background_Point->setEnabled(true);
        enableMenu(ui->menuView, true);
        ui->actionDirect_Methods->setEnabled(true);
        ui->actionSimulated_Annealing->setEnabled(true);
        ui->actionRAMM_Procedure->setEnabled(true);
        ui->actionRecycle_in_Extra->setEnabled(is_extraction_active());
        enableMenu(ui->menuRefine, true);
        ui->actionRietveld->setEnabled(true);
        enableMenu(ui->menuInfo, true);
        ui->actionProfile->setEnabled(false);
        break;
    case DialogOpenAction:
        enableMenu(ui->menuFile, false);
        enableMenu(ui->menuPattern, false);
        ui->actionAdd_Background_Point->setEnabled(false);
        ui->actionDelete_Background_Point->setEnabled(false);
        enableMenu(ui->menuSolve, false);
        enableMenu(ui->menuRefine, false);
        ui->crystalWidget->updateModify(3);
        break;
    case RunAction:
        ui->toolBarRun->setEnabled(state);
        ui->actionSkip->setEnabled(false);
        break;
    case RunSkipAction:  //Run procedure
        ui->toolBarRun->setEnabled(state);
        ui->actionSkip->setEnabled(state);
        break;
    case NextAction:        
        ui->actionLoad_Go->setEnabled(false);
        ui->menuRecent->setEnabled(false);
        ui->actionLoad_Examples->setEnabled(false);
        ui->actionLoad_Project->setEnabled(false);
        ui->actionImport_Diffraction_Pattern->setEnabled(false);
        ui->actionImport_Structure->setEnabled(false);
        ui->actionExit->setEnabled(false);
        enableMenu(ui->menuSolve,false);
        enableMenu(ui->menuRefine,false);
        //ui->menuPeaks->setEnabled(false);
        ui->menuIndexing->setEnabled(false);
        ui->menuSpacegroup->setEnabled(false);
        break;
    case PhaseAction:
        ui->actionExplore_Trials->setEnabled(state);
        break;
    case SaveAction:
        saveEnabledActions();
        break;
    case RestoreAction:
        restoreEnabledActions();
        break;
    case IntervalsAction:        
        ui->actionIntervals->setEnabled(state);
        break;
    case ExtraAction:
        enableMenu(ui->menuExportData, true);
        ui->actionSave_Project->setEnabled(true);
        ui->actionSave_Project_As->setEnabled(true);
        enableMenu(ui->menuPattern, false);
        ui->actionAdd_Background_Point->setEnabled(false);
        ui->actionDelete_Background_Point->setEnabled(false);
        enableMenu(ui->menuView, true);
        enableMenu(ui->menuSolve,false);
        enableMenu(ui->menuRefine, false);
        enableMenu(ui->menuInfo, true);
        ui->actionProfile->setEnabled(false);
        break;
    case ExtraBackgroundAction:        
        ui->actionExtraBackground->setVisible(state);
        ui->actionExtraBackground->setEnabled(state);
        ui->actionAdd_Background_Point->setEnabled(state);
        ui->actionDelete_Background_Point->setEnabled(state);
        ui->actionBackground->setVisible(!state);
        break;
    case ProfileAction:
        ui->actionProfile->setEnabled(state);
        if (!state && ui->actionProfile->isChecked()) {
            ui->actionProfile->setChecked(false);
            set_profile_enabled(false);
        }        
        break;
    case CycleAction:
        ui->actionCycle->setVisible(state);
        break;
    case PeaksAction: //unused!
        //ui->toolBarPattern
        //qInfo() << "Enable Pattern Menu";
        enableMenu(ui->menuPattern, true);
        //ui->actionIntervals->setEnabled(false);
        break;
    default:
        break;
    }
}

void MainWindow::setNextActionVisible(bool visible)
{
    ui->toolBarNext->setVisible(visible);
}

void MainWindow::enableNextAction()
{
    if (!ui->toolBarNext->isVisible()) return;

    bool isWaitCursor = stopWaitCursor();
    ui->actionNext->setEnabled(true);
    enableActions(SaveAction);
    enableActions(NextAction);
    QEventLoop loop;
    connect(ui->actionNext, &QAction::triggered, &loop, &QEventLoop::quit);
    connect(this, &MainWindow::actionStopTriggered, &loop, &QEventLoop::quit);
    loop.exec();
    ui->actionNext->setEnabled(false);
    if (isWaitCursor) QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    enableActions(RestoreAction);
}

void MainWindow::enableMenu(QMenu *menu, bool enab)
{
    QList<QAction*> actions = menu->actions();
    for (int i = 0; i < actions.size(); ++i)
        actions.at(i)->setEnabled(enab);
    QList<QMenu*> menus = menu->findChildren<QMenu*>();
    for (int i = 0; i < menus.size(); ++i) {
        menus.at(i)->setEnabled(enab);
        enableMenu(menus.at(i), enab); //to disable actions on toolbar
    }
}

bool MainWindow::openImportStructure(const QString &title)
{
    QString mTitle;
    if (title.isEmpty()) {
        mTitle = tr("Select File to Read from");
    } else {
        mTitle = title;
    }
    QSettings settings;
    QString selectedFilter = settings.value(IMPORT_STRUCTURE_FILTER_KEY).toString();;
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    mTitle, settings.value(DEFAULT_DIR_KEY).toString(),
                                                    FilesToImport, &selectedFilter);
    settings.setValue(IMPORT_STRUCTURE_FILTER_KEY,selectedFilter);
    if (!fileName.isEmpty()) {

        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
        int tipo = CrystalViewWidget::getImportFileCode(selectedFilter);
        fileName = QDir::toNativeSeparators(fileName);

        //mr la successiva per abilitare il menù Modify in Jav se la finestra è già aperta
        Properties::FromImport = true;
        importa_file(fileName.toStdString().c_str(), fileName.length(), tipo);
        Properties::FromImport = false;
        ui->crystalWidget->setModelNameFromFile(fileName);

        //Don't set currentFile if another file is already open
        if (currentFile.isEmpty()) {
            setCurrentFile(fileName, QVariant::fromValue(RecentFileType::Structure).toString());
        } else {
            setRecentFiles(fileName,QVariant::fromValue(RecentFileType::Structure).toString());
        }
        return true;
    }
    return false;
}

void MainWindow::createCrystalViewConnect()
{
    //Fragment Menu
    connect(crystalViewMenu->actionTetrahedron, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::TETRA, false); });
    connect(crystalViewMenu->actionOctahedron, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::OCTA, false); });
    connect(crystalViewMenu->actionSquare_Plane, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::SQUARE, false); });
    connect(crystalViewMenu->actionCube, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::CUBE, false); });
    connect(crystalViewMenu->actionTrigonal_Plane, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::TRIG, false); });
    connect(crystalViewMenu->actionAntiprism_Tetragonal, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::APRISM_TETRA, false); });
    connect(crystalViewMenu->actionPrism_Trigonal, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::PRISM_TRIG, false); });
    connect(crystalViewMenu->actionIcosahedron, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::ICOSA, false); });
    connect(crystalViewMenu->actionIsolated_Atoms, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::ISOLATED, false); });
    connect(crystalViewMenu->actionFrom_Smiles, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionFromSmilesTriggered(0); });
    connect(crystalViewMenu->actionFrom_COD, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionFromCODTriggered(0); });

    //Modify Menu
    connect(crystalViewMenu->actionUndo, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionUndoTriggered);
    connect(crystalViewMenu->actionRedo, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionRedoTriggered);
    connect(crystalViewMenu->actionDelete_Selected_Atoms_or_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDeleteSelectedAtomsOrBondsTriggered);
    connect(crystalViewMenu->actionDelete_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDeleteBondsTriggered);
    connect(crystalViewMenu->actionDelete_Duplicate_Atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDeleteDuplicateAtomsTriggered);
    connect(crystalViewMenu->actionRename, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionRenameTriggered);
    connect(crystalViewMenu->actionRenumber, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionRenumberTriggered);
    connect(crystalViewMenu->actionSort_by_Species, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSorBySpeciesTriggered);
    connect(crystalViewMenu->actionAdd_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAddBondsTriggered);
    connect(crystalViewMenu->actionFrom_File, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionFromFileTriggered);
    connect(crystalViewMenu->actionFrom_COD_2, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionFromCODTriggered(1); });
    connect(crystalViewMenu->actionFrom_Smiles_2, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionFromSmilesTriggered(1); });
    connect(crystalViewMenu->actionTetrahedron_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::TETRA, true); });
    connect(crystalViewMenu->actionOctahedron_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::OCTA, true); });
    connect(crystalViewMenu->actionSquare_Plane_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::SQUARE, true); });
    connect(crystalViewMenu->actionCube_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::CUBE, true); });
    connect(crystalViewMenu->actionTrigonal_Plane_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::TRIG, true); });
    connect(crystalViewMenu->actionAntiprism_Tetragonal_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::APRISM_TETRA, true); });
    connect(crystalViewMenu->actionPrism_Trigonal_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::PRISM_TRIG, true); });
    connect(crystalViewMenu->actionIcosahedron_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::ICOSA, true); });
    connect(crystalViewMenu->actionIsolated_Atoms_2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionImportFragment(PolyhedraDialog::ISOLATED, true); });
    connect(crystalViewMenu->actionDuplicate_All, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDuplicateAllTriggered);
    connect(crystalViewMenu->actionDuplicate_Selected_Atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDuplicateSelectedAtomsTriggered);
    connect(crystalViewMenu->actionAtom_Properties, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionStructurePropertiesTriggered(0); });
    connect(crystalViewMenu->actionBond_Properties, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionStructurePropertiesTriggered(1); });
    connect(crystalViewMenu->actionAngle_Properties, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionStructurePropertiesTriggered(2); });
    connect(crystalViewMenu->actionTorsion_Properties, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionStructurePropertiesTriggered(3); });
    connect(crystalViewMenu->actionDeleteGhost, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionSpecie(-1); });
    for (int i = 0; i < crystalViewMenu->deleteSpecieActions.size(); i++) {
        connect(crystalViewMenu->deleteSpecieActions.at(i), &QAction::triggered, ui->crystalWidget, [this,i](){ ui->crystalWidget->onActionSpecie(i); });
    }
    for (int i = 0; i < crystalViewMenu->changeSpecieActions.size(); i++) {
        connect(crystalViewMenu->changeSpecieActions.at(i), &QAction::triggered, ui->crystalWidget, [this,i](){ ui->crystalWidget->onActionSpecie(i); });
    }
    connect(crystalViewMenu->actionChangeSpecieOther, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionSpecie(-1); });
    for (int i = 0; i < crystalViewMenu->addAtomActions.size(); i++) {
        connect(crystalViewMenu->addAtomActions.at(i), &QAction::triggered, ui->crystalWidget, [this,i](){ ui->crystalWidget->onActionSpecie(i); });
    }
    connect(crystalViewMenu->actionAddAtomOther, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionSpecie(-1); });
    for (int i = 0; i < crystalViewMenu->bondTypeActions.size(); i++) {
        connect(crystalViewMenu->bondTypeActions.at(i), &QAction::triggered, ui->crystalWidget, [this,i](){ ui->crystalWidget->onChangeMult(i); });
    }

    //Selection
    connect(crystalViewMenu->actionAtoms_or_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAtomsOrBondsTriggered);
    connect(crystalViewMenu->actionRectangular, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionRectangularTriggered);
    connect(crystalViewMenu->actionMolecule, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionMoleculeTriggered);
    connect(crystalViewMenu->actionZoom_Mode, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionZoomModeTriggered);
    connect(crystalViewMenu->actionSelection_Mode, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSelectionModeTriggered);
    connect(crystalViewMenu->actionSelect_All, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSelectAllTriggered);
    connect(crystalViewMenu->actionSelect_None, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSelectNoneTriggered);
    connect(crystalViewMenu->actionInvert_Selection, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionInvertSelectionTriggered);
    connect(crystalViewMenu->actionExpand_to_Molecule, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExpandToMoleculeTriggered);
    connect(crystalViewMenu->actionDistances, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDistancesTriggered);
    connect(crystalViewMenu->actionAngles, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAnglesTriggered);
    connect(crystalViewMenu->actionTorsion_Angles, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionTorsionAnglesTriggered);
    connect(crystalViewMenu->actionSelectGhost, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionSpecie(-1); });
    for (int i = 0; i < crystalViewMenu->selectSpecieActions.size(); i++) {
        connect(crystalViewMenu->selectSpecieActions.at(i), &QAction::triggered, ui->crystalWidget, [this,i](){ ui->crystalWidget->onActionSpecie(i); });
    }

    //View
    connect(crystalViewMenu->actionPreferences, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPreferencesTriggered);
    connect(crystalViewMenu->actionPacking, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPacking);
    connect(crystalViewMenu->actionExpand, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionExpandTriggered);
    connect(crystalViewMenu->actionReduce, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionReduceTriggered);
    connect(crystalViewMenu->actionView_Cell_Edges, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionViewCellEdges);
    connect(crystalViewMenu->actionGrow_Fragments, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionGrowFragmentsTriggered);
    connect(crystalViewMenu->actionAssemble, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAssembleTriggered);
    connect(crystalViewMenu->actionMove_Atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionMoveAtomsTriggered);
    connect(crystalViewMenu->actionZoom_In, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionZoomInTriggered);
    connect(crystalViewMenu->actionZoom_Out, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionZoomOutTriggered);
    connect(crystalViewMenu->actionReset_Scale, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionResetScaleTriggered);
    connect(crystalViewMenu->actionReset_View, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionResetViewTriggered);
    connect(crystalViewMenu->actionAround_X, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundXTriggered);
    connect(crystalViewMenu->actionAround_X_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundX2Triggered);
    connect(crystalViewMenu->actionAround_Y, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundYTriggered);
    connect(crystalViewMenu->actionAround_Y_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundY2Triggered);
    connect(crystalViewMenu->actionAround_Z, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundZTriggered);
    connect(crystalViewMenu->actionAround_Z_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAroundZ2Triggered);
    connect(crystalViewMenu->actionStep, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionStepTriggered);
    connect(crystalViewMenu->actionAll_Atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAllAtomsTriggered);
    connect(crystalViewMenu->actionDelete_All_Labels, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDeleteAllLabelsTriggered);
    connect(crystalViewMenu->actionexcept_Hydrogens, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionexceptHydrogensTriggered);
    connect(crystalViewMenu->actionHeavy_atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionHeavyAtomsTriggered);
    connect(crystalViewMenu->actionselected_atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionselectedAtomsTriggered);
    connect(crystalViewMenu->actionAtom_Name, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAtomNameTriggered);
    connect(crystalViewMenu->actionAtom_Specie, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAtomSpecieTriggered);
    connect(crystalViewMenu->actionAtom_Number, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionAtomNumberTriggered);
    connect(crystalViewMenu->actionShow_All, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionShowAllTriggered);
    connect(crystalViewMenu->actionHide_Selected, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionHideSelectedTriggered);
    connect(crystalViewMenu->actionHide_Unselected, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionHideUnselectedTriggered);
    connect(crystalViewMenu->actionHide_Hydrogens, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionHideHydrogensTriggered);
    connect(crystalViewMenu->actiona, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionaTriggered);
    connect(crystalViewMenu->actionb, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionbTriggered);
    connect(crystalViewMenu->actionc, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActioncTriggered);
    connect(crystalViewMenu->actiona_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActiona2Triggered);
    connect(crystalViewMenu->actionb_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionb2Triggered);
    connect(crystalViewMenu->actionc_2, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionc2Triggered);
    connect(crystalViewMenu->actionLeast_Square_Plane, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionLeastSquarePlaneTriggered);
    connect(crystalViewMenu->actionSelected_Plane, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSelectedPlaneTriggered);
    connect(crystalViewMenu->actionPolyhedra, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPolyhedraTriggered);
    connect(crystalViewMenu->actionPolyhedra_Properties, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPolyhedraPropertiesTriggered);
    connect(crystalViewMenu->actionShow_Multiple_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionShowMultipleBondsTriggered);
    connect(crystalViewMenu->actionHydrogen_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionHydrogenBondsTriggered);
    connect(crystalViewMenu->actionShort_Contacts, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionShortContactsTriggered);
    connect(crystalViewMenu->actionDelete_Hanging_Contacts, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionDeleteHangingContactsTriggered);
    connect(crystalViewMenu->actionEdit_H_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionEditHBondsTriggered);
    connect(crystalViewMenu->actionShow_Fourier_Map, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionShowFourierMapTriggered);
    connect(crystalViewMenu->actionCompute_Fo, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionComputeFoTriggered);
    connect(crystalViewMenu->actionCompute_Fc, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionComputeFcTriggered);
    connect(crystalViewMenu->actionCompute_2Fo_Fc, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionCompute2FoFcTriggered);
    connect(crystalViewMenu->actionCompute_Fo_Fc, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionComputeFoFcTriggered);
    connect(crystalViewMenu->actionCompute_Fc, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionComputeFcTriggered);

    //Tools
    connect(crystalViewMenu->actionAutomatic, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(12); });
    connect(crystalViewMenu->actionMethine_C_H, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(13); });
    connect(crystalViewMenu->actionMethylene_Group, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(14); });
    connect(crystalViewMenu->actionMethyl_Group, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(15); });
    connect(crystalViewMenu->actionAromatic_C_H, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(16); });
    connect(crystalViewMenu->actionHydroxyl_O_H, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(17); });
    connect(crystalViewMenu->actionTerminal_X_CH2, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(18); });
    connect(crystalViewMenu->actionAcetylenic_C_H, &QAction::triggered, this, [this](){ ui->crystalWidget->onActionAddHydrogensTriggered(20); });
    connect(crystalViewMenu->actionNormalise_Hydrogens, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionNormaliseHydrogensTriggered);
    connect(crystalViewMenu->actionBuild_Bonds, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionBuildBondsTriggered);
    connect(crystalViewMenu->actionBond_Settings, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionBondSettingsTriggered);
    connect(crystalViewMenu->actionComplete, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionCompleteTriggered);
    connect(crystalViewMenu->actionRegularize, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionRegularizeTriggered);
    connect(crystalViewMenu->actionEnantiomorph, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionEnantiomorphTriggered);
    connect(crystalViewMenu->actionTransform, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionTransformTriggered);
    connect(crystalViewMenu->actionStandardization_of_Crystal_Structure, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionStandardizationOfCrystalStructureTriggered);
    connect(crystalViewMenu->actionOptimize_geometry, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionOptimizeGeometryTriggered);
    connect(crystalViewMenu->actionOptimize_Selected_Atoms, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionOptimizeSelectedAtomsTriggered);
    connect(crystalViewMenu->actionMOPAC, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionMOPACTriggered);
    connect(crystalViewMenu->actionView_MOPAC_Output, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionViewMOPACOutputTriggered);
    connect(crystalViewMenu->actionOptimize_Crystal_Structure_by_DFT_D, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionOptimizeCrystalStructureByDFTDTriggered);
    connect(crystalViewMenu->actionOverlay_Structures, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionOverlayStructuresTriggered);
    connect(crystalViewMenu->actionOverlay_Info, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionOverlayInfoTriggered);
    connect(crystalViewMenu->actionCentroids, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionCentroidsTriggered);
    connect(crystalViewMenu->actionPowder_Diffraction_Pattern, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPowderDiffractionPatternTriggered);
    connect(crystalViewMenu->actionPair_Distribution_Function_PDF, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionPairDistributionFunctionPDFTriggered);
    connect(crystalViewMenu->actionBond_Valence_Sum, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionBondValenceSumTriggered);

    //Info
    connect(crystalViewMenu->actionStructure, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(0); });
    connect(crystalViewMenu->actionList_of_Atoms, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(1); });
    connect(crystalViewMenu->actionList_of_Bonds, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(2); });
    connect(crystalViewMenu->actionList_of_Angles, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(3); });
    connect(crystalViewMenu->actionList_of_Torsion_Angles, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(4); });
    connect(crystalViewMenu->actionBar_Chart, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(5); });
    connect(crystalViewMenu->actionList_of_Contacts, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionStructureInfoTriggered(6); });
    connect(crystalViewMenu->actionGeometry, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionGeometryTriggered);
    connect(crystalViewMenu->actionSphere, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionSphereTriggered);

    //Orientation
    connect(crystalViewMenu->actionrxplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(0, 1.); });
    connect(crystalViewMenu->actionrxminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(0, -1.); });
    connect(crystalViewMenu->actionryplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(1, 1.); });
    connect(crystalViewMenu->actionryminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(1, -1.); });
    connect(crystalViewMenu->actionrzplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(2, 1.); });
    connect(crystalViewMenu->actionrzminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionRotateTriggered(2, -1.); });
    connect(crystalViewMenu->actionxcplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(0, 1.); });
    connect(crystalViewMenu->actionxcminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(0, -1.); });
    connect(crystalViewMenu->actionycplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(1, -1.); });
    connect(crystalViewMenu->actionycminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(1, 1.); });
    connect(crystalViewMenu->actionzcplus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(2, 1.); });
    connect(crystalViewMenu->actionzcminus, &QAction::triggered, ui->crystalWidget, [this](){ ui->crystalWidget->onActionTranslateTriggered(2, -1.); });

    connect(crystalViewMenu->actionLabels_On_Off, &QAction::triggered, this, &MainWindow::onActionLabelsOnOfftriggered);
    connect(crystalViewMenu->actionClear_Measurements, &QAction::triggered, ui->crystalWidget, &CrystalViewWidget::onActionClearMeasurementsTriggered);
}

void MainWindow::exportFile(const QString &fileName, int tipo)
{
    const char *cStr;
    QByteArray inBytes = fileName.toUtf8();
    cStr = inBytes.constData();
    size_t sz = strlen(cStr);

    int which = 1;
    int ifD = 0;
    if(tipo >= 1000)
    {
        tipo -= 1000;
        ifD = 1;
    }
    jav_export_structure_(cStr, &tipo, &which, &ifD, sz);
}

void MainWindow::createCrystalViewMenu()
{
    structureMenu = new QMenu(tr("&Structure"));
    menuBar()->insertMenu(ui->menuInfo->menuAction(),structureMenu);

    crystalViewMenu->insertMenuFragment(ui->menuFile, ui->menuExportData);
    crystalViewMenu->addMenuModify(structureMenu);
    crystalViewMenu->addMenuSelect(structureMenu);
    crystalViewMenu->addMenuView(structureMenu);
    crystalViewMenu->addMenuTools(structureMenu);
    ui->menuInfo->addSeparator();
    crystalViewMenu->addMenuInfo(ui->menuInfo);
    crystalViewMenu->addToolbars(this);
}

/*
void MainWindow::createOpenGLDock()
{
    QSurfaceFormat format;
    format.setRenderableType(QSurfaceFormat::OpenGL);
    format.setProfile(QSurfaceFormat::CoreProfile);
    format.setDepthBufferSize(24);
    format.setVersion(3,3);

    QDockWidget *dockWidget = new QDockWidget(this);
    dockWidget->setObjectName(QString::fromUtf8("dockWidget"));
    QWidget *dockWidgetContents = new QWidget();
    dockWidgetContents->setObjectName(QString::fromUtf8("dockWidgetContents"));
    QVBoxLayout *dockVLayout = new QVBoxLayout(dockWidgetContents);
    dockVLayout->setObjectName(QString::fromUtf8("dockVLayout"));
    dockWidget->setWidget(dockWidgetContents);
    addDockWidget(Qt::RightDockWidgetArea, dockWidget);

    m_crystalScene = new CrystalSceneView(this);
    m_crystalScene->setMinimumSize(QSize(600,600));
    m_crystalScene->setFormat(format);
    m_crystalScene->setFocusPolicy (Qt::StrongFocus);
    dockVLayout->setContentsMargins(0, 0, 0, 0);
    dockVLayout->setSpacing(0);
    dockVLayout->insertWidget(0, m_crystalScene);
    dockVLayout->addWidget(m_crystalScene);
}
*/

void MainWindow::enumerateEnabledActionsMenu(QMenu *menu)
{
    foreach (QAction *action, menu->actions()) {
        if (action->isSeparator()) {

        } else if (action->menu()) {
            stateActions[action] = action->isEnabled();
            if (action->isEnabled()) enumerateEnabledActionsMenu(action->menu());
        } else {
            stateActions[action] = action->isEnabled();
        }
    }
}

void MainWindow::clearProjectFile()
{
    projectFileSaved = false;
    currentProjectFile.clear();
}

QString MainWindow::getPathDataFiles()
{
    return pathDataFiles;
}

void MainWindow::setPathDataFiles(const QString &newPathDataFiles)
{
    pathDataFiles = newPathDataFiles;
}

void MainWindow::saveEnabledActions()
{
   stateActions.clear();
   const QList<QAction *> actions = ui->menubar->actions();  //QMenuBar::actions();
   foreach (auto action, actions) {
       stateActions[action] = action->isEnabled();
       if (action->isEnabled()) enumerateEnabledActionsMenu(action->menu());
   }
}

void MainWindow::restoreEnabledActions()
{
    QMutableMapIterator<QAction *, bool> i(stateActions);
    while (i.hasNext()) {
        i.next();
        if(i.key()->menu()) {
            i.key()->menu()->setEnabled(i.value());
        } else {
            i.key()->setEnabled(i.value());
        }
    }
}

QString MainWindow::getCurrentFile() const
{
    return currentFile;
}

bool MainWindow::rescalePlotEnabled() const
{
    return rescalePlot;
}

void MainWindow::enableRescalePlot(bool value)
{
    rescalePlot = value;
}

void MainWindow::setOutputFileName(const QString &value)
{
    outputFileName = value;
}

double MainWindow::getLastYOffset() const
{
    return mLastYOffset;
}
#if defined(JAV_WIDGET)
CrystalViewWidget *MainWindow::getJav() const
{
    return ui->crystalWidget;
}
#else
JavWindow *MainWindow::getJav() const
{
    return jav;
}
#endif
InfoTableWidget *MainWindow::getInfoTable() const
{
    return ui->infoTableNew;
}

CustomPlotZoom *MainWindow::getPlot() const
{
    return ui->plot;
}

CustomPlotZoom *MainWindow::getPlot2() const
{
    return ui->plot2;
}

void MainWindow::clearPlot2()
{
    ui->plot2->axisRect(0)->setVisible(true);
    ui->plot2->clearPlottables();
    ui->plot2->clearGraphs();
    ui->plot2->clearItems();

    //Remove additional elements
    if (ui->plot2->plotLayout()->elementCount() > 1) {
        ui->plot2->plotLayout()->removeAt(0);
        ui->plot2->plotLayout()->simplify();
    }
}

void MainWindow::setStatusMessage(const QString &message)
{
    statusLabel1->setText(message);
}

void MainWindow::clearStatusMessage()
{
    statusLabel1->clear();
}

void MainWindow::addPlot(float x[], int num, int type, int visible, float wave)
{
    graphItem::ItemType gtype = static_cast<graphItem::ItemType>(type);

    QVector<double> xvet(num);
    for (int i = 0; i < num; i++) {
         xvet[i] = x[i];
    }
    if (plotSettings.getAbscissa() == xpdutils::DVALUE) {
        xvet = xpdutils::dvalue(xvet,wave);
    }

    if (gtype == graphItem::Reflections) {
        if (refl.size() <= nReflections) { //new graphic element required
            graphItem item;
            item.setGtype(graphItem::Reflections);
            item.setColorLine(nReflections);
            item.setVisible(visible);
            item.wave = wave;
            refl.push_back(item);
        }
        refl[nReflections].setX(xvet);

        //Default Name: for more phases use Phase 1, Phase 2, ...
        if (nReflections == 0) {
            refl[0].setName();
        } else if (nReflections == 1) {
            refl[0].setName("Phase "+QString::number(1));
            refl[1].setName("Phase "+QString::number(2));
        } else {
            refl[nReflections].setName("Phase "+QString::number(nReflections+1));
        }
        nReflections++;
    } else if (gtype == graphItem::Intervals) {
        intervalLimit.setVisible(visible);
        intervalLimit.wave = wave;
        intervalLimit.setX(xvet);
    }
}

void MainWindow::makePlot(const QVector<double> &x, const QVector<double> &y, graphItem &item)
{
    double ymin = Minimo(y);
    double ymax = Massimo(y);    
    item.setMin(ymin);
    item.setMax(ymax);
    item.setGraphIndex(ui->plot->graphCount());
    plotWave.append(item.wave);    
    ui->plot->addGraph();
    ui->plot->graph()->setName(item.getName());
    //ui->plot->graph()->setData(x,y,true);    
    ui->plot->graph()->setData(x,y,false);    
    ui->plot->graph()->setPen(item.getPen());
    ui->plot->graph()->setLineStyle(item.getLineConnectionType());
    ui->plot->graph()->setScatterStyle(item.getScatter());
}

void MainWindow::draw()
{
    getYLimits(yLowerRangeNoRef, yUpperRange);

    //Compute space for reflections
    double spaceRef = (yUpperRange - yLowerRangeNoRef) * 0.04; //space for single set of reflections
    double lengthRef = spaceRef * 0.75; // length of the bar
    int nVisibleRef = 0;
    for (int i = 0; i < refl.size(); ++i) {
        if (refl.at(i).isVisible()) nVisibleRef++;
    }    
    //yLowerRange -= nVisibleRef*spaceRef; // make space for bars
    yLowerRange = yLowerRangeNoRef - nVisibleRef*spaceRef; // make space for bars    

    //Draw Reflections
    for (int i = 0; i < refl.size(); i++) {
        QPen mPen(refl[i].getPen());
        refl[i].setGraphIndex(ui->plot->graphCount());
        plotWave.append(refl[i].wave);
        ui->plot->addGraph();        
        ui->plot->graph()->setPen(mPen);
        ui->plot->graph()->setName(refl[i].getName());        
        double ypos = yLowerRange + (refl.size()-i-1)*spaceRef;
        drawReflections(refl[i].getX(),ypos,lengthRef,mPen, refl[i].itemIndexStart, refl[i].itemIndexEnd);
        refl[i].setYPos(ypos);
        refl[i].setLengthRef(lengthRef);
    }

    //Draw Intervals
    drawIntervals();

    if (rescalePlotEnabled()) {        
        ui->plot->yAxis->setRange(yLowerRange,yUpperRange);
        ui->plot->updateMinMax();
        ui->plot->xAxis->rescale();        
    } else {
        if (refl.size() > 0) ui->plot->yAxis->setRange(yLowerRange,yUpperRange);
    }

    ui->plot->setDraggingLegend(true);
    ui->plot->setLegendPosition(1.0,0.0);

    ui->plot->replot();
}

void MainWindow::redrawPlot(bool computeLimits)
{
    if (computeLimits) getYLimits(yLowerRangeNoRef, yUpperRange);

    //Compute space for reflections
    double spaceRef = (yUpperRange - yLowerRangeNoRef) * 0.04; //space for single set of reflections
//FIX: problem with log scale for negative value e long bar
//    spaceRef = spaceRef/100;
    double lengthRef = spaceRef * 0.75; // length of the bar    
    int nVisibleRef = 0;
    for (int i = 0; i < refl.size(); ++i) {
        if (refl.at(i).isVisible()) nVisibleRef++;
    }
    //yLowerRange -= nVisibleRef*spaceRef; // make space for bars
    yLowerRange = yLowerRangeNoRef - nVisibleRef*spaceRef; // make space for bars
//qInfo() << "LOW: " << yLowerRange << yLowerRangeNoRef << spaceRef;

    //Redraw reflections
    int iVis = -1;
    for (int i = 0; i < refl.size(); i++) {
        if (refl.at(i).isVisible()) {            
            ++iVis;
            double ypos = yLowerRange + (nVisibleRef-iVis-1)*spaceRef;
            reDrawReflections(refl[i],ypos,lengthRef);
            refl[i].setYPos(ypos);
            refl[i].setLengthRef(lengthRef);
        }
    }

    ui->plot->yAxis->setRange(yLowerRange,yUpperRange);
    ui->plot->updateMinMax();

    ui->plot->replot();
}

void MainWindow::applyOffset(double yOffset)
{
    //Unapply the last yoffset
    double newOffset = -mLastYOffset + yOffset;

    //No offset is applied to the first graph (i=0)
    double totOffset = 0;
    for (int i = 1; i < ui->plot->graphCount(); i++) {
        //totOffset += exp(i*newOffset);
        totOffset = i*newOffset;
        QCPGraphDataContainer *data = ui->plot->graph(i)->data().data();
        for(QCPGraphDataContainer::iterator it = data->begin(); it != data->end(); it++)
        {
            it->value += totOffset;
        }
    }

    //Apply offset to the y uppper range
    yUpperRange += totOffset;
    if (refl.size() > 0) {
        redrawPlot(false);
    } else {
        ui->plot->yAxis->setRange(yLowerRange,yUpperRange);
    }

    //Save last applied offset;
    mLastYOffset = yOffset;
}

void MainWindow::on_editSelected(QString filename, MyFileDialog *dlg)
{
    dlg->setModal(false);
    TextEditor *m_text = new TextEditor(this);
    m_highlighter = new Highlighter(m_text->getTextEdit()->document());    
    m_highlighter->setDirectivesRule(directives);
    m_highlighter->setCommandsRule(commands);
    m_highlighter->setCommentRule();
    m_text->LoadFile(filename);
    m_text->SetEditable(true);
    m_text->SetSave(false);
    m_text->exec();
    dlg->setModal(true);
}

void MainWindow::zoomGroupTriggered(QAction *action)
{
    qInfo() << "ZoomGroupTriggered";
    if (action == ui->actionSelection_Mode) {
        if (mAction != NoZoom) setAction(NoZoom);
    } else if (action == ui->actionHorizontal_Zoom) {
        if (mAction != HorizontalZoom) setAction(HorizontalZoom);
    } else if (action == ui->actionRectangle_Zoom) {
        if (mAction != RectangleZoom) setAction(RectangleZoom);
    } else if (action == ui->actionPan) {
        if (mAction != Pan) setAction(Pan);
    } else if (action == ui->actionAdd_Background_Point) {
        if (mAction != AddBackgroundPoint) setAction(AddBackgroundPoint, false);
    } else if (action == ui->actionDelete_Background_Point) {
        if (mAction != DeleteBackgroundPoint) setAction(DeleteBackgroundPoint, false);
    } else if (action == ui->actionAdd_Peak) {
        if (mAction != AddPeak) setAction(AddPeak, false);
    } else if (action == ui->actionDelete_Peak) {
        if (mAction != DeletePeak) setAction(DeletePeak, false);
    }
}

void MainWindow::plotStyleClosed(QDialogButtonBox::StandardButton button)
{
    if (button == QDialogButtonBox::Apply) {                
        plotStyleDialog->apply(ui->plot, obs, back, bpoints, calc, diff, cdiff, peaks, refl, plotSettings, plotWave);

    } else if (button == QDialogButtonBox::Ok) {
        if (plotStyleDialog->anyChangeToApply()) {
            plotStyleDialog->apply(ui->plot, obs, back, bpoints, calc, diff, cdiff, peaks, refl, plotSettings, plotWave);
        }

        if (plotStyleDialog->anyChangeApplied()) {
            plotStyleConfig();
        }

    } else if (button == QDialogButtonBox::Cancel) {
        plotStyleDialog->cancel(ui->plot, obs, back, bpoints, calc, diff, cdiff, peaks, refl, plotSettings, plotWave);

    } else if (button == QDialogButtonBox::RestoreDefaults) {
        for (int i = 0; i < plotStyleDialog->obsPen.count(); i++) {
            plotStyleDialog->obsPen[i] = obs.at(i).getDefaultPen(i);
            plotStyleDialog->obsScatter[i] = obs.at(i).getDefaultScatter(i);            
            plotStyleDialog->obsVisible[i] = true;
        }
        QPen backPen = back.getDefaultPen();
        QCPScatterStyle backScatter = bpoints.getDefaultScatter();
        QPen calcPen = calc.getDefaultPen();
        QPen diffPen = diff.getDefaultPen();
        QPen cDiffPen = cdiff.getDefaultPen();
        QPen peaksPen = peaks.getDefaultPen();
        for (int i = 0; i < plotStyleDialog->reflPen.count(); i++) {
            plotStyleDialog->reflPen[i] = refl.at(i).getDefaultPen(i);
            plotStyleDialog->reflVisible[i] = true;
            qInfo() << "Refl visible: " << graphItem().isVisible();
        }
        PlotSettings defaults;
        defaults.restoreDefaults();
        plotStyleDialog->setWidgets(false, backPen, backScatter, calcPen, diffPen, cDiffPen, peaksPen, defaults,
                                    back.isVisible(), bpoints.isVisible(), calc.isVisible(), diff.isVisible(), cdiff.isVisible(), peaks.isVisible());
    }
}

void MainWindow::plotStyleConfig()
{
    QSettings settings;

    for (int i = 0; i < plotStyleDialog->obsPen.count(); i++ ) {        
        settings.setValue(obs[i].getKeyString("pen", i),obs.at(i).getPen());
        settings.setValue(obs[i].getKeyString("scatterPen", i), obs.at(i).getScatter().pen());
        settings.setValue(obs[i].getKeyString("scatterStyle", i), obs.at(i).getScatter().shape());
    }

    settings.setValue(back.getKeyString("pen", -1),back.getPen());
    settings.setValue(bpoints.getKeyString("scatterPen", -1),bpoints.getScatter().pen());
    settings.setValue(bpoints.getKeyString("scatterStyle", -1),bpoints.getScatter().shape());

    settings.setValue(calc.getKeyString("pen", -1),calc.getPen());
    settings.setValue(diff.getKeyString("pen", -1),diff.getPen());
    settings.setValue(cdiff.getKeyString("pen", -1),cdiff.getPen());
    settings.setValue(peaks.getKeyString("pen", -1),peaks.getPen());

    for (int i = 0; i < plotStyleDialog->reflPen.count(); i++) {
        settings.setValue(refl[i].getKeyString("pen", i),refl.at(i).getPen());
    }
    plotSettings.write();
}

void MainWindow::on_actionAutoscale_triggered()
{
    // old method
    //Compute min and max along the yaxis in the visible range
    //Provare anche per ogni plot QCPRange range = ui->plot->graph(0)->getKeyRange(found);
//    double yLowerRange = DBL_MAX;
//    double yUpperRange = -DBL_MAX;
//    double xLowerRange = ui->plot->xAxis->range().lower;
//    double xUpperRange = ui->plot->xAxis->range().upper;
//    for (int i = 0; i < ui->plot->graphCount(); i++) {
//        QCPGraphDataContainer::const_iterator it = ui->plot->graph(i)->data()->constBegin();
//        QCPGraphDataContainer::const_iterator end = ui->plot->graph(i)->data()->constEnd();;
//        while (it != end)
//        {
//            if (it->key >= xLowerRange && it->key <= xUpperRange) {
//                if (it->value < yLowerRange) yLowerRange = it->value;
//                if (it->value > yUpperRange) yUpperRange = it->value;
//            }
//            it++;
//        }
//    }
//    if (ui->plot->yAxis->range().upper != yUpperRange) {
//        //double yLower = ui->plot->yAxis->range().lower;
//        ui->plot->yAxis->setRange(yLower(),yUpperRange);
//        ui->plot->replot();
//        qInfo() << "Autoscale: " << yUpperRange;
//    }

    //altro metodo
    double Lower = ui->plot->xAxis->range().lower;
    double Upper = ui->plot->xAxis->range().upper;
    //yLowerRange = Lower;
    double yUpperAutoScale = -DBL_MAX;
    double yLowerAutoScale = DBL_MAX;
    for (int i = 0; i < ui->plot->graphCount(); i++) {
        if (!ui->plot->graph(i)->data()->isEmpty() && ui->plot->graph(i)->visible()) {
            //int index_left = ui->plot->graph(i)->findBegin(Lower, false);
            //if (ui->plot->graph(0)->data()->at(index_left)->key < Lower) index_left++;
            //int index_right = ui->plot->graph(i)->findEnd(Upper, false) - 1;
            //double max = max_element(ui->plot->graph(i)->data()->at(index_left),ui->plot->graph(i)->data()->at(index_right));
            //qInfo() << "G: " << i << "Index L: " << index_left << "Index R: " << index_right << ui->plot->graph(i)->data()->size();
            QCPGraphDataContainer::const_iterator begin = ui->plot->graph(i)->data()->findBegin(Lower, false);
            QCPGraphDataContainer::const_iterator end = ui->plot->graph(i)->data()->findBegin(Upper, false) - 1;
            // end > begin for selection between two points (empty range!)
            if (begin <= end) {
//warning
//                const QCPGraphData* maxData = std::max_element(begin, end,
//                                              [](QCPGraphData a, QCPGraphData b){return a.value < b.value;});
                const auto maxData = std::max_element(begin, end,
                                      [](const QCPGraphData& a, const QCPGraphData& b){return a.value < b.value;});
                const auto minData = std::min_element(begin, end,
                                      [](const QCPGraphData& a, const QCPGraphData& b){return a.value < b.value;});
                if (yUpperAutoScale < maxData->value) yUpperAutoScale = maxData->value;
                if (yLowerAutoScale > minData->value) yLowerAutoScale = minData->value;
            }
        }
    }
    if (ui->plot->yAxis->range().upper != yUpperAutoScale) {
        bool lowerChanged = (ui->plot->yAxis->range().lower != yLowerAutoScale);
        double yLowerRangeSav;
        if (lowerChanged) {
            yLowerRangeSav = yLowerRange;
            yLowerRangeNoRef = yLowerAutoScale;
        }
        //TOFIX: improve this part, yUpperRange and yLowerRange should not be changed to avoid problem at reset
        double yUppperRangeSav = yUpperRange;
        yUpperRange = yUpperAutoScale;
        redrawPlot(false);
        yUpperRange = yUppperRangeSav;
        if (lowerChanged) {
            yLowerRange = yLowerRangeSav;
        }
    }
}

void MainWindow::horzScrollBarChanged(int value)
{
    //qInfo() << "ini================================ horzScrollChange";
    double xUpperRange,xLowerRange;
    ui->plot->getMinMax(xLowerRange,xUpperRange);
    double plotWidth = xUpperRange - xLowerRange;
    double lower;
    if (ui->plot->xAxis->rangeReversed()) {
        lower = xUpperRange - plotWidth * value /scrollBarLen - ui->plot->xAxis->range().size();
    } else {
        lower = plotWidth * value /scrollBarLen + xLowerRange;
    }
    //qInfo() << "HScrollChanged: " << value << "lower: " << lower << "xLower: " << xLowerRange << "range.lower: " << ui->plot->xAxis->range().lower;
    if (qAbs(ui->plot->xAxis->range().lower - lower) > 0.01) // if user is dragging plot, we don't want to replot twice
    {
        //qInfo() << "horzScrollBarChanged: " << value;
        double upper = lower + ui->plot->xAxis->range().size();
        //qInfo() << "HBar Changed, value: " << value << "L: " << lower << "U: " << upper;
        ui->plot->xAxis->setRange(lower,upper);
        ui->plot->replot();
    }
    //qInfo() << "end================================ horzScrollChange";
}

void MainWindow::xAxisChanged(QCPRange range)
{
    //qInfo() << "ini================================ xAxisChanged";
    //Avoid zoom and drag outside the range
    double xUpperRange,xLowerRange;
    ui->plot->getMinMax(xLowerRange,xUpperRange);
    QCPRange boundedRange = range;
    //qInfo() << "xAxisChanged: " << boundedRange.lower << "-" << boundedRange.upper << xLowerRange << "-" << xUpperRange;

    if (boundedRange.size() > xUpperRange - xLowerRange) {
        boundedRange = QCPRange(xLowerRange,xUpperRange);
    } else {
        double oldSize = boundedRange.size();
        if (boundedRange.lower < xLowerRange) {
            boundedRange.lower = xLowerRange;
            boundedRange.upper = xLowerRange + oldSize;
        }
        if (boundedRange.upper > xUpperRange) {
            boundedRange.upper = xUpperRange;
            boundedRange.lower = xUpperRange - oldSize;
        }        
    }

    ui->plot->xAxis->setRange(boundedRange);

    double plotWidth = xUpperRange - xLowerRange;
    double viewportWidth = boundedRange.size() / plotWidth;    

    //if (viewportWidth > 1.0)
    if (viewportWidth > 0.9999) {
        ui->horizontalScrollBar->hide();
    }
    else
    {
        //qInfo() << "Viewport: " << viewportWidth << " plotWidth: " << plotWidth << " xLowerRange: " << xLowerRange;
        ui->horizontalScrollBar->show();        
        ui->horizontalScrollBar->setPageStep(qRound(scrollBarLen*viewportWidth));
        ui->horizontalScrollBar->setSingleStep((std::min)(scrollBarLen / 100, ui->horizontalScrollBar->pageStep()));
        ui->horizontalScrollBar->setRange(0,scrollBarLen-ui->horizontalScrollBar->pageStep());
        //qInfo() << "Step: " << ui->horizontalScrollBar->pageStep() << "Range: " << boundedRange.size();
        double value;
        if (ui->plot->xAxis->rangeReversed()) {
            value = scrollBarLen*(xUpperRange - boundedRange.upper)/plotWidth;
        } else {
            value = scrollBarLen*(boundedRange.lower - xLowerRange)/plotWidth;
        }
        //qInfo() << "Value: " << value;
        ui->horizontalScrollBar->setValue(qRound(value));
    }

    //qInfo() << "Value: " << ui->horizontalScrollBar->value() << "L: " << boundedRange.lower << "Center: " << boundedRange.center() << "U: " << range.upper;
    //qInfo() << "end================================ xAxisChanged";
}

void MainWindow::myMoveEvent(QMouseEvent *event)
{
    double x_val = ui->plot->xAxis->pixelToCoord(event->pos().x());
    double y_val = ui->plot->yAxis->pixelToCoord(event->pos().y());

    double x_end = ui->plot->xAxis->range().upper;

    int graphIndex = -1;
    int indexMin = 0;
    double distMin = DBL_MAX;

    //Traverse each curve, finding the closest point to the cursor point and the corresponding curve index
    for (int i = 0; i < ui->plot->xAxis->graphs().count(); i++) {
        if (!ui->plot->graph(i)->data()->isEmpty()) {
            //Find a key value index closest to the curve by x value
            int index = 0;
            if (x_val > x_end) {
                index = ui->plot->graph(i)->data()->size() - 1;
            } else {
                int index_left = ui->plot->graph(i)->findBegin(x_val, true);
                int index_right = index_left + 1; //ui->plot->graph(0)->findEnd(x_val, true);
                double dif_left = abs(ui->plot->graph(i)->data()->at(index_left)->key - x_val);
                double dif_right = abs(ui->plot->graph(i)->data()->at(index_right)->key - x_val);
                if (dif_left < dif_right)
                    index = index_left;
                else
                    index = index_right;
            }

            //Compute distance from the cursor position
            double dx = ui->plot->graph(i)->data()->at(index)->key - x_val;
            double dy = ui->plot->graph(i)->data()->at(index)->value - y_val;
            double dist = dx*dx + dy*dy;
            if (dist < distMin) {
                distMin = dist;
                indexMin = index;
                graphIndex = i;
            }
        }
    }

    const QString twoTheta = xpdutils::abscissaString(xpdutils::TTHETA);
    double wave = -1;
    if (graphIndex >= 0) {
        double xp_val = ui->plot->graph(graphIndex)->data()->at(indexMin)->key;
        double yp_val = ui->plot->graph(graphIndex)->data()->at(indexMin)->value;

        wave = plotWave.at(graphIndex);
        if (wave > 0) {            
            QString xString1 = xpdutils::abscissaString(plotSettings.getAbscissa());
            QString xString2;
            double xp2_val;
            if (plotSettings.getAbscissa() == xpdutils::TTHETA) {
                xString2 = "d";
                xp2_val = xpdutils::dvalue(xp_val, wave);
            } else {
                xString2 = twoTheta;
                xp2_val = xpdutils::tthvalue(xp_val, wave, plotSettings.getAbscissa());
            }
            //xString2 = (plotSettings.getAbscissa() == xpdutils::TTHETA) ? twoTheta : "d";
            //xp2_val = xpdutils::tthvalue(xp_val, wave, plotSettings.getAbscissa());
            QString label1 = QString("%1: %2 I: %3").arg(xString1).arg(x_val).arg(y_val);
            ui->plotLabel1->setText(fontMetrics().elidedText(label1,Qt::ElideRight,ui->plotLabel1->size().width()-10));

            QString label2 = QString("Count: #%1 (%2, %3) %4: %5 Graph: %6").
                    arg(indexMin+1).
                    arg(xp_val).
                    arg(yp_val).
                    arg(xString2).
                    arg(xp2_val).
                    arg(ui->plot->graph(graphIndex)->name());
            ui->plotLabel2->setText(fontMetrics().elidedText(label2,Qt::ElideRight,ui->plotLabel2->size().width()-10));
            //ui->plotLabel1->setText(QString("%1: %2 I: %3").arg(xString1).arg(x_val).arg(y_val));
            // ui->plotLabel2->setText(QString("Count: #%1 (%2, %3) %4: %5 Graph: %6").
            //                         arg(indexMin+1).
            //                         arg(xp_val).
            //                         arg(yp_val).
            //                         arg(xString2).
            //                         arg(xp2_val).
            //                         arg(ui->plot->graph(graphIndex)->name()));
        }
    }

    //Find the closest reflection and the corrisponding phase
    int refIndex = -1;    
    int phaseIndex = -1;
    if (refl.count() > 0) {
        QVector<double> yPos(refl.count());
        for (int i = 0; i < refl.count(); i++) yPos[i] = refl[i].getYPos();

        phaseIndex = NR::locateClosest(yPos, y_val);

        if (plotSettings.getAbscissa() == xpdutils::TTHETA) {
            refIndex = refl[phaseIndex].findLocation(x_val);
        } else {
            if (wave > 0) {
                double x_val2 = xpdutils::tthvalue(x_val, wave, plotSettings.getAbscissa());
                //It works because refl[i].x should be in 2theta
                refIndex = refl[phaseIndex].findLocation(x_val2);
            }
        }
    }

    if (refIndex >= 0) {
        int hkl[3];
        int err;
        float tth, dval;
        phaseIndex++;
        refIndex++;
        get_reflection_info(phaseIndex, refIndex, hkl, &tth, &dval, &err);
        QString label3 = QString("Refl: #%1 H: %2 K: %3 L: %4 %5: %6 d: %7 Phase: %8").
                         arg(refIndex).
                         arg(hkl[0]).arg(hkl[1]).arg(hkl[2]).
                         arg(twoTheta).
                         arg(tth).arg(dval).
                         arg(phaseIndex);
        ui->plotLabel3->setText(fontMetrics().elidedText(label3,Qt::ElideRight,ui->plotLabel3->size().width()-10));
        // ui->plotLabel3->setText(QString("Refl: #%1 H: %2 K: %3 L: %4 %5: %6 d: %7 Phase: %8").
        //                         arg(refIndex).
        //                         arg(hkl[0]).arg(hkl[1]).arg(hkl[2]).
        //                         arg(twoTheta).
        //                         arg(tth).arg(dval).
        //                         arg(phaseIndex));
    }
}

MainWindow* getMainWindow()
{
    foreach (QWidget *w, qApp->topLevelWidgets())
        if (MainWindow* mainWin = qobject_cast<MainWindow*>(w))
            return mainWin;
    return nullptr;
}

//
//  File Menu
//

void MainWindow::createRecentActions()
{
    QAction* recentFileAction;
    for(int i = 0; i < maxFileNr; i++) {
        recentFileAction = new QAction(this);
        recentFileAction->setVisible(false);
        connect(recentFileAction,SIGNAL(triggered()),this,SLOT(openRecentFile()));
        ui->menuRecent->addAction(recentFileAction);
        recentFileActionList.append(recentFileAction);
    }

    updateRecentFileActions();
}

void MainWindow::updateRecentFileActions()
{
    QSettings settings;
    //QStringList recentFilePaths = settings.value(EXPO_RECENT_FILES_KEY).toStringList();

    QVector<RecentFileInfo> recentFiles;
    int size = settings.beginReadArray(EXPO_RECENT_FILES_KEY);
    for (int i = 0; i < size; i++) {
        settings.setArrayIndex(i);
        RecentFileInfo info;
        info.fileName = settings.value("FileName").toString();
        info.fileType = settings.value("FileType").toString();
        recentFiles.append(info);
    }
    settings.endArray();

    int numRecentFiles = qMin(recentFiles.size(),static_cast<int>(maxFileNr));    

    for(int i = 0; i < numRecentFiles; ++i) {
        recentFileActionList.at(i)->setText(QString::number(i+1)+". "+recentFiles.at(i).fileName);
        recentFileActionList.at(i)->setData(recentFiles.at(i).fileName);
        recentFileActionList.at(i)->setVisible(true);
    }

    for(int i = numRecentFiles; i < maxFileNr; i++) {
        recentFileActionList.at(i)->setVisible(false);
    }
}

void MainWindow::setRecentFiles(const QString &fullFileName, const QString &fileType)
{
    QSettings settings;
    QVector<RecentFileInfo> recentFiles;
    int size = settings.beginReadArray(EXPO_RECENT_FILES_KEY);    
    for (int i = 0; i < size; i++) {
        settings.setArrayIndex(i);
        RecentFileInfo info;
        info.fileName = settings.value("FileName").toString();
        info.fileType = settings.value("FileType").toString();
        if (info.fileName != fullFileName) {
            recentFiles.append(info);
        }
    }
    settings.endArray();
    recentFiles.prepend({fullFileName,fileType});
    while (recentFiles.size() > maxFileNr)
        recentFiles.removeLast();

    settings.beginWriteArray(EXPO_RECENT_FILES_KEY);
    for (int i = 0; i < recentFiles.size(); i++) {
        settings.setArrayIndex(i);
        settings.setValue("FileName", recentFiles.at(i).fileName);
        settings.setValue("FileType", recentFiles.at(i).fileType);
    }
    settings.endArray();
    //

    // if you have several istances of Mainwindow present,
    // call updateRecentFileActions on all top-level windows and
    // all recent files menu will be equal
    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
        QMainWindow *mainwindow = qobject_cast<QMainWindow *>(widget);
        if (mainwindow)
            updateRecentFileActions();
    }
}

QString MainWindow::getLastFile() const
{
    if (!currentFile.isEmpty()) {
        return currentFile;
    } else {
        QSettings settings;
        int size = settings.beginReadArray(EXPO_RECENT_FILES_KEY);
        for (int i = 0; i < size; i++) {
            settings.setArrayIndex(i);
            RecentFileInfo info;
            info.fileName = settings.value("FileName").toString();
            info.fileType = settings.value("FileType").toString();
            if (info.fileType == QVariant(RecentFileType::Input).toString()) {
                return info.fileName;
            }
        }
        settings.endArray();
    }
    return QString();
}

void MainWindow::setCurrentFile(const QString &fullFileName, const QString &fileType)
{
    if (!fullFileName.isEmpty()) {
        currentFile = fullFileName;
        setWindowTitle(currentFile);

        setRecentFiles(fullFileName, fileType);
    }
}

void MainWindow::loadExpFile(const QString &fileName, const QString outName)
{
    if (!fileName.isEmpty()) {
        //currentProjectFile.clear();  //Start new project
        clearProjectFile();
        QString fileNameIn = QDir::toNativeSeparators(fileName);
        QString fileNameOut = QDir::toNativeSeparators(outName);
        QString currentDir = fileutils::setCurrentDirFromFile(fileNameIn);
        if (!currentDir.isEmpty()) {
            QSettings settings;
            settings.setValue(DEFAULT_DIR_KEY,currentDir);
        }
        setCurrentFile(fileNameIn,QVariant::fromValue(RecentFileType::Input).toString());
        QFileInfo fi(fileNameIn);
        QString fileIn = fi.fileName();
        outputFileName = fileNameOut;
        int ier = 0;
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        putlab95_(fileIn.toStdString().c_str(),fileNameOut.toStdString().c_str(),&ier,fileIn.length(),fileNameOut.length());
        QApplication::restoreOverrideCursor();
    }
}

// void MainWindow::on_actionLoad_Go_triggered()
// {
//     QSettings settings;

//     // Custom file dialog
//     MyFileDialog *dlg = new MyFileDialog(this);
//     connect(dlg, SIGNAL(editSelected(QString,MyFileDialog*)), this, SLOT(on_editSelected(QString,MyFileDialog*)));
//     dlg->setNameFilter("Expo Input Files (*.exp);;All Files (*.*)");
//     dlg->setWindowTitle(tr("Load Expo Input File"));
//     QString lastFile = getLastFile();
//     if (!lastFile.isEmpty()) {
//         dlg->selectFile(lastFile);
//     } else {
//         dlg->setDirectory(settings.value(DEFAULT_DIR_KEY).toString());
//     }
//     if (dlg->exec() == QDialog::Accepted) {
//         QString fileName = dlg->selectedFiles().at(0);
//         QString outName = dlg->getOutputFile();
//         loadExpFile(fileName, outName);
//     }
// }

void MainWindow::on_actionLoad_Go_triggered()
{
    if (mFileDialog == nullptr) {
        mFileDialog = new MyFileDialog(this);
        connect(mFileDialog, SIGNAL(editSelected(QString,MyFileDialog*)), this, SLOT(on_editSelected(QString,MyFileDialog*)));
        mFileDialog->setNameFilter("Expo Input Files (*.exp);;All Files (*.*)");
        mFileDialog->setWindowTitle(tr("Load Expo Input File"));
        QString lastFile = getLastFile();
        QSettings settings;
        if (!lastFile.isEmpty()) {
            mFileDialog->selectFile(lastFile);
        } else {
            mFileDialog->setDirectory(settings.value(DEFAULT_DIR_KEY).toString());
        }
    }
    if (mFileDialog->exec() == QDialog::Accepted) {
        QString fileName = mFileDialog->selectedFiles().at(0);
        QString outName = mFileDialog->getOutputFile();
        loadExpFile(fileName, outName);
    }
}

void MainWindow::openRecentFile()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action) {
        QString fileIn = action->data().toString();
        if (!QFile::exists(fileIn)) {
            QMessageBox::critical(this,"Error",fileIn + " does not exist.");
            return;
        }

        //Extract the fileType from action text and settings
        int index = action->text().section(".",0,0).toInt()-1;
        QSettings settings;
        settings.beginReadArray(EXPO_RECENT_FILES_KEY);
        settings.setArrayIndex(index);
        QString fileType = settings.value("FileType").toString();
        settings.endArray();

        QMetaEnum metaEnum = QMetaEnum::fromType<RecentFileType>();
        int type = metaEnum.keyToValue(fileType.toStdString().c_str());
        int err;
        switch (type) {
        case RecentFileType::Input:
        {
            QString fileOut = fileutils::removeExtension(fileIn)+".out";
            loadExpFile(fileIn, fileOut);
            break;
        }
        case RecentFileType::Data:
        {
            fileIn = QDir::toNativeSeparators(fileIn);
            QString fileOut = fileutils::removeExtension(fileIn)+".out";
            open_diffraction_patt(fileIn.toStdString().c_str(), fileIn.length(),
                                  fileOut.toStdString().c_str(), fileOut.length(),
                                  0, &err);
            if (!err) {
                currentFile = fileIn;
                outputFileName = fileOut;
                setWindowTitle(currentFile);
            }
            break;
        }
        case RecentFileType::Structure:
            fileIn = QDir::toNativeSeparators(fileIn);
            //mr la successiva per abilitare il menù Modify in Jav se la finestra è già aperta
            Properties::FromImport = true;
            importa_file(fileIn.toStdString().c_str(), fileIn.length(), 0);
            ui->crystalWidget->setModelNameFromFile(fileIn);
            Properties::FromImport = false;
            break;
        case RecentFileType::Project:
            err = readprojectwin(fileIn.toStdString().c_str(), fileIn.length());
            if (!err) {
                currentFile = fileIn;
                outputFileName = fileutils::removeExtension(fileIn)+".out";
                setWindowTitle(currentFile);
            }
            break;
        }
    }
}

QString MainWindow::loadExamplesFolder()
{
    QString examplesDestDir = QDir::homePath()+"/share/"+qApp->applicationName()+"/examples";
    if (!QDir(examplesDestDir).exists()) {
        QString examplesStartDir = QCoreApplication::applicationDirPath()+"/../share/expo/examples";
        fileutils::copyDirectoryFiles(examplesStartDir,examplesDestDir,false);
    }

    return examplesDestDir;
}

void MainWindow::on_actionLoad_Examples_triggered()
{
    QString examplesDestDir = loadExamplesFolder();

    if (mFileDialog == nullptr) {
        mFileDialog = new MyFileDialog(this);
        connect(mFileDialog, SIGNAL(editSelected(QString,MyFileDialog*)), this, SLOT(on_editSelected(QString,MyFileDialog*)));
        mFileDialog->setNameFilter("Expo Input Files (*.exp);;All Files (*.*)");
        mFileDialog->setWindowTitle(tr("Load Expo Input File"));
    }
    mFileDialog->setDirectory(examplesDestDir);

    if (mFileDialog->exec() == QDialog::Accepted) {
        QString fileName = mFileDialog->selectedFiles().at(0);
        QString outName = mFileDialog->getOutputFile();
        loadExpFile(fileName, outName);
    }
}

void MainWindow::on_actionImport_Structure_triggered()
{
    openImportStructure();
}

void MainWindow::onActionImagePowderPatternTriggered()
{
    QSettings settings;
    QString selectedFilter = settings.value(EXPO_IMAGE_FILTER_KEY).toString();
    int code = 0;
    QString exts = "PNG (*.png);;"
                   "PDF (*.pdf);;"
                   "BMP (*.bmp);;"
         "JPEG (*.jpeg *jpg *jpe)";    
    QString fileName = SaveDialog::run(this,
                                       tr("Export Image As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),
                                       exts,
                                       "png",
                                       selectedFilter,
                                       &code);
    settings.setValue(EXPO_IMAGE_FILTER_KEY,selectedFilter);
    if (!fileName.isEmpty()) {
        ImgSettingsDialog dialog(this, QFileInfo(fileName).suffix());
        if (dialog.exec()) {
            settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
            if (dialog.isTransparent()) {
                // remove background for transparency
                ui->plot->setBackground(QBrush(Qt::NoBrush));
                ui->plot->axisRect()->setBackground(QBrush(Qt::NoBrush));
            }
            if( fileName.endsWith(".png") ){
                ui->plot->savePng( fileName, ui->plot->width(), ui->plot->height(), dialog.scale() );
            }
            if( fileName.endsWith(".jpg") || fileName.endsWith(".jpeg") || fileName.endsWith(".jpe")){
                ui->plot->saveJpg( fileName, ui->plot->width(), ui->plot->height(), dialog.scale() );
            }
            if( fileName.endsWith(".pdf") ){
                ui->plot->savePdf( fileName, ui->plot->width(), ui->plot->height());
            }
            if (dialog.isTransparent()) {
                // add background
                plotSettings.applyToBackground(ui->plot);
                plotSettings.applyToLayer(ui->plot);
            }
        }
    }
}

void MainWindow::on_actionPowder_Pattern_triggered()
{
    QSettings settings;
    QString selectedFilter = "xy";
    int code = 0;
    QString exts = "XY files (*.xy)";
    QString fileName = SaveDialog::run(this,
                                       tr("Export Data As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),
                                       exts,
                                       "xy",
                                       selectedFilter,
                                       &code);
    if (!fileName.isEmpty()) {
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
        int icode = 16;
        esportanew_(&icode, fileName.toStdString().c_str(), fileName.length());
    }
}

// void MainWindow::onActionExportStructureTriggered()
// {
//     // int whichModel = IsValidModel();
//     // if(whichModel == -1)
//     //     return;
//     // QString nome = ModelInfo[whichModel].fileName;

//     QSettings settings;
//     QString selectedFilter = settings.value(EXPORT_FILTER_KEY).toString();
//     int code = 2;
//     QString fileName = SaveDialog::run(this,
//                                        tr("Export Structure As"),
//                                        settings.value(DEFAULT_DIR_KEY).toString(),
//                                        QFileInfo(currentFile).baseName(),
//                                        FilesToExport,
//                                        "cif",
//                                        selectedFilter,
//                                        &code);
//     settings.setValue(EXPORT_FILTER_KEY,selectedFilter);
//     if (!fileName.isEmpty()) {
//         settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());

//         int tipo = CrystalViewWidget::getExportFileCode(selectedFilter);
//         exportFile(fileName,tipo);
//     }
// }

void MainWindow::on_actionReflections_triggered()
{
    QSettings settings;
    QString selectedFilter = "xy";
    int code = 0;
    QString exts = "HKL files (*.hkl)";
    QString fileName = SaveDialog::run(this,
                                       tr("Export Reflections As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),
                                       exts,
                                       "hkl",
                                       selectedFilter,
                                       &code);
    if (!fileName.isEmpty()) {
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
        int icode = 17;
        esportanew_(&icode, fileName.toStdString().c_str(), fileName.length());
    }
}

void MainWindow::on_actionImport_Diffraction_Pattern_triggered()
{
    QSettings settings;
    QString selectedFilter = "All Powder Diffraction Data (*.dat *.xy *.rtv *.gda *.xye *.xrdml *.pow)";
    QString exts = "All files (*.*);;"
    "All Powder Diffraction Data (*.dat *.xy *.rtv *.gda *.xye *.xrdml *.pow);;"
    "ASCII profile [start,step,end,intensities] (*.dat *.pow);;"
    "XY profile [2theta and intensities in two colomuns] (*.xy *.pow);;"
    "GSAS data (*.gda);;"
    "CIF powder data (*.rtv *.cif);;"
    "CCDC Mercury data (*.xye);;"
    "Siemens data (*.uxd);;"
    "Sietronics Sieray data (*.cpi);;"
    "XDD data (*.xdd);;"
    "DBWS data (*.dbw);;"
    "XDA data (*.xda);;"
    "Philips UDF data (*.udf);;"
    "PANalytical XRDML data (*.xrdml)";

    QStringList files = QFileDialog::getOpenFileNames(this,
                tr("Import Diffraction Pattern From"),
                settings.value(DEFAULT_DIR_KEY).toString(),
                exts, &selectedFilter);

    if (!files.isEmpty()) {
        QFileInfo info(files.at(0));
        QString outFile = info.path() + QDir::separator() + info.baseName() + ".out";
        outFile = QDir::toNativeSeparators(outFile);
        int nerr = 0;
        fileutils::setCurrentDirFromFile(files.at(0));
        for (int i = 0; i < files.size(); i++) {
            int err;
            QString filename = QDir::toNativeSeparators(files.at(i));
            open_diffraction_patt(filename.toStdString().c_str(), filename.length(),
                                  outFile.toStdString().c_str(), outFile.length(),
                                  i, &err);
            if (err) {
                nerr++;
            }
        }
        if (nerr < files.size()) {
            QString filename = QDir::toNativeSeparators(files.at(0));
            settings.setValue(DEFAULT_DIR_KEY,QFileInfo(filename).absolutePath());
            outputFileName = outFile;
            setCurrentFile(filename,QVariant::fromValue(RecentFileType::Data).toString());
        }
    }
}

void MainWindow::on_actionLoad_Project_triggered()
{
    QSettings settings;
    QString selectedFilter = "expo";
    QString exts = "Expo Project (*.expo)";

    QString fileName = QFileDialog::getOpenFileName(this,
        tr("Load Expo Project"), settings.value(DEFAULT_DIR_KEY).toString(),
        exts, &selectedFilter);
    if (!fileName.isEmpty()) {
        fileName = QDir::toNativeSeparators(fileName);
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());
        int err = readprojectwin(fileName.toStdString().c_str(), fileName.length());
        if (!err) {            
            setCurrentFile(fileName,QVariant::fromValue(RecentFileType::Project).toString());
        }
    }

}

void MainWindow::saveFile(const QString &fileName)
{
    writeprojectwin(fileName.toStdString().c_str(), fileName.length());
    projectFileSaved = true;
}

void MainWindow::on_actionSave_Project_triggered()
{
    if (currentProjectFile.isEmpty())
        on_actionSave_Project_As_triggered();
    else
        saveFile(currentProjectFile);
}

void MainWindow::on_actionSave_Project_As_triggered()
{
    QSettings settings;
    QString selectedFilter = "expo";    
    QString exts = "Expo Project (*.expo)";
    QString fileName = SaveDialog::run(this,tr("Save Project As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),exts,
                                       "expo",selectedFilter);
    if (!fileName.isEmpty()) {
        fileName = QDir::toNativeSeparators(fileName);
        currentProjectFile = fileName;
        saveFile(fileName);
    }
}

//
//  Pattern Menu
//

void MainWindow::on_actionRange_triggered()
{
    rangeDialog->show();
    rangeDialog->setRange(ui->plot);
}

void MainWindow::actionIntervalsTriggered()
{
    intervalsDialog->setIntervals(ui->plot);
    intervalsDialog->show();
}

void MainWindow::on_actionBackground_triggered()
{
    backgroundDialog->setBackground();
    backgroundDialog->show();
}

void MainWindow::actionExtraBackgroundTriggered()
{
    extraBackgroundDialog->setBackground(ui->plot);
    extraBackgroundDialog->show();
}

void MainWindow::on_actionSmoothing_triggered()
{
    smoothingDialog->setSmoothing();
    smoothingDialog->show();
}

void MainWindow::on_actionPeak_Search_triggered()
{
    run_peaksearchwin();
}

void MainWindow::on_actionPeak_Search_Conditions_triggered()
{
    peakSearchDialog->setOptions();
    peakSearchDialog->show();    
}

void MainWindow::on_actionLoad_Peaks_triggered()
{
    QSettings settings;
    QString selectedFilter = "2 theta values (*.dat *.txt *.pea)";
    QString exts = "2 theta values (*.dat *.txt *.pea);;"
                   "d values (*.dat *.txt *.pea)";
    QString fileName = QFileDialog::getOpenFileName(this,
        tr("Load File"), settings.value(DEFAULT_DIR_KEY).toString(),
        exts, &selectedFilter);

    if (!fileName.isEmpty()) {
        int tipo = (selectedFilter.startsWith("2 theta")) ? 1 : 2;
        int ier;
        LoadPeaksC(fileName.toStdString().c_str(), fileName.length(), tipo, &ier);
    }
}

void MainWindow::on_actionSave_Peaks_triggered()
{
    QSettings settings;
    QString selectedFilter = "2 theta values (*.dat *.txt *.pea)";
    QString exts = "2 theta values (*.dat *.txt *.pea);;"
                   "d values (*.dat *.txt *.pea);;"
                   "Input File for DICVOL (*.dat);;"
                   "Input File for McMaille (*.dat);;"
                   "Peak list (*.txt)";
    QString fileName = SaveDialog::run(this,tr("Save Peaks As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),exts,
                                       "txt",selectedFilter);
    if (!fileName.isEmpty()) {
        QStringList list = exts.split(";;");
        int tipo = list.lastIndexOf(selectedFilter) + 1;
        SavePeaksC(fileName.toStdString().c_str(), fileName.length(),tipo);
    }
}

void MainWindow::on_actionN_Treor_triggered()
{
    indexingDialog->setIndexing(IndexingDialog::NTREOR09);
    indexingDialog->show();
}

void MainWindow::on_actionDICVOL_triggered()
{
    indexingDialog->setIndexing(IndexingDialog::DICVOL);
    indexingDialog->show();
}

void MainWindow::on_actionMcMaille_triggered()
{
    indexingDialog->setIndexing(IndexingDialog::MCMAILLE);
    indexingDialog->show();
}

void MainWindow::on_actionOutputN_TREOR09_triggered()
{
    if (!indexingDialog->getOutFileName(IndexingDialog::NTREOR09).isNull()) {
        textViewer->LoadFile(indexingDialog->getOutFileName(IndexingDialog::NTREOR09));
        textViewer->show();
    } else
        QMessageBox::warning(this, tr("Error"), tr("Can't find the NTREOR09 output file"));
}

void MainWindow::on_actionOutputDICVOL14_triggered()
{
    if (!indexingDialog->getOutFileName(IndexingDialog::DICVOL).isNull()) {
        textViewer->LoadFile(indexingDialog->getOutFileName(IndexingDialog::DICVOL));
        textViewer->show();
    } else
        QMessageBox::warning(this, tr("Error"), tr("Can't find the DICVOL14 output file"));
}

void MainWindow::on_actionCell_List_triggered()
{    
    open_cell_list();
}

void MainWindow::exportCellList()
{
    QSettings settings;
    QString exts = "*.txt;;*.exp";
    QString selectedFilter = "*.txt";
    QString fileName = SaveDialog::run(this,tr("Export List As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(currentFile).baseName(),exts,
                                       "txt",selectedFilter);
    if (!fileName.isEmpty()) {
        save_cell_list(fileName.toStdString().c_str(), fileName.length());
    }
}

void MainWindow::openCellDialog(cell_info_type cell[], int ncell, int &selectedCell)
{
    saveEnabledActions();
    enableActions(MainWindow::DialogOpenAction);
    cellListDialog->createModel(cell, ncell, selectedCell);
    cellListDialog->show();
    cellListDialog->waitSignalIsEmitted();
    if (cellListDialog->result() == QDialog::Accepted)
        selectedCell = cellListDialog->selectedCell();
    else
        selectedCell = 0;
    restoreEnabledActions();
}

void MainWindow::on_actionOutputMcMaille_triggered()
{
    if (!indexingDialog->getOutFileName(IndexingDialog::MCMAILLE).isNull()) {
        textViewer->LoadFile(indexingDialog->getOutFileName(IndexingDialog::MCMAILLE));
        textViewer->show();
    } else
        QMessageBox::warning(this, tr("Error"), tr("Can't find the MCMAILLE output file"));
}

void MainWindow::openSpgDialog(double cell[6], int &sgIndex, QString &spg, QString &content, int &findspace, int &ret)
{
    spgDialog->setData(cell, sgIndex, spg, content, findspace);
    ret = spgDialog->exec();
    if (ret == QDialog::Accepted) {
        content = spgDialog->getContent();
        sgIndex = spgDialog->getSpaceGroup(spg);
        spgDialog->getCellParameters(cell);
        findspace = spgDialog->getFindSpace();
    }
}

void MainWindow::on_actionSpace_Group_Determination_triggered()
{    
    spg_determine();
}

void MainWindow::openSpaceGroupListDialog(spgext_info_type spgList[], int listSize, int &selectedSpace)
{
    saveEnabledActions();
    enableActions(MainWindow::DialogOpenAction);
    spaceListDialog->createModel(spgList,listSize,selectedSpace);
    spaceListDialog->show();
    spaceListDialog->waitSignalIsEmitted();
    if (spaceListDialog->result() == QDialog::Accepted)
        selectedSpace = spaceListDialog->selectedSpaceGroup();
    else
        selectedSpace = -1;
    restoreEnabledActions();
}


void MainWindow::on_actionSpace_Group_List_triggered()
{
    spg_list();
}

//
//  View Menu
//

void MainWindow::on_actionPlot_Style_triggered()
{
    plotStyleDialog->setOptions(obs, back, bpoints, calc, diff, cdiff, peaks, refl, plotSettings);
    plotStyleDialog->show();
}

// void MainWindow::on_actionJav_Molecular_Viewer_triggered()
// {
//     lanciajavs_();
// }

void MainWindow::on_actionReset_Zoom_triggered()
{
    // ui->plot->yAxis->setRange(yLowerRange,yUpperRange);
    // ui->plot->xAxis->rescale();
    // ui->plot->replot();
    ui->plot->xAxis->rescale(); //import if autoscale is on
    redrawPlot(true);
    //draw();
}

//
//  Solve
//

void MainWindow::openExploreTrialsDialog(int kproc, int saf, int nset, int selset, trialInfo tinfo[])
{
    exploreTrialsDialog->setTrials(kproc, saf, nset, selset, tinfo);
    exploreTrialsDialog->show();
}


void MainWindow::on_actionExplore_Trials_triggered()
{
    restart_dm();
}

void MainWindow::on_actionRAMM_Procedure_triggered()
{
    randomsolve();
}

void MainWindow::openGlobalOptDialog(bool ifButton)
{
    if (globOptDialog->setDialog(ifButton)) return;

    globOptDialog->show();
    globOptDialog->activateWindow();
    if (!ifButton) globOptDialog->waitSignalIsEmitted();
}

void MainWindow::on_actionSimulated_Annealing_triggered()
{
    openGlobalOptDialog(true);
}

void MainWindow::on_actionRecycle_in_Extra_triggered()
{
    recycle();
}

void MainWindow::on_actionDirect_Methods_triggered()
{
    dmDialog->setDialog();
    dmDialog->show();
}

//
//  Refine
//

void MainWindow::applyRefine(int action)
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    fsrbmw(action);
    QApplication::restoreOverrideCursor();
}

CrystalViewMenu *MainWindow::getCrystalViewMenu() const
{
    return crystalViewMenu;
}

void MainWindow::crystalViewMenuEnabled(bool enabled)
{
    enableMenu(crystalViewMenu->menu_Modify, enabled);
    enableMenu(crystalViewMenu->menu_Select, enabled);
    enableMenu(crystalViewMenu->menu_View, enabled);
    enableMenu(crystalViewMenu->menu_Tools, enabled);
    crystalViewMenu->mainToolBar->setEnabled(enabled);
    crystalViewMenu->crystalSelectionToolBar->setEnabled(enabled);
    crystalViewMenu->orientationToolbar->setEnabled(enabled);
    if (enabled)
        enableMenu(ui->menuInfo, enabled);
    else
        //disable only the part of menu specific for structure
        enableMenu(crystalViewMenu->menu_Info, enabled);
}

void MainWindow::on_actionRBM_Direct_Space_triggered()
{
    applyRefine(1);
}

void MainWindow::on_actionRBM_Direct_and_Reciprocal_Space_triggered()
{
    applyRefine(2);
}

void MainWindow::on_actionRBM_COVMAP_triggered()
{
    applyRefine(-10);
}


void MainWindow::on_actionFourier_Recycling_triggered()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    fslsq();
    QApplication::restoreOverrideCursor();
}

void MainWindow::on_actionFourier_triggered()
{
    fourDialog->setDialog();
    fourDialog->show();
}

void MainWindow::openFourierPeakDialog(FourierPeakInfo fPeaks[], int nPeaks, int nOldPeaks)
{
    fourPeakDialog->setDialog(fPeaks, nPeaks, nOldPeaks);
    fourPeakDialog->show();
    fourPeakDialog->waitSignalIsEmitted();
    fourPeakDialog->getSelectedPeaks(fPeaks);
}

void MainWindow::on_actionRietveld_triggered()
{    
    rietveld();
}

void MainWindow::openRietveldDialog(const refine_condition_type &rcond)
{
    rietDialog->setDialog(rcond);
    rietDialog->show();
    rietDialog->waitSignalIsEmitted();
}

//
//  Info Menu
//

void MainWindow::openHelp(const QString &page)
{
    QString link = "https://www.ba.ic.cnr.it/softwareic/expo/" + page;
    QDesktopServices::openUrl(QUrl(link));
}

void MainWindow::setCrystalDockTitle(const QString &title)
{
    ui->crystalViewDockWidget->setWindowTitle(title);
}

void MainWindow::showCrystalDock()
{
    ui->crystalViewDockWidget->show();
}

void MainWindow::openHklListDialog(c_hkl_info hklInfo[], int nHkl, int vis[])
{
    hklDialog->setDialog(hklInfo, nHkl, vis);
}

void MainWindow::on_actionHKL_List_triggered()
{
    open_hkl_list();
    hklDialog->show();
}

void MainWindow::on_actionSummary_triggered()
{
    QString fileName = QDir::tempPath()+QDir::separator()+"tmp.tmp";
    open_summary(fileName.toStdString().c_str(), fileName.length());
    textViewer->LoadFile(fileName);
    textViewer->show();
}

void MainWindow::on_actionView_Output_File_triggered()
{
    //FIX outputFileName for import structure, import diffraction and nowin
    qInfo() << "OUTFile: " << QFileInfo(outputFileName).absoluteFilePath();
    if (outputFileName.isEmpty()) {
        QMessageBox::warning(this, "Warning", "Cannot find the output file!");
    } else {
        textViewer->LoadFile(outputFileName);
        textViewer->show();
    }
}

void MainWindow::actionProfileTriggered(bool checked)
{    
    set_profile_enabled(checked);
}

//
//  Window Menu
//

void MainWindow::on_actionDefault_Layout_triggered()
{
    restoreState(defaultState);
    resizeDocks({ui->dockWidget,ui->dockWidget_2},{50,50},Qt::Horizontal);
}

//
//  Help
//

void MainWindow::actionDocumentationHtmlTriggered()
{
    this->openHelp();
}

void MainWindow::actionDocumentationPdfTriggered()
{
    this->openHelp("wp-content/uploads/sites/2/kalins-pdf/Expo2014.pdf");
}

void MainWindow::actionCheckForUpdatesTriggered()
{    
#if defined(Q_OS_WIN)
    auto m_updater = new MaintenanceTool(this);
    connect(m_updater, &MaintenanceTool::stateChanged, [=](MaintenanceTool::ProcessState state){
        if (state == MaintenanceTool::NotRunning && m_updater->hasUpdate()) {
            m_updater->startMaintenanceTool(MaintenanceTool::Updater);
        }
    });

    m_updater->checkUpdate();

#else

    auto m_updater = new Updater(this);
    QString url = "https://cloud.ba.cnr.it/index.php/s/Q478xC43pcagZAR/"
                  "download/updates.json";
    m_updater->setUrl(url);
    m_updater->setNotifyOnFinish(true);
    m_updater->checkForUpdates();

#endif
}

void MainWindow::actionAboutTriggered()
{
    aboutDialog->show();
}

//
//  Run Toolbar
//

void MainWindow::on_actionStop_triggered()
{
    send_event(STOP_EVENT);
    emit ui->actionNext->triggered(true);
    emit actionStopTriggered();
}

void MainWindow::on_actionSkip_triggered()
{
    send_event(SKIP_EVENT);
}

void MainWindow::actionCycleTriggered()
{
    send_event(CYCLE_EVENT);
    emit ui->actionNext->triggered(true);
    emit actionStopTriggered();
}

//
//  Crystal View
//
void MainWindow::onActionLabelsOnOfftriggered()
{
    ui->crystalWidget->onActionLabelsOnOff(crystalViewMenu->actionLabels_On_Off->isChecked(), crystalViewMenu->actionAll_Atoms);
}

