MODULE messagemod

use iso_c_binding

implicit none

integer, parameter, private  :: NMAXMESS = 12
integer, private             :: prog_current_value, prog_final_value
integer, private             :: lastpos = 1

interface

    subroutine WriteMesg(mesg, row, kprocess) bind(C,name="WriteMesg")
    use iso_c_binding
    character(c_char), dimension(*) :: mesg
    integer(c_int), value           :: row, kprocess
    end subroutine WriteMesg

    subroutine setRowCountMesgC(nrow, kprocess) bind(C,name="setRowCountMesgC")
    use iso_c_binding
    integer(c_int), value :: nrow, kprocess
    end subroutine setRowCountMesgC

    function getRowCountMesg() bind(C,name="getRowCountMesg")
    use iso_c_binding
    integer(c_int) :: getRowCountMesg
    end function getRowCountMesg

    function rowFindMesg(mesg) bind(C,name="rowFindMesg")
    use iso_c_binding
    character(c_char), dimension(*) :: mesg
    integer(c_int)                  :: rowFindMesg
    end function rowFindMesg

    subroutine ClearMesg(kprocess) bind(C,name="ClearMesg")
    use iso_c_binding
    integer(c_int), value :: kprocess
    end subroutine ClearMesg

    subroutine ClearRow(row, kprocess) bind(C,name="ClearRow")
    use iso_c_binding
    integer(c_int), intent(in), value :: row, kprocess
    end subroutine ClearRow

    subroutine progressbar_set_visible(visible, kprocess) bind(C,name="progressbar_set_visible")
    use iso_c_binding
    integer(c_int), value :: visible, kprocess
    end subroutine progressbar_set_visible

    subroutine progressbar_set_value(val, maxv, kprocess) bind(C,name="progressbar_set_value")
    use iso_c_binding
    integer(c_int), intent(in), value :: val, maxv
    integer(c_int), intent(in), value :: kprocess
    end subroutine progressbar_set_value

    subroutine progressbar_set_message(mesg, kprocess) bind(C,name="progressbar_set_message")
    use iso_c_binding
    character(c_char), dimension(*)   :: mesg
    integer(c_int), intent(in), value :: kprocess
    end subroutine progressbar_set_message

end interface

CONTAINS

    subroutine init_messages(nrow, kproc)
!
!   Inizializzi messaggi
!
    USE molcom
    integer, intent(in), optional :: nrow, kproc
    integer :: i, kprocess
    if (jscreen > 0) then
        if (present(nrow)) then
            if (present(kproc)) then
                kprocess = kproc
            else
                kprocess = 1
            endif
            call setRowCountMesg(nrow,kprocess)
        endif
        lastpos = 1
    endif
    end subroutine init_messages

!--------------------------------------------------------------

    subroutine clear_messages(pos,ipos,fpos,kproc)
    USE molcom, only:jscreen
    USE strutil
    integer, intent(in), optional :: pos
    integer, intent(in), optional :: ipos,fpos
    integer, intent(in), optional :: kproc
    character(len=1)              :: str = ' '
    integer                       :: i,kprocess
!   
    if (jscreen > 0) then
        if (present(kproc)) then
            kprocess = kproc
        else
            kprocess = 1
        endif
        if (present(pos)) then
            call ClearRow(pos,kprocess)
        elseif (present(ipos)) then
            if (present(fpos)) then
                do i=fpos,ipos,-1
                   call ClearRow(i,kprocess)
!!!FIX-clear                   call WriteMesg(toCString(str), i)
                enddo
                lastpos = ipos - 1
            else
                do i=NMAXMESS,ipos,-1
                   call ClearRow(i,kprocess)
!!!FIX-clear                   call WriteMesg(toCString(str), i)
                enddo
            endif
        else
            call ClearMesg(kprocess)  ! clear di tutti i messaggi
            lastpos = 0
        endif
    endif
!
    end subroutine clear_messages

!--------------------------------------------------------------

    subroutine write_message(string,inum,rnum,pos,dig,kproc)
    USE strutil
    USE molcom, only:jscreen
    character(len=*), intent(in)  :: string
    integer, intent(in), optional :: inum
    real, intent(in), optional    :: rnum
    integer, intent(in)           :: pos
    integer, intent(in), optional :: dig
    integer, intent(in), optional :: kproc
    logical                       :: is_num,is_int,is_real
    integer                       :: posm, digr, kprocess
!
    if (jscreen > 0 .and. pos >= 0) then
!
!       Se pos = 0 scrivi nella riga successiva all'ultima
        if (pos  == 0) then
            posm = lastpos + 1
        else
            posm = pos
        endif
!
        is_int = present(inum)
        is_real = present(rnum)
        is_num = is_int .or. is_real
        if (present(kproc)) then       
            kprocess = kproc
        else
            kprocess = 1
        endif
        if (is_num) then
            if (is_int) then
                call WriteMesg(toCString(string//trim(i_to_s(inum))), posm, kprocess)
            else
                if (present(dig)) then
                    digr = dig
                else
                    digr = 3
                endif
                call WriteMesg(toCString(string//trim(r_to_s(rnum,digr))), posm, kprocess)
            endif
        else
            call WriteMesg(toCString(string), posm, kprocess)
        endif
        lastpos = posm
    endif
!
    end subroutine write_message

!--------------------------------------------------------------

    subroutine progress_bar_init(string,fval,kproc)
    USE molcom, only:jscreen
    character(len=*), intent(in), optional :: string  ! stringa da scrivere
    integer, intent(in), optional          :: fval    ! valore finale della progress bar
    integer, intent(in), optional          :: kproc
    integer :: kprocess
!
    if (jscreen == 0) return
    if (present(kproc)) then
        kprocess = kproc
    else
        kprocess = 1
    endif
    call progressbar_set_visible(1,kprocess)
    if (present(string)) then
        call progressbar_set_message(trim(string)//c_null_char,kprocess)
    endif
!
!   inizializza il valore corrente
    prog_current_value = 0       
!
!   inizializza il valore finale
    if (present(fval)) then
        prog_final_value = fval
    else
        prog_final_value = 100
    endif
!
    end subroutine progress_bar_init

!--------------------------------------------------------------

    subroutine progress_bar_update(string,cval,fval,kproc)
    USE molcom, only:jscreen
    character(len=*), intent(in), optional :: string  ! stringa da scrivere
    integer, intent(in), optional          :: cval    ! valore corrente
    integer, intent(in), optional          :: fval    ! valore finale
    integer, intent(in), optional          :: kproc
    integer :: kprocess
!
    if (jscreen == 0) return
!
    if (present(cval)) then
        prog_current_value = cval
    else
        prog_current_value = prog_current_value + 1
    endif
!
    if (present(fval)) then
        prog_final_value = fval
    endif
!
    if (present(kproc)) then
        kprocess = kproc
    else
        kprocess = 1
    endif
!
    call progressbar_set_value(prog_current_value,prog_final_value,kprocess)
    if (present(string)) then
        call progressbar_set_message(trim(string)//c_null_char,kprocess)
    endif
!
    end subroutine progress_bar_update

!--------------------------------------------------------------

    subroutine progress_bar_increase_limit(fval)
    integer, intent(in) :: fval
    prog_final_value = prog_final_value + fval
    end subroutine progress_bar_increase_limit

!--------------------------------------------------------------

    subroutine progress_bar_close(kprocess)
    USE molcom, only:jscreen
    integer, intent(in) :: kprocess
!
    if (jscreen == 0) return
    call progressbar_set_visible(0,kprocess)
    prog_final_value = 0
    prog_current_value = 0
!
    end subroutine progress_bar_close 

!--------------------------------------------------------------
  
    integer function position_last_message()
    position_last_message = lastpos
    end function position_last_message

!--------------------------------------------------------------

    subroutine setRowCountMesg(nrow,kproc)
    USE molcom, only:jscreen
    integer, intent(in) :: nrow,kproc
!
    if (jscreen == 0) return
    call setRowCountMesgC(nrow,kproc)
!    
    end subroutine setRowCountMesg

END MODULE messagemod
