#include "missinginfodialog.h"
#include "ui_missinginfodialog.h"

#include <QPushButton>

extern "C" void get_symmetry_info(int phaseid, int *spgIndex, char *spaceg, double cell[6]);
extern "C" void put_symmetry_info(const char *space, int len_space, double cell[], int *ier);
extern "C" void importa_file_from_string(const char *cstring, int length, int *ier);
extern "C" int crystal_atom_number(int id);

MissingInfoDialog::MissingInfoDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::MissingInfoDialog),
    hasModel(false)
{
    ui->setupUi(this);

    connect(ui->symmetryWidget, &SymmetryWidget::spaceWidgetChanged, this, &MissingInfoDialog::enableOkButton);
    connect(ui->symmetryWidget, &SymmetryWidget::cellWidgetChanged, this, &MissingInfoDialog::enableOkButton);
    connect(ui->modelWidget, &ModelLoader::modelChanged, this, &MissingInfoDialog::enableOkButton);
}

void MissingInfoDialog::setDialog()
{
    int spgIndex;
    char *spaceg = new char[40];
    double cell[6];
    get_symmetry_info(1, &spgIndex, spaceg, cell);

    ui->symmetryWidget->setData(cell, spgIndex, QString::fromUtf8(spaceg));

    int phaseId = 1;
    hasModel = crystal_atom_number(phaseId);

    enableOkButton();
}

bool MissingInfoDialog::sendMissingInfo()
{
    QString spg;
    (void) ui->symmetryWidget->getSpaceGroup(spg);
    double cell[6];
    ui->symmetryWidget->getCellParameters(cell);
    int ier;
    put_symmetry_info(spg.toStdString().c_str(), spg.length(), cell, &ier);
    if (ier) return true;

    auto models = ui->modelWidget->modelList();
    if (models.isEmpty()) return !hasModel;

    int nerr = 0;
    foreach(auto model, models) {
        QString str = model.trimmed();
        importa_file_from_string(str.toStdString().c_str(), str.length(), &ier);
        if (ier) nerr++;
    }
    if (nerr == models.size()) return true; // all models are in error

    return false;
}

MissingInfoDialog::~MissingInfoDialog()
{
    delete ui;
}

void MissingInfoDialog::enableOkButton()
{
    bool enable = ui->symmetryWidget->isValidSpaceGroup() &&
            ui->symmetryWidget->isValidCell();

    if (!hasModel) {           
        enable = enable && (!ui->modelWidget->isEmpty());
    }

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(enable);
}
