#include <myheader.h>

MyHeader::MyHeader(Qt::Orientation orientation, QWidget *parent) : QHeaderView(orientation, parent)
{
  standard_model = 0;
}

void MyHeader::paintSection(QPainter *painter, const QRect &rect, int logical_index) const {
  painter->save();
  QHeaderView::paintSection(painter, rect, logical_index);
  painter->restore();
  if (standard_model && orientation() == Qt::Horizontal) {
    QStandardItem* item = standard_model->horizontalHeaderItem(logical_index);
    if (item && item->isCheckable()) {
      int offset = (height() - style()->pixelMetric(QStyle::PM_IndicatorHeight))/2;
      int pos = sectionViewportPosition(logical_index);
      QStyleOptionButton option;
      option.rect = QRect(offset + pos, offset,
                          style()->pixelMetric(QStyle::PM_IndicatorWidth),
                          style()->pixelMetric(QStyle::PM_IndicatorHeight));
      if (item->checkState() == Qt::Checked) {
        option.state = QStyle::State_On;
      } else {
        option.state = QStyle::State_Off;
      }
      option.state |= QStyle::State_Enabled;
      style()->drawPrimitive(QStyle::PE_IndicatorCheckBox, &option, painter);
    }
  }
}

void MyHeader::mousePressEvent(QMouseEvent *event) {
  int offset = (height() - style()->pixelMetric(QStyle::PM_IndicatorHeight))/2;
  if (standard_model && orientation() == Qt::Horizontal) {
    for(int logical_index = 0; logical_index < count(); logical_index++) {
      int pos = sectionViewportPosition(logical_index);      
      QRect rect(offset + pos, offset,
                 style()->pixelMetric(QStyle::PM_IndicatorWidth),
                 style()->pixelMetric(QStyle::PM_IndicatorHeight));
      if (rect.contains(event->pos())) {
        QStandardItem* item = standard_model->horizontalHeaderItem(logical_index);
        if (item && item->isCheckable()) {
          item->setCheckState(item->checkState() == Qt::Checked ? Qt::Unchecked : Qt::Checked);
          return;
        }
      }
    }
  }
  QHeaderView::mousePressEvent(event);
}

void MyHeader::setModel(QAbstractItemModel *model) {
  QHeaderView::setModel(model);
  standard_model = qobject_cast<QStandardItemModel*>(model);
}
