#include "myslider.h"

bool MySlider::do_emit = true;

MySlider::MySlider(QWidget *parent, int z, double min_val, double max_val, double val, double step, int ndec)
    : QWidget(parent)
{
    m_layout = new QHBoxLayout();
    setLayout(m_layout);
    m_slider = new QSlider(Qt::Horizontal);
    m_slider->setRange(1, 1000);
    m_layout->addWidget(m_slider, 1, Qt::AlignLeft);
    m_spin = new QDoubleSpinBox();
    m_spin->setRange(min_val, max_val);
    m_spin->setSingleStep(step);
    m_spin->setDecimals(ndec);
    m_spin->setValue(val);
    m_layout->addWidget(m_spin, 0, Qt::AlignLeft);
    m_value = val;
    m_z = z;
    DoubleSpinToSlider(m_value, m_spin, m_slider);
    connect(m_slider, &QSlider::valueChanged, this, &MySlider::on_Slider_valueChanged);
#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0))
    connect(m_spin, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &MySlider::on_SpinBox_valueChanged);
#else
    connect(m_spin, static_cast<void(QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged), this, &MySlider::on_SpinBox_valueChanged);
#endif
}

MySlider::MySlider(QWidget *parent, int z, int min_val, int max_val, int val, int step)
    : QWidget(parent)
{
    m_layout = new QHBoxLayout();
    setLayout(m_layout);
    m_slider = new QSlider(Qt::Horizontal);
    m_slider->setRange(1, 100);
    m_layout->addWidget(m_slider, 1, Qt::AlignLeft);
    m_ispin = new QSpinBox();
    m_ispin->setRange(min_val, max_val);
    m_ispin->setSingleStep(step);
    m_ispin->setValue(val);
    m_layout->addWidget(m_ispin, 0, Qt::AlignLeft);
    m_value = val;
    m_z = z;
    connect(m_slider, &QSlider::valueChanged, this, &MySlider::on_IntSlider_valueChanged);
#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0))
    connect(m_ispin, QOverload<int>::of(&QSpinBox::valueChanged), this, &MySlider::on_IntSpinBox_valueChanged);
#else
    connect(m_ispin, static_cast<void(QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &MySlider::on_IntSpinBox_valueChanged);
#endif

}

void MySlider::SliderToDoubleSpin(int value, const QSlider* slider, QDoubleSpinBox* doubleSpin)
{
    //Convert value to double for spinbox
    double coeff = (doubleSpin->maximum() - doubleSpin->minimum())/
            (slider->maximum() - slider->minimum());
    m_value =-coeff*slider->minimum()+doubleSpin->minimum()+coeff*value;

    //blockSignals is used to avoid signal valueChanged from spin
    doubleSpin->blockSignals(true);
    doubleSpin->setValue(m_value);
    doubleSpin->blockSignals(false);
}

void MySlider::DoubleSpinToSlider(double value, const QDoubleSpinBox* doubleSpin, QSlider* slider)
{
    // Convert arg1 to int for slider
    double coeff = (slider->maximum() - slider->minimum())/
            (doubleSpin->maximum() - doubleSpin->minimum());
    int sliderValue = static_cast<int>(-coeff*doubleSpin->minimum()+slider->minimum()+coeff*value);

    //blockSignals is used to avoid signal valueChanged from slider
    slider->blockSignals(true);
    slider->setValue(sliderValue);
    slider->blockSignals(false);
}

void MySlider::on_IntSlider_valueChanged(int value)
{
    m_ivalue = value;
    m_ispin->blockSignals(true);
    m_ispin->setValue(m_ivalue);
    m_ispin->blockSignals(false);
    if(m_z >= 0)
       emit(intSliderChanged(m_ivalue,m_z));
    else
       emit(intSliderChanged(m_ivalue));
}

void MySlider::on_IntSpinBox_valueChanged(int arg1)
{
    m_ivalue = arg1;
    m_slider->setValue(m_ivalue);
}

void MySlider::on_Slider_valueChanged(int value)
{
    SliderToDoubleSpin(value, m_slider, m_spin);
    if(!do_emit)
        return;
    if(m_z >= 0)
       emit(sliderChanged(m_value,m_z));
    else
       emit(sliderChanged(m_value));
}

void MySlider::on_SpinBox_valueChanged(double arg1)
{
    m_value = arg1;
    DoubleSpinToSlider(arg1, m_spin, m_slider);
    if(!do_emit)
        return;
    if(m_z >= 0)
       emit(sliderChanged(m_value,m_z));
    else
       emit(sliderChanged(m_value));
}

void MySlider::syncAll(QWidget *parentW, double val, int z)
{
    QList<MySlider *> allSliders = parentW->findChildren<MySlider *>();
    do_emit = false;
    for(int i=0; i < allSliders.size(); i++)
    {
	if(i == z)
	   continue;
        allSliders.at(i)->m_spin->setValue(val);
    }
    do_emit = true;
}


