#ifndef OPTIONSDIALOG_H
#define OPTIONSDIALOG_H

#include <QDialog>
#include <QAbstractButton>
#include <QSlider>
#include <QDoubleSpinBox>
#include <QCheckBox>
#include "libcomune.h"
#include "models.h"
#include "properties.h"
#include "glutils.h"
#include "chainbutton.h"

#if defined(JAV_WIDGET)
class CrystalViewWidget;
#else
class JavWindow;
#endif

enum pages {PAGE_MODELS, PAGE_ATOMS, PAGE_BONDS, PAGE_MAPS, PAGE_POLY,
        PAGE_DRAW, PAGE_DISP, PAGE_LIGHT};

typedef struct {
      bool ifVis;
      bool ifLabels;
      int  plotStyle;
      int  drawLigand;
      bool ifSymmetry;
      float klevel[3];
      int  FitOpt;
      bool ifDrawH;
      bool ifDrawW;
      int  colorMode;
      QColor labelcolor;
      int   nPolyStyles;
      bool  ifSurface;
      int   wireframe;
      QColor color_mappa;
      float radius;
      float step;
      polyhedron_style *PolyStyle;
      } _model_info;

typedef struct {
      double sc_atoms;
      double scale_atoms[N_ELEMENTS];
      QColor Colori[N_ELEMENTS];
      bool   RadType;
      int    ell_tr;
      int    ell_prob;
      bool   ell_princ;
      QColor ell_color;
      bool   ell_hydro;
      } _atoms_info;

typedef struct {
      double sc_bonds;
      QColor ColorHB;
      QColor ColorSC;
      bool   ShowMultBonds;
      } _bonds_info;

typedef struct {
      ColorWidget::ColorType BgType;
      QColor BgColor;
      QColor BgColor1;
      QColor BgColor2;
      QColor DistColor;
      QColor CellColor;
      bool   ifCell;
      bool   ifAxes;
      bool   ifDrawAxes;
      bool   Clip;
      bool   Fog;
      QString fontname;
      double ClipFactor;
      double FogDensity;
      double xoffset;
      double yoffset;
      } _draw_info;

typedef struct {
      bool MSEnabled;
      int  MSBuffers;
      bool ifGrid;
      int  nDivisions;
      int  GridPattern;
      bool Stereo;
      int  glasses_type;
      bool ifOrtho;
      } _disp_info;

typedef struct {
      float centro[3];
      float raggio;
      } _map_info;


namespace Ui {
class OptionsDialog;
}

class OptionsDialog : public QDialog
{
    Q_OBJECT

public:
    explicit OptionsDialog(QWidget *parent = 0);
    ~OptionsDialog();
    void setOptions();
    void fillOptionsModel(int kmodel, bool ifSave=false);    
    void fillPageAtoms();
    void fillPageBonds();
    void fillPageMaps();
    void fillPageDraw();
    void fillPagePoly();
    void fillPageDisp();
    void fillPageLight();
    void SetPage(int n);

private slots:

    //void on_listWidget_clicked(const QModelIndex &index);
    void on_listWidget_currentRowChanged(int currentRow);
    void on_buttonBox_clicked(QAbstractButton *button);
    void on_labelsColorButton_colorChanged(QColor newcolor);
    void on_coloursSettingButton_colorChanged(QColor newcolor);
    void on_coloursSettingComboBox_activated(int index);
    void on_spinA_valueChanged(double arg1);
    void on_spinB_valueChanged(double arg1);
    void on_spinC_valueChanged(double arg1);
    void on_styleVdWColorButton_colorChanged(QColor newcolor);
    void on_comboBoxModels_currentIndexChanged(int index);
    void on_ligandsComboBox_currentIndexChanged(int index);
    void on_checkBoxLigands_stateChanged(int arg1);
    void on_checkBoxLabel_clicked();
    void on_groupBoxPacking_clicked();
    void reject() override;

    void on_plotStyleComboBox_currentIndexChanged(int index);
    void on_checkBoxHydrogens_stateChanged(int arg1);
    void on_comboBoxPacking_activated(int index);
    void on_groupBoxVdW_clicked(bool checked);
    void on_comboBoxVdW_activated(int index);
    void on_radiusSpinBox_valueChanged(double arg1);
    void on_gridSpinBox_valueChanged(double arg1);
    void on_radioCovalent_toggled(bool checked);
    void on_radioThermal_toggled(bool checked);
    void on_spinEllTransparency_valueChanged(int arg1);
    void on_spinProb_valueChanged(int value);
    void on_checkEllPrinc_stateChanged(int arg1);
    void on_checkEllHydro_stateChanged(int arg1);
    void on_ellPrincColor_colorChanged(QColor newcolor);
    void on_bondRadiusSlider_valueChanged(int value);
    void on_bondRadiusSpinBox_valueChanged(double arg1);
    void on_HBondsColorButton_colorChanged(QColor newcolor);
    void on_SBondsColorButton_colorChanged(QColor newcolor);
    void on_cellEdgesCheck_clicked();
    void on_cellEdgesColorButton_colorChanged(QColor newcolor);
    void on_cAxesVisibleCheck_stateChanged(int arg1);
    void on_displayInCellCheck_stateChanged(int arg1);
    void on_clipEnableCheck_stateChanged(int arg1);
    void on_clipSpinBox_valueChanged(double arg1);
    void on_fogEnableCheck_stateChanged(int arg1);
    void on_fogSpinBox_valueChanged(double arg1);
    void on_distancesColorButton_colorChanged(QColor newcolor);
    void on_labelFontButton_clicked();
    void on_xOffsetSpin_valueChanged(double arg1);
    void on_yOffsetSpin_valueChanged(double arg1);
    void on_MultipleBondscheck_stateChanged(int arg1);
    void colorspecies_changed(QColor newcolor, int k);
    void radiusspecies_changed(double val, int z);
    void polyVis_changed(QCheckBox *check, int i);
    void colorpoly_changed(QColor newColor, int k);
    void polyTransp_changed(int val, int i);
    void polyLinewidth_changed(QDoubleSpinBox *spin, int i);    
    void on_checkBoxFillUC_clicked(bool checked);
    void on_comboBoxGridStyle_currentIndexChanged(int index);
    void on_boxViewGrid_clicked();
    void on_spinBoxGridDiv_valueChanged(int arg1);
    void on_radioButtonRB_toggled(bool checked);
    void on_radioButtonRG_toggled(bool checked);
    void on_radioButtonRC_toggled(bool checked);
    void on_radioButtonBR_toggled(bool checked);
    void on_radioButtonGR_toggled(bool checked);
    void on_radioButtonCR_toggled(bool checked);
    void on_groupBox3D_clicked();
    void on_radioOrtho_toggled(bool checked);
    void on_checkBoxMultiS_stateChanged(int arg1);
    void on_comboBoxFilter_currentIndexChanged(int index);
    void on_boxLight1_toggled(bool checked);
    void on_boxLight2_toggled(bool checked);
    void on_boxLight3_toggled(bool checked);
    void on_spinBoxL1x_valueChanged(double arg1);
    void on_spinBoxL1y_valueChanged(double arg1);
    void on_spinBoxL1z_valueChanged(double arg1);
    void on_spinBoxL2x_valueChanged(double arg1);
    void on_spinBoxL2y_valueChanged(double arg1);
    void on_spinBoxL2z_valueChanged(double arg1);    
    void on_pushButtonL1D_colorChanged(QColor newcolor); 
    void on_pushButtonL1S_colorChanged(QColor newcolor); 
    void on_pushButtonL2D_colorChanged(QColor newcolor); 
    void on_pushButtonL2S_colorChanged(QColor newcolor); 
    void on_pushButtonL3D_colorChanged(QColor newcolor);     
    void on_spinBoxMaps_valueChanged(double arg1);
    void on_spinBoxX_valueChanged(double arg1);
    void on_spinBoxY_valueChanged(double arg1);
    void on_spinBoxZ_valueChanged(double arg1);
    void on_pushButtonCentre_clicked();

private:
    Ui::OptionsDialog *ui;
    Model *md;
    ChainButton *m_chainbutton;
#if defined(JAV_WIDGET)
    CrystalViewWidget *jav;
#else
    JavWindow *jav;
#endif
    _model_info *ModelSav = nullptr;
    _map_info MapSav;
    _map_info MapDefault = {{0, 0, 0}, 10.f};
    _model_info ModelDefault = {
	    true, false, STICK_AND_BALLS, 0, false, {0.1f, 0.1f, 0.1f},
	    1, true, false, BY_SPECIES, "#ffffff", 0, false, 0,
            Properties::MapColors[0], 1.2f, 0.3f,
            nullptr};
    _atoms_info AtomsSav;
    _atoms_info AtomsDefault;
    _bonds_info BondsSav;
    _bonds_info BondsDefault = { 0.65, ToQColor(0.00f, 1.00f,1.00f, 1.0f),
	                         ToQColor(1.00f, 1.00f,0.00f, 1.0f), false} ;

    _draw_info DrawSav;
    _draw_info DrawDefault = {ColorWidget::TYPE_DGRAD, Properties::defaultBgColor,
	    Properties::defaultBgColor1,
	    Properties::defaultBgColor2, ToQColor(0.83f, 0.83f,0.83f, 1.0f),
	    ToQColor(0.83f, 0.83f,0.83f), false, true, false, false, false,
	    QFont().defaultFamily()+", 10", 1.0, 0.3, 0, 0};

    _disp_info DispSav;
    _disp_info DispDefault = {false, 0, false, 0, 0, false, 0, true};
    _LightInfo LightSav[3];

    int m_kmodel;
    void reDraw();
    void set_color_pushbutton(QPushButton *button, const QColor &color);
    void setCellRange(double arg1, int spinNum);
    void setOptionsModel(int kmodel);
    void writeSettings();
    void saveInfo(int k=-1);
    void fillAtomsDefault();
    void restoreInfo(bool ifDefault=false);
    void restoreModelInfo(bool ifDefault=false);
    void restoreModelInfo(_model_info md, int k, bool ifPoly=true);
    void restoreAtomInfo(bool ifDefault=false);
    void restoreBondInfo(bool ifDefault=false);
    void restoreMapInfo(bool ifDefault=false);
    void restorePolyInfo();
    void restoreDrawInfo(bool ifDefault=false);
    void restoreDispInfo(bool ifDefault=false);
    void restoreLightInfo(bool ifDefault=false);
    void backgroundChanged();

    Model *GetModel(int k);
    void SliderToDoubleSpin(int value, const QSlider *slider, QDoubleSpinBox *doubleSpin);
    void DoubleSpinToSlider(double value, const QDoubleSpinBox *doubleSpin, QSlider *slider);
};

#endif // OPTIONSDIALOG_H
