#ifndef PROPERTIESDIALOG_H
#define PROPERTIESDIALOG_H

#include <QDialog>
#include <QStandardItem>
#include <QStandardItemModel>
#include <QItemSelection>
#include "models.h"
#include "myproxymodel.h"

extern "C" void jav_edit_structure_(int *code, atom_type *atml, int *natoms, int *wm);
extern "C" void jav_edit_bond_properties_(int *code, f_bond_type *legs, int *wm,                                          int *legame);
extern "C" void jav_edit_angle_properties_(int *code, jav_anglebond_type *angs, int *wm, int *angolo);
extern "C" void jav_edit_torsion_properties_(int *code, jav_torsion_type *tors,
                   float *tval, int *whichModel);
extern "C" void getangle_(int *whichModel);
extern "C" void gettorsion_(int *whichModel);

namespace Ui {
class PropertiesDialog;
}

class PropertiesDialog : public QDialog
{
    Q_OBJECT

public:
    explicit PropertiesDialog(QWidget *parent = 0);
    ~PropertiesDialog();
    void setTipo(int tipo) {mTipo = tipo;}
    void setModel(int km) {kmodel = km;}
    Model *ModelEdit;

private:
    Ui::PropertiesDialog *ui;
    QStandardItemModel *mModel;
    MyProxyModel *proxyModel;
    int mTipo;
    int kmodel;
    int mAtom;
    int mBond;
    int mAngle;
    int mTors;
    bool mAddDelete;
    QStandardItemModel *createAndPopulateModel();
    void reorderItems(int rowDel);
    void applyEdit(int *kat=nullptr);
    jav_anglebond_type *FromListToAngs();
    f_bond_type *FromListToLegm();
    atom_type *FromListToAtom(int *natoms);
    void addRow();

private slots:
    void on_addButton_clicked();
    void on_deleteButton_clicked();
    void on_okButton_clicked();
    void on_cancelButton_clicked();
    void rowSelectionChanged(const QModelIndex &current, const QModelIndex &);
    void tableView_dataChanged(const QModelIndex &, const QModelIndex &);

protected:
    virtual void showEvent(QShowEvent *event);

signals:
    void selectedAtom(int which);
    void selectedBond(int which);
    void selectedAngle(int which);
    void selectedTorsion(int which);
    void propertiesDialogClosed();
};

#endif // PROPERTIESDIALOG_H
