#ifndef RESTRAINTSWIDGET_H
#define RESTRAINTSWIDGET_H

#include "crystaldata.h"
#include "checkboxheader.h"

#include <QWidget>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

namespace Ui {
class RestraintsWidget;
}

class RestraintsWidget : public QWidget
{
    Q_OBJECT

public:
    explicit RestraintsWidget(bool esdCol, QWidget *parent = nullptr);
    ~RestraintsWidget();
    void setRestraints(const QVector<crydata::RestraintProperties> &restr, const QStringList &labelList, double wei, const QString &restraintString);
    QVector<crydata::RestraintProperties> getRestraints();
    void getSettings(double &wei) const;
    void printRestraints(QTextStream &stream, const QString &weiString);

signals:
    void restraintsNumberChanged(int num);

private slots:
    void headerCheckBoxClicked(bool state, int column);
    void checkBoxClicked(const QModelIndex &index, bool state);
    void markSelectedRestraints();

private:
    Ui::RestraintsWidget *ui;
    QVector<crydata::RestraintProperties> restrList;
    QStringList labels;
    bool isEsd;
    QStandardItemModel *resModel;
    QSortFilterProxyModel *resProxyModel;
    CheckBoxHeader *cbHeader;
    int activeCol;
    enum {ID_COL, TYPE_COL, ATOMS_COL, CURRENT_COL, TARGET_COL, ESD_COL, ACTIVATE_COL};

    void createRestraintModel();
    void setRestraintTableView();
    void updateRestraintModel(const QVector<crydata::RestraintProperties> &restr);
    QString restraintString(const crydata::RestraintProperties &restr, const QStringList &labels, const QString &sep = "-");
    int rowCheckBoxClickedCount(int col) const;
};

#endif // RESTRAINTSWIDGET_H
