#include "sequencedialog.h"
#include "qt_utils.h"
#include "ui_sequencedialog.h"
#include <QString>
#include <QSettings>
#include <QMessageBox>
#include "savedialog.h"
#include "javkeysettings.h"
#include "libcomune.h"

const char *Translate(char Code);
int TypeS(char Code);

SequenceDialog::SequenceDialog(QWidget *parent, Model *md1) :
    QDialog(parent),
    ui(new Ui::SequenceDialog)
{
    ui->setupUi(this);
    md = md1;
    QVBoxLayout * layout = new QVBoxLayout(ui->scroll);
    m_layout = layout;
    ui->checkBoxNotAmino->setChecked(false);
}

SequenceDialog::~SequenceDialog()
{
    delete ui;
}

void SequenceDialog::FillSequence(bool ifOther)
{
    if(md == nullptr)
        return;
    m_Sequence.reset();
    removeChildren(m_layout);
    QStringList bgColors = {"#00FF00", "#FF0000", "#FFFF00", "#0000FF","#FF00FF"};
    QStringList fgColors = {"#000000", "#FFFFFF", "#000000", "#FFFFFF","#FFFFFF"};
    for(int kk=0; kk<md->nChains; kk++)
    {
        CHAIN ch = md->Chains[kk];
        if( !ifOther && (ch.nAmino == 0) )
            continue;
        QString lab_text = "Chain: " + QString(ch.id.c_str());
        QLabel *label = new QLabel(lab_text, this);
        label->setStyleSheet("font-weight: bold;");
        m_layout->addWidget(label);
        m_layout->setAlignment(label, Qt::AlignLeft|Qt::AlignTop);
        QLayout *hbox = new QHBoxLayout();
        m_layout->addLayout(hbox);
        m_layout->setAlignment(hbox, Qt::AlignLeft|Qt::AlignTop);
        hbox->setContentsMargins(0, 0, 0, 0);
        for(int rr=0; rr<ch.nResidues; rr++)
        {
            std::string tname(ch.Residues[rr].name);
            int code = ch.Residues[rr].code;
            int p = (int) tname.find_last_of( " " );
            std::string rname;
            if (p >= 0)
                rname = tname.substr(p+1);
            else
                rname = tname;

            if (rname.at(0) == ' ')
                rname.erase(rname.begin());
            int which = IsAminoNucle(rname);
            std::string t;
            QString color(""), color2("");
            if(NANonStd(code))
               t = "N";
            else if(which)
               t = get_AminoCode(which);
            else
               t = "X";
            RESIDUE *r1 = nullptr;
           if(t.compare("X"))
           {
              r1 = md->GetResiduo(kk, rr);
              tname.append("\n");
              tname.append(Translate(r1->secondary));
              color  = bgColors[TypeS(r1->secondary)];
              color2 = fgColors[TypeS(r1->secondary)];
           }
           else if(!ifOther)
              continue;
           else
           {
              color  = "";
              color2 = "";
           }
           QLabel *ebox = new QLabel(this);

// le seguenti per rendere clickabile la label e settare background e foraground
           QString lab("<a href=\"xxx\" style=\"color: '");
           if(!color.isEmpty() && !color2.isEmpty())
           {
               lab.append(color2);
               ebox->setStyleSheet(LabelStyle(color, color2));
           }
           else
               lab.append("#000000");
           lab.append("'\">");
           lab.append(t.c_str());
           lab.append("</a>");
//
           ebox->setText(lab);
           ebox->setMinimumSize(16, 16);
           ebox->setFixedSize(16,16);
           ebox->setAlignment(Qt::AlignCenter);
           hbox->setAlignment(ebox, Qt::AlignLeft|Qt::AlignTop);
           if(which && (r1 != nullptr))
           {
              int katom;
              if(which < N_AMINO)
                 katom = md->FindAtom("CA", *r1);
              else
                 katom = md->FindAtom("O5'", *r1);
              connect(ebox, &QLabel::linkActivated, [=] { goToResidue(katom); });
           }
           ebox->setToolTip(QString(tname.c_str()));
           hbox->addWidget(ebox);
       }
    }
}

void SequenceDialog::goToResidue(int katom)
{
     float posi[3];
     for (int i=0; i<3; i++)
           posi[i] = md->atm[katom].xyzc[i];
     md->MoveCenter(posi, katom, false);
}

void SequenceDialog::on_closeButton_clicked()
{
    this->destroy();
}

void SequenceDialog::exportFasta(QFile &file)
{
    QTextStream ts(&file);
    QString header = get_header(md->modname.c_str());
    for(int kk=0; kk<md->nChains; kk++)
    {
        CHAIN ch = md->Chains[kk];
        if(ch.nAmino == 0)
            continue;
        ts << ">" << header << ":" << ch.id.c_str() << "|PDBID|CHAIN|SEQUENCE" <<  "\n";
        int punt = 0;
        for(int rr=0; rr<ch.nResidues; rr++)
        {
            std::string tname(ch.Residues[rr].name);
            int p = (int) tname.find_last_of( " " );
            std::string rname;
            if (p >= 0)
                rname = tname.substr(p+1);
            else
                rname = tname;

            if (rname.at(0) == ' ')
                rname.erase(rname.begin());
            int which = IsAminoNucle(rname.c_str());
            std::string t;
            if(which)
               t = get_AminoCode(which);
            else
               continue;
            if(t.compare("X"))
            {
               ts << t.c_str();
               punt++;
            }
            if(punt == 80)
            {
               ts << "\n";
               punt = 0;
            }
        }
        ts << "\n";
    }
}

void SequenceDialog::exportPir(QFile &file)
{
    QTextStream ts(&file);
    ts << ">---------+----------+----------+----------+----------+" << "\n\n";
    int punt = 0;
    for(int kk=0; kk<md->nChains; kk++)
    {
        CHAIN ch = md->Chains[kk];
        if(ch.nAmino == 0)
            continue;
        for(int rr=0; rr<ch.nResidues; rr++)
        {
            std::string tname(ch.Residues[rr].name);
            int p = (int) tname.find_last_of( " " );
            std::string rname;
            if (p >= 0)
                rname = tname.substr(p+1);
            else
                rname = tname;

            if (rname.at(0) == ' ')
                rname.erase(rname.begin());
            int which = IsAminoNucle(rname.c_str());
            std::string t;
            if(which)
               t = get_AminoCode(which);
            else
               continue;
            if(t.compare("X"))
            {
               ts << t.c_str();
               punt++;
            }
            if((punt % 10) == 0)
            {
               ts << " ";
               if(punt >= 50)
               {
                  ts << "\n";
                  punt = 0;
               }
            }
        }
    }
    ts << "*\n";
}

void SequenceDialog::on_exportButton_clicked()
{
    QSettings settings;
    QString selectedFilter = "Fasta format(*.seq)";
    QString fileName = SaveDialog::run(this, tr("Export as"),
                                    settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(QString(md->modname.c_str())).baseName(),
                                       "Fasta format(*.seq);;"
                                       "Pir format(*.pir)",
                                       "seq", selectedFilter);
    if (!fileName.isEmpty()) {
         QFile file(fileName);
         if (!file.open(QFile::WriteOnly | QFile::Text)) {
             QMessageBox::warning(this, "Warning", "Cannot save file: " + file.errorString());
             return;
         }
         if(selectedFilter.contains("Fasta"))
             exportFasta(file);
         else
             exportPir(file);
         file.close();
    }
}

void SequenceDialog::on_checkBoxNotAmino_toggled(bool checked)
{
    FillSequence(checked);
}
