#ifndef SG_DATABASE_H
#define SG_DATABASE_H

#include <QString>

namespace sgdat {

enum CrystalSystem {Triclinic, Monoclinic, Orthorhombic, Tetragonal, Trigonal, Hexagonal, Cubic};

int systemFromSgNumber(int sgNumber);

typedef struct
{
    int         SgNumber;
    QString     name;
} TabSpaceGroup;

const TabSpaceGroup TabSg[] =
{
    {   1, "P 1" },
    {   2, "P -1" },
    {   3, "P 2" },
    {   3, "P 1 1 2" },
    {   3, "P 2 1 1" },
    {   4, "P 21" },
    {   4, "P 1 1 21" },
    {   4, "P 21 1 1" },
    {   5, "C 2" },
    {   5, "A 2" },
    {   5, "I 2" },
    {   5, "A 1 1 2" },
    {   5, "B 1 1 2" },
    {   5, "I 1 1 2" },
    {   5, "B 2 1 1" },
    {   5, "C 2 1 1" },
    {   5, "I 2 1 1" },
    {   6, "P m" },
    {   6, "P 1 1 m" },
    {   6, "P m 1 1" },
    {   7, "P c" },
    {   7, "P n" },
    {   7, "P a" },
    {   7, "P 1 1 a" },
    {   7, "P 1 1 n" },
    {   7, "P 1 1 b" },
    {   7, "P b 1 1" },
    {   7, "P n 1 1" },
    {   7, "P c 1 1" },
    {   8, "C m" },
    {   8, "A m" },
    {   8, "I m" },
    {   8, "A 1 1 m" },
    {   8, "B 1 1 m" },
    {   8, "I 1 1 m" },
    {   8, "B m 1 1" },
    {   8, "C m 1 1" },
    {   8, "I m 1 1" },
    {   9, "C c" },
    {   9, "A n" },
    {   9, "I a" },
    {   9, "A a" },
    {   9, "C n" },
    {   9, "I c" },
    {   9, "A 1 1 a" },
    {   9, "B 1 1 n" },
    {   9, "I 1 1 b" },
    {   9, "B 1 1 b" },
    {   9, "A 1 1 n" },
    {   9, "I 1 1 a" },
    {   9, "B b 1 1" },
    {   9, "C n 1 1" },
    {   9, "I c 1 1" },
    {   9, "C c 1 1" },
    {   9, "B n 1 1" },
    {   9, "I b 1 1" },
    {  10, "P 2/m" },
    {  10, "P 1 1 2/m" },
    {  10, "P 2/m 1 1" },
    {  11, "P 21/m" },
    {  11, "P 1 1 21/m" },
    {  11, "P 21/m 1 1" },
    {  12, "C 2/m" },
    {  12, "A 2/m" },
    {  12, "I 2/m" },
    {  12, "A 1 1 2/m" },
    {  12, "B 1 1 2/m" },
    {  12, "I 1 1 2/m" },
    {  12, "B 2/m 1 1" },
    {  12, "C 2/m 1 1" },
    {  12, "I 2/m 1 1" },
    {  13, "P 2/c" },
    {  13, "P 2/n" },
    {  13, "P 2/a" },
    {  13, "P 1 1 2/a" },
    {  13, "P 1 1 2/n" },
    {  13, "P 1 1 2/b" },
    {  13, "P 2/b 1 1" },
    {  13, "P 2/n 1 1" },
    {  13, "P 2/c 1 1" },
    {  14, "P 21/c" },
    {  14, "P 21/n" },
    {  14, "P 21/a" },
    {  14, "P 1 1 21/a" },
    {  14, "P 1 1 21/n" },
    {  14, "P 1 1 21/b" },
    {  14, "P 21/b 1 1" },
    {  14, "P 21/n 1 1" },
    {  14, "P 21/c 1 1" },
    {  15, "C 2/c" },
    {  15, "A 2/n" },
    {  15, "I 2/a" },
    {  15, "A 2/a" },
    {  15, "C 2/n" },
    {  15, "I 2/c" },
    {  15, "A 1 1 2/a" },
    {  15, "B 1 1 2/n" },
    {  15, "I 1 1 2/b" },
    {  15, "B 1 1 2/b" },
    {  15, "A 1 1 2/n" },
    {  15, "I 1 1 2/a" },
    {  15, "B 2/b 1 1" },
    {  15, "C 2/n 1 1" },
    {  15, "I 2/c 1 1" },
    {  15, "C 2/c 1 1" },
    {  15, "B 2/n 1 1" },
    {  15, "I 2/b 1 1" },
    {  16, "P 2 2 2" },
    {  17, "P 2 2 21" },
    {  17, "P 21 2 2" },
    {  17, "P 2 21 2" },
    {  18, "P 21 21 2" },
    {  18, "P 2 21 21" },
    {  18, "P 21 2 21" },
    {  19, "P 21 21 21" },
    {  20, "C 2 2 21" },
    {  20, "A 21 2 2" },
    {  20, "B 2 21 2" },
    {  21, "C 2 2 2" },
    {  21, "A 2 2 2" },
    {  21, "B 2 2 2" },
    {  22, "F 2 2 2" },
    {  23, "I 2 2 2" },
    {  24, "I 21 21 21" },
    {  25, "P m m 2" },
    {  25, "P 2 m m" },
    {  25, "P m 2 m" },
    {  26, "P m c 21" },
    {  26, "P c m 21" },
    {  26, "P 21 m a" },
    {  26, "P 21 a m" },
    {  26, "P b 21 m" },
    {  26, "P m 21 b" },
    {  27, "P c c 2" },
    {  27, "P 2 a a" },
    {  27, "P b 2 b" },
    {  28, "P m a 2" },
    {  28, "P b m 2" },
    {  28, "P 2 m b" },
    {  28, "P 2 c m" },
    {  28, "P c 2 m" },
    {  28, "P m 2 a" },
    {  29, "P c a 21" },
    {  29, "P b c 21" },
    {  29, "P 21 a b" },
    {  29, "P 21 c a" },
    {  29, "P c 21 b" },
    {  29, "P b 21 a" },
    {  30, "P n c 2" },
    {  30, "P c n 2" },
    {  30, "P 2 n a" },
    {  30, "P 2 a n" },
    {  30, "P b 2 n" },
    {  30, "P n 2 b" },
    {  31, "P m n 21" },
    {  31, "P n m 21" },
    {  31, "P 21 m n" },
    {  31, "P 21 n m" },
    {  31, "P n 21 m" },
    {  31, "P m 21 n" },
    {  32, "P b a 2" },
    {  32, "P 2 c b" },
    {  32, "P c 2 a" },
    {  33, "P n a 21" },
    {  33, "P b n 21" },
    {  33, "P 21 n b" },
    {  33, "P 21 c n" },
    {  33, "P c 21 n" },
    {  33, "P n 21 a" },
    {  34, "P n n 2" },
    {  34, "P 2 n n" },
    {  34, "P n 2 n" },
    {  35, "C m m 2" },
    {  35, "A 2 m m" },
    {  35, "B m 2 m" },
    {  36, "C m c 21" },
    {  36, "C c m 21" },
    {  36, "A 21 m a" },
    {  36, "A 21 a m" },
    {  36, "B b 21 m" },
    {  36, "B m 21 b" },
    {  37, "C c c 2" },
    {  37, "A 2 a a" },
    {  37, "B b 2 b" },
    {  38, "A m m 2" },
    {  38, "B m m 2" },
    {  38, "B 2 m m" },
    {  38, "C 2 m m" },
    {  38, "C m 2 m" },
    {  38, "A m 2 m" },
    {  39, "A b m 2" },
    {  39, "B m a 2" },
    {  39, "B 2 c m" },
    {  39, "C 2 m b" },
    {  39, "C m 2 a" },
    {  39, "A c 2 m" },
    {  40, "A m a 2" },
    {  40, "B b m 2" },
    {  40, "B 2 m b" },
    {  40, "C 2 c m" },
    {  40, "C c 2 m" },
    {  40, "A m 2 a" },
    {  41, "A b a 2" },
    {  41, "B b a 2" },
    {  41, "B 2 c b" },
    {  41, "C 2 c b" },
    {  41, "C c 2 a" },
    {  41, "A c 2 a" },
    {  42, "F m m 2" },
    {  42, "F 2 m m" },
    {  42, "F m 2 m" },
    {  43, "F d d 2" },
    {  43, "F 2 d d" },
    {  43, "F d 2 d" },
    {  44, "I m m 2" },
    {  44, "I 2 m m" },
    {  44, "I m 2 m" },
    {  45, "I b a 2" },
    {  45, "I 2 c b" },
    {  45, "I c 2 a" },
    {  46, "I m a 2" },
    {  46, "I b m 2" },
    {  46, "I 2 m b" },
    {  46, "I 2 c m" },
    {  46, "I c 2 m" },
    {  46, "I m 2 a" },
    {  47, "P m m m" },
    {  48, "P n n n :1" },
    {  48, "P n n n :2" },
    {  49, "P c c m" },
    {  49, "P m a a" },
    {  49, "P b m b" },
    {  50, "P b a n :1" },
    {  50, "P b a n :2" },
    {  50, "P n c b :1" },
    {  50, "P n c b :2" },
    {  50, "P c n a :1" },
    {  50, "P c n a :2" },
    {  51, "P m m a" },
    {  51, "P m m b" },
    {  51, "P b m m" },
    {  51, "P c m m" },
    {  51, "P m c m" },
    {  51, "P m a m" },
    {  52, "P n n a" },
    {  52, "P n n b" },
    {  52, "P b n n" },
    {  52, "P c n n" },
    {  52, "P n c n" },
    {  52, "P n a n" },
    {  53, "P m n a" },
    {  53, "P n m b" },
    {  53, "P b m n" },
    {  53, "P c n m" },
    {  53, "P n c m" },
    {  53, "P m a n" },
    {  54, "P c c a" },
    {  54, "P c c b" },
    {  54, "P b a a" },
    {  54, "P c a a" },
    {  54, "P b c b" },
    {  54, "P b a b" },
    {  55, "P b a m" },
    {  55, "P m c b" },
    {  55, "P c m a" },
    {  56, "P c c n" },
    {  56, "P n a a" },
    {  56, "P b n b" },
    {  57, "P b c m" },
    {  57, "P c a m" },
    {  57, "P m c a" },
    {  57, "P m a b" },
    {  57, "P b m a" },
    {  57, "P c m b" },
    {  58, "P n n m" },
    {  58, "P m n n" },
    {  58, "P n m n" },
    {  59, "P m m n :1" },
    {  59, "P m m n :2" },
    {  59, "P n m m :1" },
    {  59, "P n m m :2" },
    {  59, "P m n m :1" },
    {  59, "P m n m :2" },
    {  60, "P b c n" },
    {  60, "P c a n" },
    {  60, "P n c a" },
    {  60, "P n a b" },
    {  60, "P b n a" },
    {  60, "P c n b" },
    {  61, "P b c a" },
    {  61, "P c a b" },
    {  62, "P n m a" },
    {  62, "P m n b" },
    {  62, "P b n m" },
    {  62, "P c m n" },
    {  62, "P m c n" },
    {  62, "P n a m" },
    {  63, "C m c m" },
    {  63, "C c m m" },
    {  63, "A m m a" },
    {  63, "A m a m" },
    {  63, "B b m m" },
    {  63, "B m m b" },
    {  64, "C m c a" },
    {  64, "C c m b" },
    {  64, "A b m a" },
    {  64, "A c a m" },
    {  64, "B b c m" },
    {  64, "B m a b" },
    {  65, "C m m m" },
    {  65, "A m m m" },
    {  65, "B m m m" },
    {  66, "C c c m" },
    {  66, "A m a a" },
    {  66, "B b m b" },
    {  67, "C m m a" },
    {  67, "C m m b" },
    {  67, "A b m m" },
    {  67, "A c m m" },
    {  67, "B m c m" },
    {  67, "B m a m" },
    {  68, "C c c a :1" },
    {  68, "C c c a :2" },
    {  68, "C c c b :1" },
    {  68, "C c c b :2" },
    {  68, "A b a a :1" },
    {  68, "A b a a :2" },
    {  68, "A c a a :1" },
    {  68, "A c a a :2" },
    {  68, "B b c b :1" },
    {  68, "B b c b :2" },
    {  68, "B b a b :1" },
    {  68, "B b a b :2" },
    {  69, "F m m m" },
    {  70, "F d d d :1" },
    {  70, "F d d d :2" },
    {  71, "I m m m" },
    {  72, "I b a m" },
    {  72, "I m c b" },
    {  72, "I c m a" },
    {  73, "I b c a" },
    {  73, "I c a b" },
    {  74, "I m m a" },
    {  74, "I m m b" },
    {  74, "I b m m" },
    {  74, "I c m m" },
    {  74, "I m c m" },
    {  74, "I m a m" },
    {  75, "P 4" },
    {  76, "P 41" },
    {  77, "P 42" },
    {  78, "P 43" },
    {  79, "I 4" },
    {  80, "I 41" },
    {  81, "P -4" },
    {  82, "I -4" },
    {  83, "P 4/m" },
    {  84, "P 42/m" },
    {  85, "P 4/n :1" },
    {  85, "P 4/n :2" },
    {  86, "P 42/n :1" },
    {  86, "P 42/n :2" },
    {  87, "I 4/m" },
    {  88, "I 41/a :1" },
    {  88, "I 41/a :2" },
    {  89, "P 4 2 2" },
    {  90, "P 4 21 2" },
    {  91, "P 41 2 2" },
    {  92, "P 41 21 2" },
    {  93, "P 42 2 2" },
    {  94, "P 42 21 2" },
    {  95, "P 43 2 2" },
    {  96, "P 43 21 2" },
    {  97, "I 4 2 2" },
    {  98, "I 41 2 2" },
    {  99, "P 4 m m" },
    { 100, "P 4 b m" },
    { 101, "P 42 c m" },
    { 102, "P 42 n m" },
    { 103, "P 4 c c" },
    { 104, "P 4 n c" },
    { 105, "P 42 m c" },
    { 106, "P 42 b c" },
    { 107, "I 4 m m" },
    { 108, "I 4 c m" },
    { 109, "I 41 m d" },
    { 110, "I 41 c d" },
    { 111, "P -4 2 m" },
    { 112, "P -4 2 c" },
    { 113, "P -4 21 m" },
    { 114, "P -4 21 c" },
    { 115, "P -4 m 2" },
    { 116, "P -4 c 2" },
    { 117, "P -4 b 2" },
    { 118, "P -4 n 2" },
    { 119, "I -4 m 2" },
    { 120, "I -4 c 2" },
    { 121, "I -4 2 m" },
    { 122, "I -4 2 d" },
    { 123, "P 4/m m m" },
    { 124, "P 4/m c c" },
    { 125, "P 4/n b m :1" },
    { 125, "P 4/n b m :2" },
    { 126, "P 4/n n c :1" },
    { 126, "P 4/n n c :2" },
    { 127, "P 4/m b m" },
    { 128, "P 4/m n c" },
    { 129, "P 4/n m m :1" },
    { 129, "P 4/n m m :2" },
    { 130, "P 4/n c c :1" },
    { 130, "P 4/n c c :2" },
    { 131, "P 42/m m c" },
    { 132, "P 42/m c m" },
    { 133, "P 42/n b c :1" },
    { 133, "P 42/n b c :2" },
    { 134, "P 42/n n m :1" },
    { 134, "P 42/n n m :2" },
    { 135, "P 42/m b c" },
    { 136, "P 42/m n m" },
    { 137, "P 42/n m c :1" },
    { 137, "P 42/n m c :2" },
    { 138, "P 42/n c m :1" },
    { 138, "P 42/n c m :2" },
    { 139, "I 4/m m m" },
    { 140, "I 4/m c m" },
    { 141, "I 41/a m d :1" },
    { 141, "I 41/a m d :2" },
    { 142, "I 41/a c d :1" },
    { 142, "I 41/a c d :2" },
    { 143, "P 3" },
    { 144, "P 31" },
    { 145, "P 32" },
    { 146, "R 3 :H" },
    { 146, "R 3 :R" },
    { 147, "P -3" },
    { 148, "R -3 :H" },
    { 148, "R -3 :R" },
    { 149, "P 3 1 2" },
    { 150, "P 3 2 1" },
    { 151, "P 31 1 2" },
    { 152, "P 31 2 1" },
    { 153, "P 32 1 2" },
    { 154, "P 32 2 1" },
    { 155, "R 3 2 :H" },
    { 155, "R 3 2 :R" },
    { 156, "P 3 m 1" },
    { 157, "P 3 1 m" },
    { 158, "P 3 c 1" },
    { 159, "P 3 1 c" },
    { 160, "R 3 m :H" },
    { 160, "R 3 m :R" },
    { 161, "R 3 c :H" },
    { 161, "R 3 c :R" },
    { 162, "P -3 1 m" },
    { 163, "P -3 1 c" },
    { 164, "P -3 m 1" },
    { 165, "P -3 c 1" },
    { 166, "R -3 m :H" },
    { 166, "R -3 m :R" },
    { 167, "R -3 c :H" },
    { 167, "R -3 c :R" },
    { 168, "P 6" },
    { 169, "P 61" },
    { 170, "P 65" },
    { 171, "P 62" },
    { 172, "P 64" },
    { 173, "P 63" },
    { 174, "P -6" },
    { 175, "P 6/m" },
    { 176, "P 63/m" },
    { 177, "P 6 2 2" },
    { 178, "P 61 2 2" },
    { 179, "P 65 2 2" },
    { 180, "P 62 2 2" },
    { 181, "P 64 2 2" },
    { 182, "P 63 2 2" },
    { 183, "P 6 m m" },
    { 184, "P 6 c c" },
    { 185, "P 63 c m" },
    { 186, "P 63 m c" },
    { 187, "P -6 m 2" },
    { 188, "P -6 c 2" },
    { 189, "P -6 2 m" },
    { 190, "P -6 2 c" },
    { 191, "P 6/m m m" },
    { 192, "P 6/m c c" },
    { 193, "P 63/m c m" },
    { 194, "P 63/m m c" },
    { 195, "P 2 3" },
    { 196, "F 2 3" },
    { 197, "I 2 3" },
    { 198, "P 21 3" },
    { 199, "I 21 3" },
    { 200, "P m -3" },
    { 201, "P n -3 :1" },
    { 201, "P n -3 :2" },
    { 202, "F m -3" },
    { 203, "F d -3 :1" },
    { 203, "F d -3 :2" },
    { 204, "I m -3" },
    { 205, "P a -3" },
    { 206, "I a -3" },
    { 207, "P 4 3 2" },
    { 208, "P 42 3 2" },
    { 209, "F 4 3 2" },
    { 210, "F 41 3 2" },
    { 211, "I 4 3 2" },
    { 212, "P 43 3 2" },
    { 213, "P 41 3 2" },
    { 214, "I 41 3 2" },
    { 215, "P -4 3 m" },
    { 216, "F -4 3 m" },
    { 217, "I -4 3 m" },
    { 218, "P -4 3 n" },
    { 219, "F -4 3 c" },
    { 220, "I -4 3 d" },
    { 221, "P m -3 m" },
    { 222, "P n -3 n :1" },
    { 222, "P n -3 n :2" },
    { 223, "P m -3 n" },
    { 224, "P n -3 m :1" },
    { 224, "P n -3 m :2" },
    { 225, "F m -3 m" },
    { 226, "F m -3 c" },
    { 227, "F d -3 m :1" },
    { 227, "F d -3 m :2" },
    { 228, "F d -3 c :1" },
    { 228, "F d -3 c :2" },
    { 229, "I m -3 m" },
    { 230, "I a -3 d" }
};

const int sizeSgTable = sizeof(TabSg)/sizeof(TabSg[0]);

}
#endif // SG_DATABASE_H
