#include "spacegrouplistdialog.h"
#include "ui_spacegrouplistdialog.h"
#include "floatdelegate.h"
#include "mainwindow.h"

#include <QDebug>
#include <QPushButton>
#include <QDialog>

extern "C" void spg_selection(int row);

SpaceGroupListDialog::SpaceGroupListDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SpaceGroupListDialog),
    absRefDialog(new AbsRefDialog(this))
{
    ui->setupUi(this);

    QPushButton *listButton = new QPushButton("&List");
    listButton->setIcon(QIcon(":/pixmaps/pixmaps/output.xpm"));
    ui->buttonBox->addButton(listButton, QDialogButtonBox::HelpRole);

    spaceListModel = new QStandardItemModel(this);
    proxyModel = new QSortFilterProxyModel(this);
    proxyModel->setSourceModel(spaceListModel);

    spaceListModel->setColumnCount(9);
    QStringList titles = {"Space Group", "Extinction symbol", "FoM", "Nabs", "Nasym",
                          "No. in CSD", "% of CSD", "Rank", "Chiral"};
    spaceListModel->setHorizontalHeaderLabels(titles);

    ui->spaceTableView->setModel(proxyModel);
    ui->spaceTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->spaceTableView->setSortingEnabled(true);
    ui->spaceTableView->horizontalHeader()->setSortIndicator(2,Qt::DescendingOrder); //In default the table is ordered by FoM (column 2)
    FloatDelegate *fomDelegate = new FloatDelegate(ui->spaceTableView);
    ui->spaceTableView->setItemDelegateForColumn(2, fomDelegate);
    FloatDelegate *csdDelegate = new FloatDelegate(ui->spaceTableView,2);
    ui->spaceTableView->setItemDelegateForColumn(6, csdDelegate);    

    connect(ui->spaceTableView->selectionModel(),&QItemSelectionModel::currentRowChanged,this,
            [this](const QModelIndex &current){
        QModelIndex index = proxyModel->mapToSource(current);
        spg_selection(index.row()+1);
        if (absRefDialog->isVisible()) absRefDialog->updateAbsRefList(index.row());
    });

    connect(listButton, &QPushButton::clicked, this, [this](){
       absRefDialog->show();
       absRefDialog->updateAbsRefList(selectedSpaceGroup());
    });

    connect(this, &SpaceGroupListDialog::finished, this, &SpaceGroupListDialog::onSpaceGroupListDialogFinished);
    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &SpaceGroupListDialog::onButtonBoxClicked);
}

SpaceGroupListDialog::~SpaceGroupListDialog()
{
    delete ui;
}

void SpaceGroupListDialog::createModel(spgext_info_type spaceList[], int listSize, int selectedSpace)
{
    spaceListModel->setRowCount(listSize);
    for (int r = 0; r < listSize; r++) {
        QStandardItem *itemSpace = new QStandardItem(spaceList[r].symb);
        spaceListModel->setItem(r,0,itemSpace);

        QStandardItem *itemExtS = new QStandardItem(spaceList[r].exts);
        spaceListModel->setItem(r,1,itemExtS);

        QStandardItem *itemFoM = new QStandardItem;
        itemFoM->setData(spaceList[r].fig,Qt::DisplayRole);
        spaceListModel->setItem(r,2,itemFoM);

        QStandardItem *itemNabs = new QStandardItem;
        itemNabs->setData(spaceList[r].nabs,Qt::DisplayRole);
        itemNabs->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        spaceListModel->setItem(r,3,itemNabs);

        QStandardItem *itemNasym = new QStandardItem;
        itemNasym->setData(spaceList[r].nasym,Qt::DisplayRole);
        itemNasym->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        spaceListModel->setItem(r,4,itemNasym);

        QStandardItem *itemNCSD = new QStandardItem;
        itemNCSD->setData(spaceList[r].freq_no,Qt::DisplayRole);
        itemNCSD->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        spaceListModel->setItem(r,5,itemNCSD);

        QStandardItem *itemPCSD = new QStandardItem;
        itemPCSD->setData(spaceList[r].freq_perc,Qt::DisplayRole);        
        spaceListModel->setItem(r,6,itemPCSD);

        QStandardItem *itemRank = new QStandardItem;
        itemRank->setData(spaceList[r].freq_rank,Qt::DisplayRole);
        itemRank->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        spaceListModel->setItem(r,7,itemRank);

        QStandardItem *itemChiral = new QStandardItem(spaceList[r].chiral ? "Yes" : "No");
        spaceListModel->setItem(r,8,itemChiral);
    }

    ui->spaceTableView->sortByColumn(2,Qt::DescendingOrder);

    //force visualization of absent reflections
    if(ui->spaceTableView->currentIndex().row() == selectedSpace) spg_selection(selectedSpace+1);

    ui->spaceTableView->selectRow(selectedSpace);
    ui->spaceTableView->resizeColumnsToContents();
}

void SpaceGroupListDialog::waitSignalIsEmitted()
{
    QEventLoop loop;
    connect(this,&QDialog::finished,&loop,&QEventLoop::quit);
    loop.exec();
}

int SpaceGroupListDialog::selectedSpaceGroup() const
{
    QModelIndex index = ui->spaceTableView->currentIndex();
    return proxyModel->mapToSource(index).row();
}

void SpaceGroupListDialog::onSpaceGroupListDialogFinished(int)
{    
    absRefDialog->close();
    MainWindow *mw = qobject_cast<MainWindow *>(this->parent());
    mw->clearSystematicAbsences();
}

void SpaceGroupListDialog::onButtonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Help) {
        MainWindow *mw = qobject_cast<MainWindow *>(this->parent());
        mw->openHelp("space-group-determination-2");
    }
}
