MODULE plot2

use :: iso_c_binding

interface

   subroutine init_plot2(xLabel, yLabel) bind(c)
   use, intrinsic :: iso_c_binding, only: c_char
   character(kind=c_char), dimension(*) :: xLabel, yLabel
   end subroutine init_plot2

   subroutine enable_rescale(rescale) bind(C, name="enable_rescale")
   use, intrinsic :: iso_c_binding, only: c_int
   integer(c_int), intent(in), value :: rescale
   end subroutine enable_rescale

   subroutine add_plot(x,y,num,ktype,visible,wave,pname) bind(C,name="add_plot")
   use, intrinsic :: iso_c_binding, only: c_int, c_float, c_char
   real(c_float), dimension(*)          :: x,y
   integer(c_int), value                :: num,ktype,visible
   real(c_float), value                 :: wave
   character(kind=c_char), dimension(*) :: pname
   end subroutine add_plot

   subroutine add_plot2(x,num,ktype,visible,wave) bind(C,name="add_plot2")
   use, intrinsic :: iso_c_binding, only: c_int, c_float
   real(c_float), dimension(*) :: x
   integer(c_int), value       :: num,ktype,visible
   real(c_float), value        :: wave
   end subroutine add_plot2

   subroutine draw_graphic() bind(C,name="draw_graphic")
   end subroutine draw_graphic

   subroutine set_graphic_area() bind(C,name="set_graphic_area")
   end subroutine set_graphic_area
end interface

END MODULE plot2

MODULE view

   use plotstyle

   implicit none

   !private

   public  :: vedinew,plottanew,process_action_points

   type, private :: peak_type
     real    :: xpos,ypos  ! posizione del picco
     integer :: col        ! colore del picco
   end type 

   type gr_phase_type
     integer, dimension(:,:), allocatable :: hkl
     real, dimension(:), allocatable      :: tth
   end type
   type :: pd_graph_type
     type(gr_phase_type), dimension(:), allocatable :: grph
   end type pd_graph_type

   logical, public :: kzoomn = .FALSE.
   integer, public :: xzleft,xzright
   real, private :: xzoom1,xzoom2

   integer, public                                     :: pd_action 
   integer, public                                     :: pinioss_sav,pfinoss_sav
   integer, public                                     :: pinicalc_sav,pfincalc_sav
   integer, public                                     :: iAct_sav = 1

   integer, dimension(:), allocatable, public :: idph

   type(style_type), dimension(:), allocatable, public :: stylecr_sav,styledt_sav

     enum, bind(c)
       enumerator :: Observed,Calculated,BBackground,Background_Points,         &
                     Difference,Cumulative,Reflections,Peaks,Smoothing,         &
                     Unindexed_Peaks,Systematic_Absences,Selected_Reflections,  &
                     Selected_Peaks,Intervals,Profile_Curves
     endenum

   CONTAINS

     subroutine vedinew(iAct,pinioss,pfinoss,pinicalc,pfincalc,lsetstyle,rescale)
     use, intrinsic :: iso_c_binding, only: c_int
     USE PatternRef
     USE Conteggi, only : theta_int,back_int
     USE CommonExpo, only : backp,scala,nback
     USE Riflessi, only : TThet
     USE General, only : nrefl 
     USE plotstyle
     USE variables, only: dataset
     USE datasetmod, only: ndataset
     USE arrayutil
     use plot2, only: enable_rescale
!
     integer, intent(in), optional                 :: iAct
     integer, intent(in), optional                 :: pinioss,pfinoss   !limiti per l'osservato
     integer, intent(in), optional                 :: pinicalc,pfincalc !limiti per il calcolato
     logical, intent(in), optional                 :: lsetstyle         !se vero setta lo stile all'interno
     integer, intent(in), optional                 :: rescale
     integer                                       :: pinicalc1,pfincalc1
     real, dimension(:,:), allocatable             :: backpur
     integer                                       :: i
     integer, dimension(:), allocatable :: inid,ifid
!
     if (present(iAct)) then
         pd_action = iAct
         if (present(lsetstyle)) then
             if (lsetstyle) call set_plot_style(pd_action)
         else
             call set_plot_style(pd_action)
         endif
     endif
     if (present(pinioss))then         
         xzleft = pinioss
         xzright = pfinoss
     endif
     if (present(pinicalc)) then
         pinicalc1 = pinicalc
         pfincalc1 = pfincalc
     else
         pinicalc1 = xzleft
         pfincalc1 = xzright
     endif
     if (present(rescale)) then
         call enable_rescale(rescale)
     else
         call enable_rescale(1)
     endif
!     
     select case (pd_action)
       case (1)
         if (allocated(dataset(1)%yb) .and. dataset(1)%npoints_back() > 0) then
             call dataset(1)%get_points(backpur)
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet,   &
                            ybac=dataset(1)%yb(pinicalc1:pfincalc1),bacp=backpur)
         else
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet)
         endif

       case (2)
         !call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet(:nrefl))
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet)

       case (3)
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),tthr=TThet,  &
         ybac=back_int(pinicalc1:pfincalc1,1),bacp=backp(:nback,:))

       case (4)
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),     & 
         scala(jjj)*theta_int(pinicalc1:pfincalc1,3),TThet,back_int(pinicalc1:pfincalc1,1), &
         bacp=backp(:nback,:))

       case (5)
         call dataset(1)%get_points(backpur)
         if (nrefl == 0) then
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,dataset(1)%x0,dataset(1)%y,ybac=dataset(1)%yb(pinicalc1:pfincalc1), &
             bacp=backpur)
         else
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,dataset(1)%x0,dataset(1)%y,ybac=dataset(1)%yb(pinicalc1:pfincalc1), &
             tthr=TThet,bacp=backpur)
         endif

       case (6)
         if (nrefl == 0) then
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
             scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),bacp=backp(:nback,:))
         else
             call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
             scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet,  & 
             bacp=backp(:nback,:))
         endif
  
       case (7)
         call plottanew(xzleft,xzright,pinicalc1,pfincalc1,theta_int(:,7),theta_int(:,2),      &
         scala(1)*theta_int(pinicalc1:pfincalc1,3),ybac=back_int(pinicalc1:pfincalc1,1),tthr=TThet,  &   
         bacp=backp(:nback,:))

       case (8)
         allocate(inid(ndataset(dataset)))
         allocate(ifid(ndataset(dataset)))
         if (kzoomn) then
             do i=1,ndataset(dataset)
                inid(i) = clocate(dataset(i)%x0,xzoom1)
                ifid(i) = clocate(dataset(i)%x0,xzoom2)         
             enddo
         else
             do i=1,ndataset(dataset)
                inid(i) = 1
                ifid(i) = size(dataset(i)%x)
             enddo
         endif
         call plottanew2(inid,ifid)   !!!!,pinicalc1,pfincalc1)

     end select
!
!    salva limiti per redraw
     pinioss_sav = xzleft
     pfinoss_sav = xzright
     pinicalc_sav = pinicalc1
     pfincalc_sav = pfincalc1
     iAct_sav = pd_action
!
     end subroutine vedinew

  !-----------------------------------------------------------------------------------------------

     subroutine plottanew(inic,ific,inic1,ific1,ttet,yoss,ycal,tthr,ybac,bacp)
     USE peak_mod
     USE plotstyle
     USE variables, only: cryst,dataset
     USE General, only: wavel
     USE arrayutil, only: clocate
     USE CommonExpo, only : nnt,ncoun1,ncoun2
     USE fileutil
     USE plot2
     USE strutil, only: toCString
     USE PatternRef, only: kprofile,kprofile_type,PROFILE_EXTRA
!
     integer, intent(in)                                   :: inic  ! conteggio iniziale per l'osservato
     integer, intent(in)                                   :: ific  ! conteggio finale per l'osservato
     integer, intent(in)                                   :: inic1 ! conteggio iniziale per background e calcolato
     integer, intent(in)                                   :: ific1 ! conteggio finale per background e calcolato
     real, dimension(:), intent(in)                        :: ttet  ! 2theta del conteggio
     real, dimension(:), intent(in)                        :: yoss  ! conteggio osservato
     real, dimension(:), intent(in), optional              :: ycal  ! conteggio calcolato
     real, dimension(:), intent(in), allocatable, optional :: tthr  ! 2theta del riflesso 
     real, dimension(:), intent(in), optional              :: ybac  ! background
     real, dimension(:,:), intent(in), optional            :: bacp  ! punti di background
     real, dimension(ific1-inic1+1)                        :: diffc 
     real, dimension(ific1-inic1+1)                        :: ycalb,ydiff
     real, dimension(4)                                    :: maxview,minview
     real                                                  :: maxv,minv
     integer                                               :: nplot
     real                                                  :: trdiff
     integer                                               :: i,j
     real                                                  :: maxdiff,mindiff
     real                                                  :: adiff
     integer                                               :: ncalcc,startp,endp,npk
     logical                                               :: drawref, drawcalc, drawdiff
     real, dimension(:), allocatable                       :: intpeak
     integer                                               :: ninf, nsup
     real, allocatable, dimension(:)                       :: intervalx  !,intervaly
     integer                                               :: npinter
!
     call set_graphic_area()
     ncalcc = ific1 - inic1 + 1
!
!    Calcola il massimo e minimo valore lungo y
!corr     nplot = 1
!corr     maxview(nplot) = MAXVAL(yoss(inic:ific))
!corr     minview(nplot) = MINVAL(yoss(inic:ific))
     drawcalc = style(STYLE_CALC)%vis == 1 .and. present(ycal) .and. present(ybac)
     if (drawcalc) then
!corr         nplot = nplot + 1
         ycalb = ycal(:ncalcc)+ybac(:ncalcc)
!corr         maxview(nplot) = MAXVAL(ycalb)
!corr         minview(nplot) = MINVAL(ycalb)
     endif
!corr     if (style(STYLE_BACK)%vis == 1 .and. present(ybac)) then
!corr         nplot = nplot + 1
!corr         maxview(nplot) = MAXVAL(ybac)
!corr         minview(nplot) = MINVAL(ybac)
!corr     endif
     drawdiff = style(STYLE_DIFF)%vis == 1 .and. present(ycal) .and. present(ybac)
     if (drawdiff) then
!corr         trdiff = -nint(0.07*MAXVAL(maxview(1:nplot)))  ! traslazione asse differenza
!corr         nplot = nplot + 1
         ydiff = yoss(inic1:ific1)-ycal(:ncalcc)-ybac(:ncalcc)
!corr         maxview(nplot)=MAXVAL(ydiff+trdiff)
!corr         minview(nplot)=MINVAL(ydiff+trdiff)
     endif
!corr     maxv = MAXVAL(maxview(:nplot))
!corr     minv = MINVAL(minview(:nplot))
!qt     maxv = maxv !+ trpeak
!
     drawref = .false.
     if (cryst(1)%style%vis == 1 .and. present(tthr)) then
         if (allocated(tthr)) then
             drawref = .true.
         endif
     endif
!
!    Disegna il profilo osservato
     if (dataset(1)%style%vis == 1) then
         call add_plot(ttet(inic:ific),yoss(inic:ific),ific-inic+1,Observed,1,  &
                 dataset(1)%wave(1),toCString(file_basename(dataset(1)%fname)))
     endif
!
!    Disegna il profilo calcolato
     if (drawcalc) then
         call add_plot(ttet(inic1:ific1),ycalb,ific1-inic1+1,Calculated,1,wavel,c_null_char)
     endif
!
!    Disegna il background
     if (style(STYLE_BACK)%vis == 1 .and. present(ybac)) then
         call add_plot(ttet(inic1:ific1),ybac(:ncalcc),ific1-inic1+1,BBackground,1,wavel,c_null_char)
         if (style(STYLE_BACKP)%vis == 1 .and. present(bacp)) then    ! disegna i punti di background
             call add_plot(bacp(:,1),bacp(:,2),size(bacp,1),Background_Points,1,wavel,c_null_char)
         endif
     endif
!
!    Disegna differenza
     if (drawdiff) then
         call add_plot(ttet(inic1:ific1),ydiff,ific1-inic1+1,Difference,1,wavel,c_null_char)
     endif
!
!    calcola e disegna differrenza cumulativa
     if (style(STYLE_CUMUL)%vis == 1) then
         diffc(1) = abs(ydiff(inic1))
         do i=inic1+1,ific1
            j = i-inic1+1
            diffc(j) = diffc(j-1) + abs(ydiff(i))
         enddo
!
!        riscala diffc tra 0 e il massimo
         maxdiff = diffc(j)
         mindiff = diffc(1)
         adiff = diffc(j) - diffc(1)
         maxv = MAXVAL(ycalb)
         diffc = diffc*(maxv)/adiff - maxv*diffc(1)/adiff
         call add_plot(ttet(inic1:ific1),diffc,ific1-inic1+1,Cumulative,1,wavel,c_null_char)
     endif
!
!    Disegna i marcatori di picco 
     if (style(STYLE_PEAKS)%vis == 1) then
         if (numpeaks(pkind) > 0) then
             startp = clocate(pkind(:)%getx(),ttet(inic))
             if (pkind(startp)%getx() < ttet(inic)) startp = startp +1 
             endp = clocate(pkind(:)%getx(),ttet(ific))
             if (pkind(endp)%getx() > ttet(ific)) endp = endp -1 
             npk = endp - startp + 1
             if (npk > 0) then
                 allocate(intpeak(npk))
                 do i=1,npk
                    intpeak(i) = peak_intensity(pkind(startp+i-1)%getx(),dataset(1)%x0,dataset(1)%y)
                 enddo
                 call add_plot(pkind(startp:endp)%getx(),intpeak,npk,Peaks,1,wavel,c_null_char)
             endif
         endif
     endif
!
!    Disegna i riflessi
     if (drawref) then
         call add_plot2(tthr,size(tthr),Reflections,1,wavel)
     endif
!
!    Disegna gli intervalli
     if (style(STYLE_INTERVALS)%vis == 1) then
         npinter = (nnt-1)*2
         allocate(intervalx(npinter)) 
         npinter = 0
         do i=1,nnt-1
            ninf = ncoun1(i)
            nsup = ncoun2(i)
            npinter = npinter + 1
            intervalx(npinter) = ttet(ninf)
            npinter = npinter + 1
            intervalx(npinter) = ttet(nsup)
         enddo
         call add_plot2(intervalx,npinter,Intervals,1,wavel)
     endif
!
     if (kprofile == 1) then
         if (kprofile_type == PROFILE_EXTRA) then
             call profrefnew(ttet(inic),ttet(ific))  
         else
             call prof_curves_computation(ttet(inic),ttet(ific))
         endif
     endif
!
     call draw_graphic()
!
     end subroutine plottanew

  !-----------------------------------------------------------------------------------------------

     subroutine plottanew2(inic,ific)
     USE peak_mod
     USE plotstyle
     USE variables, only: cryst,dataset
     USE crystal_phase
     USE datasetmod
     USE arrayutil
     USE fileutil
     USE plot2
     USE strutil, only: toCString
     USE PatternRef, only: kprofile,kprofile_type,PROFILE_EXTRA
     USE background, only: BK_NONE
!
     integer, intent(in), dimension(:) :: inic  ! conteggio iniziale per l'osservato
     integer, intent(in), dimension(:) :: ific  ! conteggio finale per l'osservato
     integer                           :: inic1,ific1
     real, dimension(:), allocatable   :: ycalb,ydiff,diffc
     real                              :: maxview,minview
     real                              :: maxv,minv,diff
     real                              :: trdiff    !!!!!,trpeak
     integer                           :: i,j
     real                              :: maxdiff,mindiff
     real                              :: adiff
     integer                           :: ncalc
     logical                           :: drawref, drawcalc, drawdiff
     integer                           :: nset_ref
     integer                           :: ids
     real, dimension(:), allocatable   :: intpeak
     integer                           :: nund
     real, dimension(:), allocatable   :: xund,yund
!
     call set_graphic_area()
!
!    Compute maximum and minimum along y
!corr     maxview = tiny(0.0)
!corr     minview = huge(0.0)
     do ids=1,ndataset(dataset)
!corr        maxview = max(maxview,MAXVAL(dataset(ids)%y(inic(ids):ific(ids))))
!corr        minview = min(minview,MINVAL(dataset(ids)%y(inic(ids):ific(ids))))
        drawcalc = style(STYLE_CALC)%vis == 1 .and. allocated(dataset(ids)%yb)  &
                   .and. allocated(dataset(ids)%yc)
        inic1 = max(dataset(ids)%nc1,inic(ids))
        ific1 = min(dataset(ids)%nc2,ific(ids))
        ncalc = ific1 - inic1 + 1
        if (drawcalc) then
            call new_array(ycalb,ncalc)
            ycalb = dataset(ids)%yc(inic1:ific1)+dataset(ids)%yb(inic1:ific1)
!corr            maxview = max(maxview,MAXVAL(ycalb))
!corr            minview = min(minview,MINVAL(ycalb))
        endif
!corr        if (style(STYLE_BACK)%vis == 1 .and. allocated(dataset(ids)%yb) ) then
!corr            maxview = max(maxview,MAXVAL(dataset(ids)%yb))
!corr            minview = min(minview,MINVAL(dataset(ids)%yb))
!corr        endif
        drawdiff = (style(STYLE_DIFF)%vis == 1 .or. style(STYLE_CUMUL)%vis == 1) &
                    .and. allocated(dataset(ids)%yb) .and. allocated(dataset(ids)%yc)
        if (drawdiff) then
            call new_array(ydiff,ncalc)
            trdiff = -nint(0.07*maxview)  ! traslazione asse differenza
            if (drawcalc) then
                ydiff = dataset(ids)%y(inic1:ific1)-ycalb
            else
                ydiff = dataset(ids)%y(inic1:ific1)-dataset(ids)%yc(inic1:ific1)-dataset(ids)%yb(inic1:ific1)
            endif
!corr            maxview = max(maxview,MAXVAL(ydiff+trdiff))
!corr            minview = min(minview,MINVAL(ydiff+trdiff))
        endif
     enddo
!corr     maxv = maxview
!corr     minv = minview
!corr     if (minv > maxv) then
!corr         minv = maxv - 10
!corr     endif
!corr     diff= maxv - minv
!corr     maxv = maxv !+ trpeak
!
     drawref = .false.
     if (cryst(1)%style%vis == 1) then
         !space_ref = diff*0.04
         nset_ref = count(cryst%numref() > 0)
         if (nset_ref > 0) then
             !minv = minv - nset_ref * space_ref
             drawref = .true.
         endif
     endif
!
!    Disegna il profilo osservato
     do ids=1,ndataset(dataset)
        if (dataset(ids)%style%vis == 1) then
            call add_plot(dataset(ids)%x0(inic(ids):ific(ids)),  &
                 dataset(ids)%y(inic(ids):ific(ids)),ific(ids)-inic(ids)+1,Observed,1,   &
                 dataset(ids)%wave(1),toCString(file_basename(dataset(ids)%fname)))
        endif
     enddo
!
!    Disegna il profilo calcolato
     if (drawcalc) then
         call add_plot(dataset(1)%x0(inic1:ific1),ycalb,ific1-inic1+1,Calculated,1,dataset(1)%wave(1),c_null_char)
     endif
!
!    Disegna il background
     if (style(STYLE_BACK)%vis == 1 .and.      &
         allocated(dataset(1)%yb)   .and.      &
         dataset(1)%cond%btype /= BK_NONE) then
         call add_plot(dataset(1)%x0(inic1:ific1),  &
              dataset(1)%yb(inic1:ific1),ific1-inic1+1,BBackground,1,dataset(1)%wave(1),c_null_char)
         if (style(STYLE_BACKP)%vis == 1 .and. dataset(1)%npoints_back() > 0) then
             call add_plot(dataset(1)%points%x,dataset(1)%points%y,   &
                  dataset(1)%npoints_back(),Background_Points,1,dataset(1)%wave(1),c_null_char)
         endif
     endif
!
!    Disegna differenza
     if (drawdiff) then
         call add_plot(dataset(1)%x0(inic1:ific1),ydiff,ific1-inic1+1,Difference,1,dataset(1)%wave(1),c_null_char)
     endif
!
!    Draw smoothing curve
     if (style(STYLE_SMOOTH)%vis == 1) then
         if (allocated(dataset(1)%smoothvec)) then
             call add_plot(dataset(1)%x0(inic1:ific1),dataset(1)%smoothvec,ific1-inic1+1,Smoothing,1,dataset(1)%wave(1),c_null_char)
         endif
     endif
!
!    calcola e disegna differrenza cumulativa
     if (style(STYLE_CUMUL)%vis == 1) then
         call new_array(diffc,ncalc)
         diffc(1) = abs(ydiff(inic1))
         do i=inic1+1,ific1
            j = i-inic1+1
            diffc(j) = diffc(j-1) + abs(ydiff(i))
         enddo
!
!        riscala diffc tra 0 e il massimo
         maxdiff = diffc(j)
         mindiff = diffc(1)
         adiff = diffc(j) - diffc(1)
         maxv = MAXVAL(ycalb)
         diffc = diffc*(maxv)/adiff - maxv*diffc(1)/adiff
         call add_plot(dataset(1)%x0(inic1:ific1),diffc,ific1-inic1+1,Cumulative,1,dataset(1)%wave(1),c_null_char)
     endif
!
!    Disegna i marcatori di picco 
     if (style(STYLE_PEAKS)%vis == 1) then
         if (numpeaks(pkind) > 0) then
             allocate(intpeak(numpeaks(pkind)))
             do i=1,numpeaks(pkind)
                intpeak(i) = peak_intensity(pkind(i)%getx(),dataset(1)%x0,dataset(1)%y)
             enddo
             call add_plot(pkind%getx(),intpeak,numpeaks(pkind),Peaks,1,dataset(1)%wave(1),c_null_char)

             if (style(STYLE_UND_PEAKS)%vis == 1) then
                 nund = 0
                 do i=1,numpeaks(pkind)
                    if (pkind(i)%info > 0) then
                        if (nund == 0) allocate(xund(pkind(i)%info),yund(pkind(i)%info))
                        nund = nund + 1
                        xund(nund) = pkind(i)%getx()
                        yund(nund) = intpeak(i)
                    endif
                 enddo
                 if (nund > 0) then
                     call add_plot(xund,yund,nund,Unindexed_Peaks,1,dataset(1)%wave(1),c_null_char)
                 endif
             endif
         endif
     endif
!
!    Disegna i riflessi
     if (drawref) then
         do i=1,numphase(cryst)
            if (cryst(i)%numref() > 0) then
                !call add_plot2(cryst(i)%ref%tthd(1),cryst(i)%numref(),Reflections,1,dataset(1)%wave(1),char(0))
                call add_plot2(cryst(i)%ref%tthd(1),cryst(i)%numref(),Reflections,1,dataset(1)%wave(1))
            endif
         enddo
     endif
!
!    kprofile =  1 --> calcola e disegna profili 
     if (kprofile == 1) then
         if (kprofile_type == PROFILE_EXTRA) then
             call profrefnew(dataset(1)%x0(inic(1)),dataset(1)%x0(ific(1)))  
         else
             call prof_curves_computation(dataset(1)%x0(inic(1)),dataset(1)%x0(ific(1)))
         endif
     endif
!
     call draw_graphic()
!
     end subroutine plottanew2

!-----------------------------------------------------------------------

     function get_plot_size(plot_type) bind(C,name="get_plot_size") result(psize)
     use iso_c_binding
     use peak_mod
     use variables, only: dataset
     integer(c_int), intent(in), value :: plot_type
     integer(c_int)                    :: psize
!
     select case (plot_type)
       case (Peaks)
         psize = numpeaks(pkind)

       case (Background_Points)
         psize = dataset(1)%npoints_back()
         
       case default
         psize = 0       
     end select
!
     end function get_plot_size

!-----------------------------------------------------------------------

     subroutine get_plot_xy(x,y,wave,plot_type) bind(C,name="get_plot_xy")
     use iso_c_binding
     use peak_mod
     use variables, only: dataset
     real(c_float), dimension(*) :: x,y
     real(c_float)               :: wave
     integer(c_int), value       :: plot_type
     integer                     :: i
!
     wave = dataset(1)%wave(1)
     select case (plot_type)
       case (Peaks)
         do i=1,numpeaks(pkind)
            x(i) = pkind(i)%getx()
            y(i) = peak_intensity(x(i),dataset(1)%x0,dataset(1)%y)
         enddo

       case (Background_Points)
          do i=1,dataset(1)%npoints_back()
             x(i) = dataset(1)%points(i)%x
             y(i) = dataset(1)%points(i)%y
          enddo

       case default
     end select
!
     end subroutine get_plot_xy

!-----------------------------------------------------------------------
!corr
!corr   subroutine get_unindexed_peaks(xund,yund,nund)
!corr   USE peak_mod
!corr   USE variables, only: dataset
!corr   real, dimension(:), allocatable, intent(out) :: xund,yund
!corr   integer, intent(out)                         :: nund
!corr   integer                                      :: i
!corr   real                                         :: intensity
!corr!
!corr   nund = 0
!corr   do i=1,numpeaks(pkind)
!corr      if (pkind(i)%info > 0) then
!corr          if (nund == 0) allocate(xund(pkind(i)%info),yund(pkind(i)%info))
!corr          nund = nund + 1
!corr          xund(nund) = pkind(i)%getx()
!corr          yund(nund) = peak_intensity(pkind(i)%getx(),dataset(1)%x0,dataset(1)%y)
!corr      endif
!corr   enddo
!corr!
!corr   end subroutine get_unindexed_peaks
!corr
!corr
!corr  !-----------------------------------------------------------------------------------------------
!corr
!corr   subroutine display_unindexed_peaks()
!corr   use variables, only: dataset
!corr   real, dimension(:), allocatable :: xund, yund
!corr   integer                         :: nund
!corr!
!corr   call get_unindexed_peaks(xund, yund, nund)
!corr   if (nund > 0) then
!corr       call add_plot(xund,yund,nund,Unindexed_Peaks,1,dataset(1)%wave(1),char(0))
!corr   endif
!corr!
!corr   end subroutine display_unindexed_peaks

  !-----------------------------------------------------------------------------------------------

     subroutine set_plot_style(kaction)
     USE plotstyle
     USE variables, only: cryst,dataset
     USE crystal_phase
     USE datasetmod
     integer, intent(in) :: kaction
     integer             :: i
!
     select case (kaction)
       case (1)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 0
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
!corr         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 

       case (2)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = -1
         style(STYLE_BACKP)%vis = -1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         !style(STYLE_REFL)%vis = 1
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
!corr         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = 1
         style(STYLE_SMOOTH)%vis = 0 

       case (3)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
!corr         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = 1
         style(STYLE_SMOOTH)%vis = 0 

       case (4)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 0  !!!!!-1
!corr         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 

       case (5)      ! background per peak-search
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 1
         style(STYLE_CALC)%vis = -1
         style(STYLE_DIFF)%vis = -1
         style(STYLE_CUMUL)%vis = -1
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 1
!corr         style(STYLE_UND_PEAKS)%vis = 0  !!!!!-1
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 

       case (6)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 1
!corr         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 
  
       case (7)
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 0 
!corr         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 

       case (8)    ! Rietveld
         !style(STYLE_OBS)%vis = 1
         do i=1,ndataset(dataset)
            dataset(i)%style%vis = 1
         enddo
         style(STYLE_BACK)%vis = 1
         style(STYLE_BACKP)%vis = 0
         style(STYLE_CALC)%vis = 1
         style(STYLE_DIFF)%vis = 1
         style(STYLE_CUMUL)%vis = 0
         !style(STYLE_REFL)%vis = 1
         do i=1,numphase(cryst)
            cryst(i)%style%vis = 1
         enddo
         style(STYLE_PEAKS)%vis = 0 
!corr         style(STYLE_UND_PEAKS)%vis = 0
         style(STYLE_INTERVALS)%vis = -1
         style(STYLE_SMOOTH)%vis = 0 

     end select
!qt     if (style(STYLE_PEAKS)%vis == 1) then
!qt         call wmenu_set_button('Hide Peaks',0)
!qt     else
!qt         call wmenu_set_button('Hide Peaks',1)
!qt     endif
!
     end subroutine set_plot_style

  !-----------------------------------------------------------------------------------------------

   subroutine process_action_points(kaction,xp,yp,ier) bind(C,name="process_action_points")   
   use iso_c_binding
   use peak_mod
   use variables, only: dataset
   use conteggi, only: theta_int
   use arrayutil, only: clocate
   use General, only:alambda, is_extra_active
   use messagemod
   use proginterface, only: fillcounts
   integer(c_int), intent(in), value :: kaction
   real(c_double), intent(in), value :: xp,yp
   integer(c_int), intent(out)       :: ier
   real                              :: thp
   real                              :: xc,yc
   integer                           :: iok
   type(peak_type)                   :: padd
   real                              :: pkx,pky,fwp
   integer                           :: ierfw,ipos

   enum, bind(c)
     enumerator :: AddBackgroundPoint=4, DeleteBackgroundPoint, AddPeak, DeletePeak
   endenum

   ier = 0
   write(0,*)'ACTION:',is_extra_active,xp,yp
   select case (kaction)
     case (AddBackgroundPoint)
        if (is_extra_active) then
            call update_background_extra(1, real(xp), real(yp))
        else
            call update_background(1, real(xp), real(yp))
        endif
!corr        select case (pd_action)
!corr           case (2,3,4)   ! extra
!corr               call update_background_extra(1, real(xp), real(yp))
!corr           case default
!corr               call update_background(1, real(xp), real(yp))
!corr        end select

     case (DeleteBackgroundPoint)
        !write(0,*)'Action: ',kaction,xp,yp
        if (is_extra_active) then
            call update_background_extra(2, real(xp), real(yp))
        else
            call update_background(2, real(xp), real(yp))
        endif
!corr        select case (pd_action)
!corr           case (2,3,4)   ! extra
!corr               call update_background_extra(2, real(xp), real(yp))
!corr           case default
!corr               call update_background(2, real(xp), real(yp))
!corr        end select
     case (AddPeak)
        thp = real(xp)
        call check_peak(pkind,thp,xc,yc,iok) 
        if (iok == 1) then
!
!           Set background is not available
            if (.not.allocated(dataset(1)%yb)) then
                call fillcounts()
                call back_for_peaksearch(.false.)
            endif
!
!           Compute fwhm
            call peak_gaussian_fit(theta_int(:,1),theta_int(:,2),clocate(theta_int(:,1),xc),4,pkx,pky,fwp,ierfw)
            if (ierfw /= 0) fwp = peak_fwhm(pkind,xc)
!
            padd = peak_type(fwhm=fwp)
            call padd%sety(yc)
            call padd%setxd(xc,alambda)
            call padd%get_int(theta_int(:,1),theta_int(:,2),dataset(1)%yb(:))
!
            call padd%integrated_intensity(theta_int(:,1),theta_int(:,2),dataset(1)%yb)
!
!           add peak to list
            call peak_add(pkind,padd,INSERT_PEAK)
            if (numpeaks(pkindtot) > 0) then
                if (.not.is_peak_position(pkindtot,padd%getx()))  &
                    call peak_add(pkindtot,padd,INSERT_PEAK)  ! add to array pkindtot if absent
            else
                call peak_add(pkindtot,padd,INSERT_PEAK)
            endif
!
            style(STYLE_PEAKS)%vis = 1  ! force visualization
            !call vedinew(rescale=0)
            !call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
            call update_peak_graph()
         endif

     case (DeletePeak)
        ipos = clocate(pkind(:)%getx(),real(xp))
        call peak_delete(pkindtot,pkind(ipos)%getx())
        call peak_delete(pkind,ipos)
        !call vedinew(rescale=0)
        !call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
        call update_peak_graph()

   end select

   end subroutine process_action_points

! ----------------------------------------------------------------------

     subroutine check_peak(peaks,tt,xc,yc,iok)
!
     USE Conteggi
     USE arrayutil
     USE peak_util
!
     type(peak_type), dimension(:), allocatable, intent(in) :: peaks
     real, intent(in)     :: tt
     real, intent(out)    :: xc,yc
     integer, intent(out) :: iok
     integer              :: jmin
     integer              :: iniz,ifin
     real                 :: yjmin
     real                 :: yjmaxr,yjmaxl
     integer              :: jmaxr,jmaxl
     integer              :: i
     integer              :: iadd
     logical              :: is_peak
!
     iok = 1
     xc = tt
     iadd = 3
     jmin = clocate(theta_int(:,1),tt)   ! localizza conteggio piu' vicino
     if (numpeaks(peaks) == 0) then
         yc = theta_int(jmin,2)
         return
     endif
!
!    Controlla subito se nella posizione cliccata non esiste gia' un picco
     !is_peak = any(abs(xmax(:npk) - theta_int(jmin,1)) < 1.e-07)
!corr     is_peak = any(abs(peaks%x - theta_int(jmin,1)) < 1.e-07)
     is_peak = is_peak_position(peaks,theta_int(jmin,1))
!
     if (jmin > 0 .and. is_peak .eqv. .false.) then
         iniz = jmin - iadd
         if (iniz.lt.1) iniz = 1
         ifin = jmin + iadd
         if (ifin.gt.npunti) ifin = npunti
         if (iniz.gt.ifin) iniz = ifin
         yjmin = theta_int(jmin,2)
!
!        Cerca il massimo + vicino a destra
         jmaxr = 0
         yjmaxr = yjmin
         do i=jmin+1,ifin
            if (theta_int(i,2) > yjmaxr) then
                yjmaxr = theta_int(i,2)
                jmaxr = i
            elseif (theta_int(i,2) < yjmaxr .and. jmaxr > 0) then
                exit
            endif
         enddo
         if (jmaxr == ifin) jmaxr = 0  ! il massimo non puo' coincidere col limite
         if (jmaxr > 0) then
         !if (any(abs(xmax(:npk) - theta_int(jmaxr,1)) < 1.e-07))jmaxr=-1  ! annulla il punto se esiste gia'
!corr         if (any(abs(peaks%x - theta_int(jmaxr,1)) < 1.e-07))jmaxr=-1  ! annulla il punto se esiste gia'
         if(is_peak_position(peaks,theta_int(jmaxr,1)))jmaxr=-1   ! annulla il punto se esiste gia'
         !if (jmaxr > 0)write(0,*)'massimo a destra=',jmaxr,theta_int(jmaxr,1),yjmaxr
         endif
!
!        Cerca il massimo + vicino a sinistra
         jmaxl = 0
         yjmaxl = yjmin
         do i=jmin-1,iniz,-1
            if (theta_int(i,2) > yjmaxl) then
                yjmaxl = theta_int(i,2)
                jmaxl = i
            elseif (theta_int(i,2) < yjmaxl .and. jmaxl > 0) then
                exit
            endif
         enddo
         if (jmaxl == iniz) jmaxl = 0  ! il massimo non puo' coincidere col limite
         if (jmaxl > 0) then
         !if (any(abs(peaks%x - theta_int(jmaxl,1)) < 1.e-07))jmaxl=-1  ! annulla il punto se esiste gia'
         if(is_peak_position(peaks,theta_int(jmaxl,1)))jmaxl=-1  ! annulla il punto se esiste gia'
         !if (any(abs(xmax(:npk) - theta_int(jmaxl,1)) < 1.e-07))jmaxl=-1  ! annulla il punto se esiste gia'
         !if (jmaxl > 0)write(0,*)'massimo a sinistra=',jmaxl,theta_int(jmaxl,1),yjmaxl
         endif
!
         if ((jmaxl + jmaxr == 0) .or. (jmaxl + jmaxr == -1)) then       ! non trova nessun massimo
             yc = yjmin                              ! accetta selezione dell'utente
             xc = theta_int(jmin,1)
         elseif (jmaxl > 0 .and. jmaxr <= 0) then    ! massimo a sinistra
             yc = yjmaxl
             xc = theta_int(jmaxl,1)
         elseif (jmaxr > 0 .and. jmaxl <= 0) then    ! massimo a destra
             yc = yjmaxr
             xc = theta_int(jmaxr,1)
         elseif (jmaxr > 0 .and. jmaxl > 0) then     ! massimo a destra e sinistra: prendi quello pi� vicino
             if (jmaxr - jmin  > jmin - jmaxl) then
                 yc = yjmaxl               ! a sinistra e' piu' vicino
                 xc = theta_int(jmaxl,1)
             else
                 yc = yjmaxr               ! piu' vicino a destra o uguali
                 xc = theta_int(jmaxr,1)
             endif
         elseif (jmaxr < 0 .and. jmaxl < 0) then     ! il picco trovato esiste gia'
             !write(0,*)"picco annullato perche' esiste gia'"
             iok = 0
         endif
         !write(0,*)'scelgo massimo=',yc,xc,jmaxl,jmaxr,jmin
      else
         iok = 0
      endif

      end subroutine check_peak

  !-----------------------------------------------------------------------------------------------
                                                                                                                  
     subroutine update_background_extra(paction,xp,yp)
!qt     USE Molcom
     USE PatternRef, only:jjj
     USE Commonexpo, only:nnt,ntyp,ncoun1,ncoun2,nback,backp
     USE conteggi
     USE arrayutil, only: clocate
     integer, intent(in) :: paction
     real, intent(in)    :: xp,yp
!qt     integer             :: iix2,iiy2
!qt     integer             :: ira
     integer             :: i,j
     integer             :: jjjsav
     integer             :: ipos
!
     jjjsav = jjj  
     if (paction == 1) then      ! aggiungo punti col tasto sinistro
         !ier = 0
         nback=nback+1
         backp(nback,1) = xp
         backp(nback,2) = yp
         !backp(nback,1) = (xpixel-dispx(iselview)) / scalex(iselview)
         !backp(nback,2) = (ypixel-dispy(iselview)) / scaley(iselview)
!
!        localizza l'intervallo jjj a cui appartiene il punto aggiunto
         do i=2,nnt-1
            if (backp(nback,1) >= theta_int(ncoun1(i),7) .and.    &
                backp(nback,1) <= theta_int(ncoun2(i),7)) then
                jjj = i
            endif
         enddo
         ntyp(nback)=jjj
         call CalcoBacExt(1)

     elseif (paction == 2) then  ! elimino punti col tasto destro
         !ira=5
         !ier = -1
         !do i=1, nback
         !   iix2 = nint(backp(i,1)*scalex(iselview) + dispx(iselview))
         !   iiy2 = nint(backp(i,2)*scaley(iselview) + dispy(iselview))
         !   if(iabs(xpixel-iix2) < ira .and. iabs(ypixel-iiy2) < ira) then
         !      ier = 0
         !      backp(i:nback-1,:) = backp(i+1:,:)
         !      ntyp(i:nback-1) = ntyp(i+1:)
         !      nback = nback - 1
!
!        !      localizza l'intervallo jjj a cui appartiene il punto da eliminare
         !      do j=2,nnt-1
         !         if (backp(i,1) >= theta_int(ncoun1(j),7) .and.    &
         !             backp(i,1) <= theta_int(ncoun2(j),7)) then
         !             jjj = j
         !         endif
         !      enddo
         !      exit
         !   end if
         !enddo
!        locate and remove point
         ipos = clocate(backp(:nback,1),xp)
         write(0,*)'DELETE: ',xp,yp,ipos,nback
         backp(ipos:nback-1,:) = backp(ipos+1:nback,:)
         ntyp(ipos:nback-1) = ntyp(ipos+1:nback)
         nback = nback - 1
!        localizza l'intervallo jjj a cui appartiene il punto da eliminare
         do j=2,nnt-1
            if (backp(ipos,1) >= theta_int(ncoun1(j),7) .and.    &
                backp(ipos,1) <= theta_int(ncoun2(j),7)) then
                jjj = j
            endif
         enddo
         call CalcoBacExt(1)
     endif
     jjj = jjjsav
     call vedinew(rescale=0)
!
     end subroutine update_background_extra

  !-----------------------------------------------------------------------------------------------
!qt
!qt     subroutine reset_selection()
!qt!
!qt     nselection = 0
!qt!
!qt     end subroutine reset_selection
!qt
  !-----------------------------------------------------------------------------------------------

!qt     subroutine add_to_selection(vet,vph)
!qt     use arrayutil
!qt     integer, dimension(:), intent(in) :: vet,vph
!qt!corr     integer                           :: nsel
!qt!corr     integer                           :: i
!qt!corr     integer, parameter :: NPLUS=3000
!qt!
!qt!corr     if (nselection+size(vet) > size_array(vselection)) then
!qt!corr         call resize_array(vselection,size_array(vselection)+NPLUS)
!qt!corr     endif
!qt!corr     vselection(nselection+1:nselection+size(vet)) = vet(:)
!qt     call insert_array(vselection,vet,nselection)
!qt     call insert_array(phselection,vph,nselection)
!qt     nselection = nselection + size(vet)
!qt!corr     nsel = nselection
!qt!corr     do i=1,size(vet)
!qt!corr        nsel = nsel + i
!qt!corr        !if (nsel > NMAXSEL) nsel = 1
!qt!corr        vselection(nsel) = vet(i)
!qt!corr     enddo
!qt!corr     nselection = nsel
!qt!
!qt     end subroutine add_to_selection
!qt
  !-----------------------------------------------------------------------------------------------

!qt     subroutine draw_selection(tthr,lungr,yini,minref,maxref,nph)
!qt     use grexpomod
!qt     real, dimension(:), intent(in) :: tthr
!qt     real, intent(in)               :: yini,lungr
!qt     integer, intent(in)            :: minref,maxref,nph
!qt     integer                        :: jsel,i
!qt!
!qt     if (nselection > 0) then
!qt         !yyini = yini !+ lungr + 0.20*lungr
!qt         call gr_setsize(6)
!qt         do i=1,nselection
!qt            if (phselection(i) == nph) then
!qt                jsel = vselection(i)
!qt                if (jsel >= minref .and. jsel <= maxref) then
!qt                    call gr_setcolor(1-style(STYLE_BG)%colline)
!qt                    call gr_draw1marker(tthr(jsel),yini+lungr/2,1)
!qt                    call gr_draw1marker(tthr(jsel),yini+lungr/2,3)
!qt                endif
!qt            endif
!qt         enddo
!qt     endif
!qt!
!qt     end subroutine draw_selection

!----------------------------------------------------------------------------------------------------

   subroutine set_reflection_table(table,vet,ftype,ier,id_phase)
!
!  Usata per generare una tabella di informazioni sui riflessi
!
   USE General, only:alambda,neutro,is_anomalous
   USE variables, only:cryst,dataset
   USE counts
   USE prog_constants
   USE reflection_type_util
   use crystal_phase
   use datasetmod
   use cryutil
   use arrayutil
   type(reflection_type), intent(inout), dimension(:), allocatable :: table
   integer, parameter                                              :: NCOL = 9
   integer, dimension(NCOL)                                        :: vet
   integer, intent(in)                                             :: ftype
   integer, dimension(:), allocatable, optional                    :: id_phase
   integer, intent(out)                                            :: ier
   integer                                                         :: nref,nph
   real                                                            :: thmin,thmax
   integer, dimension(:), allocatable                              :: ord,id_ph
!
   vet(:) = 0
   ier = 0
!
   if (alambda == 0.0) alambda = CU_WAVE
   do nph=1,numphase(cryst)
      if (cryst(nph)%numref() == 0) then 
          if (ndataset(dataset) == 0) then
              thmin = 1.0; thmax = 100.
          else
              thmin = dataset(1)%tmin; thmax = dataset(1)%tmax
          endif
          call cryst(nph)%make_reflections(thmin,thmax,[alambda])
      endif
      if (cryst(nph)%numref() > 0 .and. cryst(nph)%natoms() > 0) then
          call fcalcang(cryst(nph)%ref,cryst(nph)%at,cryst(nph)%spg,cryst(nph)%elem,neutro,is_anomalous)
          vet(7:8) = 1
      endif
   enddo
   call combine_reflections(cryst,nref,table,id_ph)
   if (nref > 0) then
       if (numphase(cryst) > 1) then
           call new_array(ord,nref)
           call sort_reflections(table,ORD_BY_2THETA,ord=ord)
           if (present(id_phase)) then
               id_ph = id_ph(ord)
               call copy_array(id_phase,id_ph)
           endif
       else
           if (present(id_phase)) then
               call new_array(id_phase,nref,1)
           endif
       endif
       vet(1:6) = 1     
       if (numphase(cryst) == 1) vet(2) = 0
       vet(9) = 1   
       table(:)%slaq = dvalue(table(:)%tthd(1),alambda) ! col. 4: d
       if (ftype == 2) then
           table(:)%fo = table(:)%fo**2          ! col. 5: Fo^2
           table(:)%fc = table(:)%fc**2          ! col. 5: Fo^2
       endif
   else
       ier = 1
   endif
!
   end subroutine set_reflection_table

!----------------------------------------------------------------------------------------------------

   subroutine get_reflection_info(phaseIndex,refIndex,hkl,tth,dval,err) bind(C,name="get_reflection_info")
   use variables, only: cryst
   use crystal_phase
   use counts
   use general, only: wavel
   use iso_c_binding
   integer(c_int), value, intent(in)         :: phaseIndex, refIndex
   integer(c_int), dimension(3), intent(out) :: hkl
   real(c_float), intent(out)                :: tth, dval
   integer(c_int), intent(out)               :: err
   err = 1
   if (numphase(cryst) < phaseIndex) return
   if (cryst(phaseIndex)%numref() < refIndex) return
   hkl = cryst(phaseIndex)%ref(refIndex)%hkl
   tth = cryst(phaseIndex)%ref(refIndex)%tthd(1)
   dval = dvalue(tth,wavel)
   err = 0
   end subroutine get_reflection_info

!----------------------------------------------------------------------------------------------------

   subroutine prof_curves_computation(tmin, tmax)
   use variables, only: dataset,cryst
   use profile_function
   use crystal_phase
   use pearsonf
   use psvoigtf
   use tchzf
   use plot2, only: add_plot
   use iso_c_binding, only: c_null_char
   use refinecomref, only: rfcond
   use calculate_pdp, only: RIET_REFINE
   use arrayutil, only: clocate
   real, intent(in)                :: tmin, tmax
   integer                         :: nph, jref, startr, endr
   real, dimension(:), allocatable :: xp,yp
   integer                         :: ier

   do nph=1,numphase(cryst)
      startr = clocate(cryst(nph)%ref%tthd(1),tmin)
      endr = clocate(cryst(nph)%ref%tthd(1),tmax)
      select case (dataset(1)%typefun(nph))
        case (PEARSON7)
              do jref=startr,endr
                 call profile_curve_pearson(dataset(1)%pear(nph),cryst(nph)%ref(jref),         &
                                            jref,rfcond%raction == RIET_REFINE,                &
                                            dataset(1)%x0,dataset(1)%yb,cryst(nph)%scal,       &
                                            dataset(1)%nwave,dataset(1)%ratio,nph,xp,yp,ier)
                 if (ier == 0) then
                     call add_plot(xp,yp,size(xp),Profile_Curves,1,dataset(1)%wave(1),c_null_char)
                 endif
              enddo
        case (PVOIG)
              do jref=startr,endr
                 call profile_curve_psvoigt(dataset(1)%pear(nph),cryst(nph)%ref(jref),         &
                                            jref,rfcond%raction == RIET_REFINE,                &
                                            dataset(1)%x0,dataset(1)%yb,cryst(nph)%scal,       &
                                            dataset(1)%nwave,dataset(1)%ratio,nph,xp,yp,ier)
                 if (ier == 0) then
                     call add_plot(xp,yp,size(xp),Profile_Curves,1,dataset(1)%wave(1),c_null_char)
                 endif
              enddo

        case (TCHZ)
              do jref=startr,endr
                 call profile_curve_tchz(dataset(1)%pear(nph),cryst(nph)%ref(jref),         &
                                         jref,rfcond%raction == RIET_REFINE,                &
                                         dataset(1)%x0,dataset(1)%yb,cryst(nph)%scal,       &
                                         dataset(1)%nwave,dataset(1)%ratio,nph,xp,yp,ier)
                 if (ier == 0) then
                     call add_plot(xp,yp,size(xp),Profile_Curves,1,dataset(1)%wave(1),c_null_char)
                 endif
              enddo

      end select
   enddo

   end subroutine prof_curves_computation

   END MODULE view
